@@ -1,61 +1,18 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the qmake application of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "msvc_nmake.h"
 #include "option.h"
 
-#include <qregexp.h>
+#include <qregularexpression.h>
 #include <qdir.h>
-#include <qdiriterator.h>
+#include <qdirlisting.h>
 #include <qset.h>
 
-#include <registry_p.h>
-
 #include <time.h>
 
 QT_BEGIN_NAMESPACE
 
-static QString nmakePathList(const QStringList &list)
-{
-    QStringList pathList;
-    pathList.reserve(list.size());
-    for (const QString &path : list)
-        pathList.append(QDir::cleanPath(path));
-
-    return QDir::toNativeSeparators(pathList.join(QLatin1Char(';')))
-            .replace('#', QLatin1String("^#")).replace('$', QLatin1String("$$"));
-}
-
-NmakeMakefileGenerator::NmakeMakefileGenerator() : usePCH(false), usePCHC(false)
-{
-
-}
-
 bool
 NmakeMakefileGenerator::writeMakefile(QTextStream &t)
 {
@@ -66,184 +23,6 @@ NmakeMakefileGenerator::writeMakefile(QTextStream &t)
     if(project->first("TEMPLATE") == "app" ||
        project->first("TEMPLATE") == "lib" ||
        project->first("TEMPLATE") == "aux") {
-#if 0
-        if(Option::mkfile::do_stub_makefile)
-            return MakefileGenerator::writeStubMakefile(t);
-#endif
-        if (!project->isHostBuild()) {
-            if (project->isActiveConfig(QStringLiteral("winrt"))) {
-                QString arch = project->first("VCPROJ_ARCH").toQString().toLower();
-                QString compiler;
-                QString compilerArch;
-                const QString msvcVer = project->first("MSVC_VER").toQString();
-                if (msvcVer.isEmpty()) {
-                    fprintf(stderr, "Mkspec does not specify MSVC_VER. Cannot continue.\n");
-                    return false;
-                }
-
-                if (msvcVer == QStringLiteral("15.0")) {
-                    const ProStringList hostArch = project->values("QMAKE_TARGET.arch");
-                    if (hostArch.contains("x86_64"))
-                        compiler = QStringLiteral("HostX64/");
-                    else
-                        compiler = QStringLiteral("HostX86/");
-                    if (arch == QLatin1String("arm")) {
-                        compiler += QStringLiteral("arm");
-                        compilerArch = QStringLiteral("arm");
-                    } else if (arch == QLatin1String("x64")) {
-                        compiler += QStringLiteral("x64");
-                        compilerArch = QStringLiteral("amd64");
-                    } else {
-                        arch = QStringLiteral("x86");
-                        compiler += QStringLiteral("x86");
-                    }
-                } else {
-                    if (arch == QLatin1String("arm")) {
-                        compiler = QStringLiteral("x86_arm");
-                        compilerArch = QStringLiteral("arm");
-                    } else if (arch == QLatin1String("x64")) {
-                        const ProStringList hostArch = project->values("QMAKE_TARGET.arch");
-                        if (hostArch.contains("x86_64"))
-                            compiler = QStringLiteral("amd64");
-                        else
-                            compiler = QStringLiteral("x86_amd64");
-                        compilerArch = QStringLiteral("amd64");
-                    } else {
-                        arch = QStringLiteral("x86");
-                    }
-                }
-
-                const QString winsdkVer = project->first("WINSDK_VER").toQString();
-                if (winsdkVer.isEmpty()) {
-                    fprintf(stderr, "Mkspec does not specify WINSDK_VER. Cannot continue.\n");
-                    return false;
-                }
-                const QString targetVer = project->first("WINTARGET_VER").toQString();
-                if (targetVer.isEmpty()) {
-                    fprintf(stderr, "Mkspec does not specify WINTARGET_VER. Cannot continue.\n");
-                    return false;
-                }
-
-#ifdef Q_OS_WIN
-                QString regKey;
-                if (msvcVer == QStringLiteral("15.0"))
-                    regKey = QStringLiteral("Software\\Microsoft\\VisualStudio\\SxS\\VS7\\") + msvcVer;
-                else
-                    regKey = QStringLiteral("Software\\Microsoft\\VisualStudio\\") + msvcVer + ("\\Setup\\VC\\ProductDir");
-                const QString vcInstallDir = qt_readRegistryKey(HKEY_LOCAL_MACHINE, regKey, KEY_WOW64_32KEY);
-                if (vcInstallDir.isEmpty()) {
-                    fprintf(stderr, "Failed to find the Visual Studio installation directory.\n");
-                    return false;
-                }
-
-                const QString windowsPath = "Software\\Microsoft\\Microsoft SDKs\\Windows\\v";
-
-                regKey = windowsPath + winsdkVer + QStringLiteral("\\InstallationFolder");
-                const QString kitDir = qt_readRegistryKey(HKEY_LOCAL_MACHINE, regKey, KEY_WOW64_32KEY);
-                if (kitDir.isEmpty()) {
-                    fprintf(stderr, "Failed to find the Windows Kit installation directory.\n");
-                    return false;
-                }
-#else
-                const QString vcInstallDir = "/fake/vc_install_dir";
-                const QString kitDir = "/fake/sdk_install_dir";
-#endif // Q_OS_WIN
-                QStringList incDirs;
-                QStringList libDirs;
-                QStringList binDirs;
-                if (msvcVer == QStringLiteral("15.0")) {
-                    const QString toolsInstallDir = qgetenv("VCToolsInstallDir");
-                    if (toolsInstallDir.isEmpty()) {
-                        fprintf(stderr, "Failed to access tools installation dir.\n");
-                        return false;
-                    }
-
-                    binDirs << toolsInstallDir + QStringLiteral("bin/") + compiler;
-                    if (arch == QStringLiteral("x64"))
-                        binDirs << toolsInstallDir + QStringLiteral("bin/HostX86/X86");
-                    binDirs << kitDir + QStringLiteral("bin/x86");
-                    binDirs << vcInstallDir + QStringLiteral("Common7/Tools");
-                    binDirs << vcInstallDir + QStringLiteral("Common7/ide");
-                    binDirs << vcInstallDir + QStringLiteral("MSBuild/15.0/bin");
-
-                    incDirs << toolsInstallDir + QStringLiteral("include");
-                    incDirs << vcInstallDir + QStringLiteral("VC/Auxiliary/VS/include");
-
-                    const QString crtVersion = qgetenv("UCRTVersion");
-                    if (crtVersion.isEmpty()) {
-                        fprintf(stderr, "Failed to access CRT version.\n");
-                        return false;
-                    }
-                    const QString crtInclude = kitDir + QStringLiteral("Include/") + crtVersion;
-                    const QString crtLib = kitDir + QStringLiteral("Lib/") + crtVersion;
-                    incDirs << crtInclude + QStringLiteral("/ucrt");
-                    incDirs << crtInclude + QStringLiteral("/um");
-                    incDirs << crtInclude + QStringLiteral("/shared");
-                    incDirs << crtInclude + QStringLiteral("/winrt");
-
-                    incDirs << kitDir + QStringLiteral("Extension SDKs/WindowsMobile/")
-                                      + crtVersion + QStringLiteral("/Include/WinRT");
-
-                    libDirs << toolsInstallDir + QStringLiteral("lib/") + arch + QStringLiteral("/store");
-
-                    libDirs << vcInstallDir + QStringLiteral("VC/Auxiliary/VS/lib/") + arch;
-
-                    libDirs << crtLib + QStringLiteral("/ucrt/") + arch;
-                    libDirs << crtLib + QStringLiteral("/um/") + arch;
-                } else if (msvcVer == QStringLiteral("14.0")) {
-                    binDirs << vcInstallDir + QStringLiteral("bin/") + compiler;
-                    binDirs << vcInstallDir + QStringLiteral("bin/"); // Maybe remove for x86 again?
-                    binDirs << kitDir + QStringLiteral("bin/") + (arch == QStringLiteral("arm") ? QStringLiteral("x86") : arch);
-                    binDirs << vcInstallDir + QStringLiteral("../Common7/Tools/bin");
-                    binDirs << vcInstallDir + QStringLiteral("../Common7/Tools");
-                    binDirs << vcInstallDir + QStringLiteral("../Common7/ide");
-                    binDirs << kitDir + QStringLiteral("Windows Performance Toolkit/");
-
-                    incDirs << vcInstallDir + QStringLiteral("include");
-                    incDirs << vcInstallDir + QStringLiteral("atlmfc/include");
-
-                    const QString crtVersion = qgetenv("UCRTVersion");
-                    if (crtVersion.isEmpty()) {
-                        fprintf(stderr, "Failed to access CRT version.\n");
-                        return false;
-                    }
-                    const QString crtInclude = kitDir + QStringLiteral("Include/") + crtVersion;
-                    const QString crtLib = kitDir + QStringLiteral("Lib/") + crtVersion;
-                    incDirs << crtInclude + QStringLiteral("/ucrt");
-                    incDirs << crtInclude + QStringLiteral("/um");
-                    incDirs << crtInclude + QStringLiteral("/shared");
-                    incDirs << crtInclude + QStringLiteral("/winrt");
-
-                    incDirs << kitDir + QStringLiteral("Extension SDKs/WindowsMobile/")
-                                      + crtVersion + QStringLiteral("/Include/WinRT");
-
-                    libDirs << vcInstallDir + QStringLiteral("lib/store/") + compilerArch;
-                    libDirs << vcInstallDir + QStringLiteral("atlmfc/lib") + compilerArch;
-
-                    libDirs << crtLib + QStringLiteral("/ucrt/") + arch;
-                    libDirs << crtLib + QStringLiteral("/um/") + arch;
-                } else {
-                    incDirs << vcInstallDir + QStringLiteral("/include");
-                    libDirs << vcInstallDir + QStringLiteral("/lib/store/") + compilerArch
-                            << vcInstallDir + QStringLiteral("/lib/") + compilerArch;
-                    binDirs << vcInstallDir + QStringLiteral("/bin/") + compiler
-                            << vcInstallDir + QStringLiteral("/../Common7/IDE");
-                    libDirs << kitDir + QStringLiteral("/Lib/") + targetVer + ("/um/") + arch;
-                    incDirs << kitDir + QStringLiteral("/include/um")
-                            << kitDir + QStringLiteral("/include/shared")
-                            << kitDir + QStringLiteral("/include/winrt");
-                }
-
-                binDirs << vcInstallDir + QStringLiteral("/bin");
-
-                // Inherit PATH
-                binDirs << QString::fromLocal8Bit(qgetenv("PATH")).split(QLatin1Char(';'));
-
-                t << "\nINCLUDE = " << nmakePathList(incDirs);
-                t << "\nLIB = " << nmakePathList(libDirs);
-                t << "\nPATH = " << nmakePathList(binDirs) << '\n';
-            }
-        }
         writeNmakeParts(t);
         return MakefileGenerator::writeMakefile(t);
     }
@@ -263,6 +42,11 @@ void NmakeMakefileGenerator::writeSubMakeCall(QTextStream &t, const QString &cal
     Win32MakefileGenerator::writeSubMakeCall(t, callPrefix, makeArguments);
 }
 
+ProStringList NmakeMakefileGenerator::extraSubTargetDependencies()
+{
+    return { "$(MAKEFILE)" };
+}
+
 QString NmakeMakefileGenerator::defaultInstall(const QString &t)
 {
     QString ret = Win32MakefileGenerator::defaultInstall(t);
@@ -277,7 +61,9 @@ QString NmakeMakefileGenerator::defaultInstall(const QString &t)
 
     if (project->isActiveConfig("debug_info")) {
         if (t == "dlltarget" || project->values(ProKey(t + ".CONFIG")).indexOf("no_dll") == -1) {
-            QString pdb_target = project->first("TARGET") + project->first("TARGET_VERSION_EXT") + ".pdb";
+            const QFileInfo targetFileInfo(project->first("DESTDIR") + project->first("TARGET")
+                    + project->first("TARGET_EXT"));
+            const QString pdb_target = targetFileInfo.completeBaseName() + ".pdb";
             QString src_targ = (project->isEmpty("DESTDIR") ? QString("$(DESTDIR)") : project->first("DESTDIR")) + pdb_target;
             QString dst_targ = filePrefixRoot(root, fileFixify(targetdir + pdb_target, FileFixifyAbsolute));
             if(!ret.isEmpty())
@@ -321,17 +107,17 @@ void NmakeMakefileGenerator::writeNmakeParts(QTextStream &t)
         QString precompRule = QString("-c -Yc -Fp%1 -Fo%2")
                 .arg(escapeFilePath(precompPch), escapeFilePath(precompObj));
         t << escapeDependencyPath(precompObj) << ": " << escapeDependencyPath(precompH) << ' '
-          << escapeDependencyPaths(findDependencies(precompH)).join(" \\\n\t\t")
+          << finalizeDependencyPaths(findDependencies(precompH)).join(" \\\n\t\t")
           << "\n\t$(CXX) " + precompRule +" $(CXXFLAGS) $(INCPATH) -TP "
-          << escapeFilePath(precompH) << endl << endl;
+          << escapeFilePath(precompH) << Qt::endl << Qt::endl;
     }
     if (usePCHC) {
         QString precompRuleC = QString("-c -Yc -Fp%1 -Fo%2")
                 .arg(escapeFilePath(precompPchC), escapeFilePath(precompObjC));
         t << escapeDependencyPath(precompObjC) << ": " << escapeDependencyPath(precompH) << ' '
-          << escapeDependencyPaths(findDependencies(precompH)).join(" \\\n\t\t")
+          << finalizeDependencyPaths(findDependencies(precompH)).join(" \\\n\t\t")
           << "\n\t$(CC) " + precompRuleC +" $(CFLAGS) $(INCPATH) -TC "
-          << escapeFilePath(precompH) << endl << endl;
+          << escapeFilePath(precompH) << Qt::endl << Qt::endl;
     }
 }
 
@@ -345,21 +131,17 @@ QString NmakeMakefileGenerator::var(const ProKey &value) const
                                || value == "QMAKE_RUN_CXX_IMP"
                                || value == "QMAKE_RUN_CXX");
         if ((isRunCpp && usePCH) || (isRunC && usePCHC)) {
-            QFileInfo precompHInfo(fileInfo(precompH));
-            QString precompH_f = escapeFilePath(precompHInfo.fileName());
+            QString precompH_f = escapeFilePath(fileFixify(precompH, FileFixifyBackwards));
             QString precompRule = QString("-c -FI%1 -Yu%2 -Fp%3")
                     .arg(precompH_f, precompH_f, escapeFilePath(isRunC ? precompPchC : precompPch));
+            // ### For clang_cl 8 we force inline methods to be compiled here instead
+            // linking them from a pch.o file. We do this by pretending we are also doing
+            // the pch.o generation step.
+            if (project->isActiveConfig("clang_cl"))
+                precompRule += QString(" -Xclang -building-pch-with-obj");
             QString p = MakefileGenerator::var(value);
             p.replace(QLatin1String("-c"), precompRule);
-            // Cannot use -Gm with -FI & -Yu, as this gives an
-            // internal compiler error, on the newer compilers
-            // ### work-around for a VS 2003 bug. Move to some prf file or remove completely.
-            p.remove("-Gm");
             return p;
-        } else if (value == "QMAKE_CXXFLAGS") {
-            // Remove internal compiler error option
-            // ### work-around for a VS 2003 bug. Move to some prf file or remove completely.
-            return MakefileGenerator::var(value).remove("-Gm");
         }
     }
 
@@ -367,6 +149,10 @@ QString NmakeMakefileGenerator::var(const ProKey &value) const
     return MakefileGenerator::var(value);
 }
 
+void NmakeMakefileGenerator::suppressBuiltinRules(QTextStream &) const
+{
+}
+
 void NmakeMakefileGenerator::init()
 {
     /* this should probably not be here, but I'm using it to wrap the .t files */
@@ -383,7 +169,7 @@ void NmakeMakefileGenerator::init()
 
     processVars();
 
-    project->values("QMAKE_LIBS") += project->values("RES_FILE");
+    project->values("LIBS") += project->values("RES_FILE");
 
     if (!project->values("DEF_FILE").isEmpty()) {
         QString defFileName = fileFixify(project->first("DEF_FILE").toQString());
@@ -418,7 +204,10 @@ void NmakeMakefileGenerator::init()
         precompObj = var("PRECOMPILED_DIR") + project->first("TARGET") + "_pch" + Option::obj_ext;
         precompPch = var("PRECOMPILED_DIR") + project->first("TARGET") + "_pch.pch";
         // Add linking of precompObj (required for whole precompiled classes)
-        project->values("OBJECTS") += precompObj;
+        // ### For clang_cl we currently let inline methods be generated in the normal objects,
+        // since the PCH object is buggy (as of clang 8.0.0)
+        if (!project->isActiveConfig("clang_cl"))
+            project->values("OBJECTS") += precompObj;
         // Add pch file to cleanup
         project->values("QMAKE_CLEAN") += precompPch;
         // Return to variable pool
@@ -428,21 +217,23 @@ void NmakeMakefileGenerator::init()
     if (usePCHC) {
         precompObjC = var("PRECOMPILED_DIR") + project->first("TARGET") + "_pch_c" + Option::obj_ext;
         precompPchC = var("PRECOMPILED_DIR") + project->first("TARGET") + "_pch_c.pch";
-        project->values("OBJECTS") += precompObjC;
+        if (!project->isActiveConfig("clang_cl"))
+            project->values("OBJECTS") += precompObjC;
         project->values("QMAKE_CLEAN") += precompPchC;
         project->values("PRECOMPILED_OBJECT_C") = ProStringList(precompObjC);
         project->values("PRECOMPILED_PCH_C")    = ProStringList(precompPchC);
     }
 
-    ProString tgt = project->first("DESTDIR")
-                    + project->first("TARGET") + project->first("TARGET_VERSION_EXT");
-    if(project->isActiveConfig("shared")) {
-        project->values("QMAKE_CLEAN").append(tgt + ".exp");
-        project->values("QMAKE_DISTCLEAN").append(tgt + ".lib");
+    const QFileInfo targetFileInfo(project->first("DESTDIR") + project->first("TARGET")
+            + project->first("TARGET_EXT"));
+    const ProString targetBase = targetFileInfo.path() + '/' + targetFileInfo.completeBaseName();
+    if (project->first("TEMPLATE") == "lib" && project->isActiveConfig("shared")) {
+        project->values("QMAKE_CLEAN").append(targetBase + ".exp");
+        project->values("QMAKE_DISTCLEAN").append(targetBase + ".lib");
     }
     if (project->isActiveConfig("debug_info")) {
         QString pdbfile;
-        QString distPdbFile = tgt + ".pdb";
+        QString distPdbFile = targetBase + ".pdb";
         if (project->isActiveConfig("staticlib")) {
             // For static libraries, the compiler's pdb file and the dist pdb file are the same.
             pdbfile = distPdbFile;
@@ -458,12 +249,8 @@ void NmakeMakefileGenerator::init()
         project->values("QMAKE_DISTCLEAN").append(distPdbFile);
     }
     if (project->isActiveConfig("debug")) {
-        project->values("QMAKE_CLEAN").append(tgt + ".ilk");
-        project->values("QMAKE_CLEAN").append(tgt + ".idb");
-    } else {
-        ProStringList &defines = project->values("DEFINES");
-        if (!defines.contains("NDEBUG"))
-            defines.append("NDEBUG");
+        project->values("QMAKE_CLEAN").append(targetBase + ".ilk");
+        project->values("QMAKE_CLEAN").append(targetBase + ".idb");
     }
 
     if (project->values("QMAKE_APP_FLAG").isEmpty() && project->isActiveConfig("dll")) {
@@ -477,9 +264,9 @@ QStringList NmakeMakefileGenerator::sourceFilesForImplicitRulesFilter()
 {
     QStringList filter;
     const QChar wildcard = QLatin1Char('*');
-    for (const QString &ext : qAsConst(Option::c_ext))
+    for (const QString &ext : std::as_const(Option::c_ext))
         filter << wildcard + ext;
-    for (const QString &ext : qAsConst(Option::cpp_ext))
+    for (const QString &ext : std::as_const(Option::cpp_ext))
         filter << wildcard + ext;
     return filter;
 }
@@ -493,21 +280,21 @@ void NmakeMakefileGenerator::writeImplicitRulesPart(QTextStream &t)
         t << " " << (*cit);
     for(QStringList::Iterator cppit = Option::cpp_ext.begin(); cppit != Option::cpp_ext.end(); ++cppit)
         t << " " << (*cppit);
-    t << endl << endl;
+    t << Qt::endl << Qt::endl;
 
     bool useInferenceRules = !project->isActiveConfig("no_batch");
     QSet<QString> source_directories;
     if (useInferenceRules) {
         source_directories.insert(".");
-        static const char * const directories[] = { "UI_SOURCES_DIR", "UI_DIR", 0 };
+        static const char * const directories[] = { "UI_SOURCES_DIR", "UI_DIR", nullptr };
         for (int y = 0; directories[y]; y++) {
             QString dirTemp = project->first(directories[y]).toQString();
             if (dirTemp.endsWith("\\"))
-                dirTemp.truncate(dirTemp.length()-1);
+                dirTemp.truncate(dirTemp.size()-1);
             if(!dirTemp.isEmpty())
                 source_directories.insert(dirTemp);
         }
-        static const char * const srcs[] = { "SOURCES", "GENERATED_SOURCES", 0 };
+        static const char * const srcs[] = { "SOURCES", "GENERATED_SOURCES", nullptr };
         for (int x = 0; srcs[x]; x++) {
             const ProStringList &l = project->values(srcs[x]);
             for (ProStringList::ConstIterator sit = l.begin(); sit != l.end(); ++sit) {
@@ -525,18 +312,17 @@ void NmakeMakefileGenerator::writeImplicitRulesPart(QTextStream &t)
         QHash<QString, QString> fileNames;
         bool duplicatesFound = false;
         const QStringList sourceFilesFilter = sourceFilesForImplicitRulesFilter();
-        QStringList fixifiedSourceDirs = fileFixify(source_directories.toList(), FileFixifyAbsolute);
+        QStringList fixifiedSourceDirs = fileFixify(QList<QString>(source_directories.constBegin(), source_directories.constEnd()), FileFixifyAbsolute);
         fixifiedSourceDirs.removeDuplicates();
-        for (const QString &sourceDir : qAsConst(fixifiedSourceDirs)) {
-            QDirIterator dit(sourceDir, sourceFilesFilter, QDir::Files | QDir::NoDotAndDotDot);
-            while (dit.hasNext()) {
-                dit.next();
-                QString &duplicate = fileNames[dit.fileName()];
+        using F = QDirListing::IteratorFlag;
+        for (const QString &sourceDir : std::as_const(fixifiedSourceDirs)) {
+            for (const auto &dirEntry : QDirListing(sourceDir, sourceFilesFilter, F::FilesOnly | F::ResolveSymlinks)) {
+                QString &duplicate = fileNames[dirEntry.completeBaseName()];
                 if (duplicate.isNull()) {
-                    duplicate = dit.filePath();
+                    duplicate = dirEntry.filePath();
                 } else {
                     warn_msg(WarnLogic, "%s conflicts with %s", qPrintable(duplicate),
-                             qPrintable(dit.filePath()));
+                             qPrintable(dirEntry.filePath()));
                     duplicatesFound = true;
                 }
             }
@@ -552,7 +338,7 @@ void NmakeMakefileGenerator::writeImplicitRulesPart(QTextStream &t)
         project->variables().remove("QMAKE_RUN_CXX");
         project->variables().remove("QMAKE_RUN_CC");
 
-        for (const QString &sourceDir : qAsConst(source_directories)) {
+        for (const QString &sourceDir : std::as_const(source_directories)) {
             if (sourceDir.isEmpty())
                 continue;
             QString objDir = var("OBJECTS_DIR");
@@ -561,19 +347,19 @@ void NmakeMakefileGenerator::writeImplicitRulesPart(QTextStream &t)
             for(QStringList::Iterator cppit = Option::cpp_ext.begin(); cppit != Option::cpp_ext.end(); ++cppit)
                 t << '{' << escapeDependencyPath(sourceDir) << '}' << (*cppit)
                   << '{' << escapeDependencyPath(objDir) << '}' << Option::obj_ext << "::\n\t"
-                  << var("QMAKE_RUN_CXX_IMP_BATCH").replace(QRegExp("\\$@"), fileVar("OBJECTS_DIR"))
+                  << var("QMAKE_RUN_CXX_IMP_BATCH").replace(QRegularExpression("\\$@"), fileVar("OBJECTS_DIR"))
                   << "\n\t$<\n<<\n\n";
             for(QStringList::Iterator cit = Option::c_ext.begin(); cit != Option::c_ext.end(); ++cit)
                 t << '{' << escapeDependencyPath(sourceDir) << '}' << (*cit)
                   << '{' << escapeDependencyPath(objDir) << '}' << Option::obj_ext << "::\n\t"
-                  << var("QMAKE_RUN_CC_IMP_BATCH").replace(QRegExp("\\$@"), fileVar("OBJECTS_DIR"))
+                  << var("QMAKE_RUN_CC_IMP_BATCH").replace(QRegularExpression("\\$@"), fileVar("OBJECTS_DIR"))
                   << "\n\t$<\n<<\n\n";
         }
     } else {
         for(QStringList::Iterator cppit = Option::cpp_ext.begin(); cppit != Option::cpp_ext.end(); ++cppit)
-            t << (*cppit) << Option::obj_ext << ":\n\t" << var("QMAKE_RUN_CXX_IMP") << endl << endl;
+            t << (*cppit) << Option::obj_ext << ":\n\t" << var("QMAKE_RUN_CXX_IMP") << Qt::endl << Qt::endl;
         for(QStringList::Iterator cit = Option::c_ext.begin(); cit != Option::c_ext.end(); ++cit)
-            t << (*cit) << Option::obj_ext << ":\n\t" << var("QMAKE_RUN_CC_IMP") << endl << endl;
+            t << (*cit) << Option::obj_ext << ":\n\t" << var("QMAKE_RUN_CC_IMP") << Qt::endl << Qt::endl;
     }
 
 }
@@ -584,8 +370,9 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
 
     t << "first: all\n";
     t << "all: " << escapeDependencyPath(fileFixify(Option::output.fileName()))
-      << ' ' << depVar("ALL_DEPS") << " $(DESTDIR_TARGET)\n\n";
-    t << "$(DESTDIR_TARGET): " << depVar("PRE_TARGETDEPS") << " $(OBJECTS) " << depVar("POST_TARGETDEPS");
+      << ' ' << depVar("ALL_DEPS") << ' ' << depVar("DEST_TARGET") << "\n\n";
+    t << depVar("DEST_TARGET") << ": "
+      << depVar("PRE_TARGETDEPS") << " $(OBJECTS) " << depVar("POST_TARGETDEPS");
     if (templateName == "aux") {
         t << "\n\n";
         return;
@@ -619,10 +406,12 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
                 }
             } else {
                 manifest = fileFixify(manifest);
+                if (linkerSupportsEmbedding)
+                    extraLFlags = "/MANIFEST:embed /MANIFESTINPUT:" + escapeFilePath(manifest);
             }
 
             const QString resourceId = (templateName == "app") ? "1" : "2";
-            const bool incrementalLinking = project->values("QMAKE_LFLAGS").toQStringList().filter(QRegExp("(/|-)INCREMENTAL:NO")).isEmpty();
+            const bool incrementalLinking = project->values("QMAKE_LFLAGS").toQStringList().filter(QRegularExpression("(/|-)INCREMENTAL:NO")).isEmpty();
             if (incrementalLinking && !linkerSupportsEmbedding) {
                 // Link a resource that contains the manifest without modifying the exe/dll after linking.
 
@@ -673,7 +462,7 @@ void NmakeMakefileGenerator::writeBuildRulesPart(QTextStream &t)
     if(!project->isEmpty("QMAKE_POST_LINK")) {
         t << "\n\t" << var("QMAKE_POST_LINK");
     }
-    t << endl;
+    t << Qt::endl;
 }
 
 void NmakeMakefileGenerator::writeLinkCommand(QTextStream &t, const QString &extraFlags, const QString &extraInlineFileContent)
