@@ -225,12 +225,12 @@ QMakeEvaluator::getMemberArgs(const ProKey &func, int srclen, const ProStringLis
                               int *start, int *end)
 {
     *start = 0, *end = 0;
-    if (args.count() >= 2) {
+    if (args.size() >= 2) {
         bool ok = true;
         const ProString &start_str = args.at(1);
         *start = start_str.toInt(&ok);
         if (!ok) {
-            if (args.count() == 2) {
+            if (args.size() == 2) {
                 int dotdot = start_str.indexOf(statics.strDotDot);
                 if (dotdot != -1) {
                     *start = start_str.left(dotdot).toInt(&ok);
@@ -246,7 +246,7 @@ QMakeEvaluator::getMemberArgs(const ProKey &func, int srclen, const ProStringLis
             }
         } else {
             *end = *start;
-            if (args.count() == 3)
+            if (args.size() == 3)
                 *end = args.at(2).toInt(&ok);
             if (!ok) {
                 ProStringRoUser u1(func, m_tmp1);
@@ -546,7 +546,7 @@ void QMakeEvaluator::populateDeps(
             if (depends.isEmpty()) {
                 rootSet.insert(first(ProKey(prefix + item + priosfx)).toInt(), item);
             } else {
-                for (const ProString &dep : qAsConst(depends)) {
+                for (const ProString &dep : std::as_const(depends)) {
                     dset.insert(dep.toKey());
                     dependees[dep.toKey()] << item;
                 }
@@ -595,7 +595,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
             var = args[0];
             sep = args.at(1).toQString();
             beg = args.at(2).toInt();
-            if (args.count() == 4)
+            if (args.size() == 4)
                 end = args.at(3).toInt();
         } else {
             var = args[0];
@@ -630,7 +630,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     case E_SPRINTF: {
         ProStringRwUser u1(args.at(0), m_tmp1);
         QString tmp = u1.str();
-        for (int i = 1; i < args.count(); ++i)
+        for (int i = 1; i < args.size(); ++i)
             tmp = tmp.arg(args.at(i).toQStringView());
         ret << u1.extract(tmp);
         break;
@@ -642,7 +642,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         bool zeropad = false;
         bool leftalign = false;
         enum { DefaultSign, PadSign, AlwaysSign } sign = DefaultSign;
-        if (args.count() >= 2) {
+        if (args.size() >= 2) {
             const auto opts = split_value_list(args.at(1).toQStringView());
             for (const ProString &opt : opts) {
                 if (opt.startsWith(QLatin1String("ibase="))) {
@@ -687,7 +687,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
             outstr = QLatin1Char(' ');
         }
         QString numstr = QString::number(num, obase);
-        int space = width - outstr.length() - numstr.length();
+        int space = width - outstr.size() - numstr.size();
         if (space <= 0) {
             outstr += numstr;
         } else if (leftalign) {
@@ -703,7 +703,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     }
     case E_NUM_ADD: {
         qlonglong sum = 0;
-        for (const ProString &arg : qAsConst(args)) {
+        for (const ProString &arg : std::as_const(args)) {
             if (arg.contains(QLatin1Char('.'))) {
                 evalError(fL1S("num_add(): floats are currently not supported."));
                 goto allfail;
@@ -722,11 +722,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     }
     case E_JOIN: {
         ProString glue, before, after;
-        if (args.count() >= 2)
+        if (args.size() >= 2)
             glue = args.at(1);
-        if (args.count() >= 3)
+        if (args.size() >= 3)
             before = args[2];
-        if (args.count() == 4)
+        if (args.size() == 4)
             after = args[3];
         const ProStringList &var = values(map(args.at(0)));
         if (!var.isEmpty()) {
@@ -742,7 +742,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     }
     case E_SPLIT: {
         ProStringRoUser u1(m_tmp1);
-        const QString &sep = (args.count() == 2) ? u1.set(args.at(1)) : statics.field_sep;
+        const QString &sep = (args.size() == 2) ? u1.set(args.at(1)) : statics.field_sep;
         const auto vars = values(map(args.at(0)));
         for (const ProString &var : vars) {
             // FIXME: this is inconsistent with the "there are no empty strings" dogma.
@@ -816,7 +816,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         bool blob = false;
         bool lines = false;
         bool singleLine = true;
-        if (args.count() > 1) {
+        if (args.size() > 1) {
             if (!args.at(1).compare(QLatin1String("false"), Qt::CaseInsensitive))
                 singleLine = false;
             else if (!args.at(1).compare(QLatin1String("blob"), Qt::CaseInsensitive))
@@ -883,7 +883,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         bool blob = false;
         bool lines = false;
         bool singleLine = true;
-        if (args.count() > 1) {
+        if (args.size() > 1) {
             if (!args.at(1).compare(QLatin1String("false"), Qt::CaseInsensitive))
                 singleLine = false;
             else if (!args.at(1).compare(QLatin1String("blob"), Qt::CaseInsensitive))
@@ -893,7 +893,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         }
         int exitCode;
         QByteArray bytes = getCommandOutput(args.at(0).toQString(), &exitCode);
-        if (args.count() > 2 && !args.at(2).isEmpty()) {
+        if (args.size() > 2 && !args.at(2).isEmpty()) {
             m_valuemapStack.top()[args.at(2).toKey()] =
                     ProStringList(ProString(QString::number(exitCode)));
         }
@@ -936,7 +936,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         for (int i = 0; i < args.size(); ++i) {
             QString str = args.at(i).toQString();
             QChar *i_data = str.data();
-            int i_len = str.length();
+            int i_len = str.size();
             for (int x = 0; x < i_len; ++x) {
                 if (*(i_data+x) == QLatin1Char('\\') && x < i_len-1) {
                     if (*(i_data+x+1) == QLatin1Char('\\')) {
@@ -981,14 +981,14 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     case E_UPPER:
     case E_LOWER:
     case E_TITLE:
-        for (int i = 0; i < args.count(); ++i) {
+        for (int i = 0; i < args.size(); ++i) {
             ProStringRwUser u1(args.at(i), m_tmp1);
             QString rstr = u1.str();
             if (func_t == E_UPPER) {
                 rstr = rstr.toUpper();
             } else {
                 rstr = rstr.toLower();
-                if (func_t == E_TITLE && rstr.length() > 0)
+                if (func_t == E_TITLE && rstr.size() > 0)
                     rstr[0] = rstr.at(0).toTitleCase();
             }
             ret << u1.extract(rstr);
@@ -996,7 +996,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         break;
     case E_FILES: {
         bool recursive = false;
-        if (args.count() == 2)
+        if (args.size() == 2)
             recursive = isTrue(args.at(1));
         QStringList dirs;
         ProStringRoUser u1(args.at(0), m_tmp1);
@@ -1022,7 +1022,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
             evalError(fL1S("section(): Encountered invalid wildcard expression '%1'.").arg(pattern));
             goto allfail;
         }
-        for (int d = 0; d < dirs.count(); d++) {
+        for (int d = 0; d < dirs.size(); d++) {
             QString dir = dirs[d];
             QDir qdir(pfx + dir);
             for (int i = 0, count = int(qdir.count()); i < count; ++i) {
@@ -1044,7 +1044,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         ProStringRoUser u1(args.at(0), m_tmp1);
         QString msg = m_option->expandEnvVars(u1.str());
         bool decorate = true;
-        if (args.count() == 2)
+        if (args.size() == 2)
             decorate = isTrue(args.at(1));
         if (decorate) {
             if (!msg.endsWith(QLatin1Char('?')))
@@ -1091,10 +1091,10 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         ProValueMap dependees;
         QMultiMap<int, ProString> rootSet;
         ProStringList orgList = values(args.at(0).toKey());
-        ProString prefix = args.count() < 2 ? ProString() : args.at(1);
-        ProString priosfx = args.count() < 4 ? ProString(".priority") : args.at(3);
+        ProString prefix = args.size() < 2 ? ProString() : args.at(1);
+        ProString priosfx = args.size() < 4 ? ProString(".priority") : args.at(3);
         populateDeps(orgList, prefix,
-                     args.count() < 3 ? ProStringList(ProString(".depends"))
+                     args.size() < 3 ? ProStringList(ProString(".depends"))
                                       : split_value_list(args.at(2).toQStringView()),
                      priosfx, dependencies, dependees, rootSet);
         while (!rootSet.isEmpty()) {
@@ -1103,7 +1103,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
             rootSet.erase(it);
             if ((func_t == E_RESOLVE_DEPENDS) || orgList.contains(item))
                 ret.prepend(item);
-            for (const ProString &dep : qAsConst(dependees[item.toKey()])) {
+            for (const ProString &dep : std::as_const(dependees[item.toKey()])) {
                 QSet<ProKey> &dset = dependencies[dep.toKey()];
                 dset.remove(item.toKey());
                 if (dset.isEmpty())
@@ -1114,11 +1114,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     }
     case E_ENUMERATE_VARS: {
         QSet<ProString> keys;
-        for (const ProValueMap &vmap : qAsConst(m_valuemapStack))
+        for (const ProValueMap &vmap : std::as_const(m_valuemapStack))
             for (ProValueMap::ConstIterator it = vmap.constBegin(); it != vmap.constEnd(); ++it)
                 keys.insert(it.key());
         ret.reserve(keys.size());
-        for (const ProString &key : qAsConst(keys))
+        for (const ProString &key : std::as_const(keys))
             ret << key;
         break; }
     case E_SHADOWED: {
@@ -1131,7 +1131,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     case E_ABSOLUTE_PATH: {
         ProStringRwUser u1(args.at(0), m_tmp1);
         ProStringRwUser u2(m_tmp2);
-        QString baseDir = args.count() > 1
+        QString baseDir = args.size() > 1
                 ? IoUtils::resolvePath(currentDirectory(), u2.set(args.at(1)))
                 : currentDirectory();
         QString rstr = u1.str().isEmpty() ? baseDir : IoUtils::resolvePath(baseDir, u1.str());
@@ -1141,7 +1141,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     case E_RELATIVE_PATH: {
         ProStringRwUser u1(args.at(0), m_tmp1);
         ProStringRoUser u2(m_tmp2);
-        QString baseDir = args.count() > 1
+        QString baseDir = args.size() > 1
                 ? IoUtils::resolvePath(currentDirectory(), u2.set(args.at(1)))
                 : currentDirectory();
         QString absArg = u1.str().isEmpty() ? baseDir : IoUtils::resolvePath(baseDir, u1.str());
@@ -1252,7 +1252,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::testFunc_cache(const ProStringList &
     enum { TargetStash, TargetCache, TargetSuper } target = TargetCache;
     enum { CacheSet, CacheAdd, CacheSub } mode = CacheSet;
     ProKey srcvar;
-    if (args.count() >= 2) {
+    if (args.size() >= 2) {
         const auto opts = split_value_list(args.at(1).toQStringView());
         for (const ProString &opt : opts) {
             if (opt == QLatin1String("transient")) {
@@ -1272,7 +1272,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::testFunc_cache(const ProStringList &
                 return ReturnFalse;
             }
         }
-        if (args.count() >= 3) {
+        if (args.size() >= 3) {
             srcvar = args.at(2).toKey();
         } else if (mode != CacheSet) {
             evalError(fL1S("cache(): modes other than 'set' require a source variable."));
@@ -1367,7 +1367,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::testFunc_cache(const ProStringList &
             varstr += QLatin1String(" -=");
         else
             varstr += QLatin1String(" =");
-        if (diffval.count() == 1) {
+        if (diffval.size() == 1) {
             varstr += QLatin1Char(' ');
             varstr += quoteValue(diffval.at(0));
         } else if (!diffval.isEmpty()) {
@@ -1425,7 +1425,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
     switch (func_t) {
     case T_DEFINED: {
         const ProKey &var = args.at(0).toKey();
-        if (args.count() > 1) {
+        if (args.size() > 1) {
             if (args[1] == QLatin1String("test")) {
                 return returnBool(m_functionDefs.testFunctions.contains(var));
             } else if (args[1] == QLatin1String("replace")) {
@@ -1512,7 +1512,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         VisitReturn ok = evaluateFileInto(fn, &vars, LoadProOnly);
         if (ok != ReturnTrue)
             return ok;
-        if (args.count() == 2)
+        if (args.size() == 2)
             return returnBool(vars.contains(map(args.at(1))));
         QRegularExpression regx;
         regx.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
@@ -1562,14 +1562,14 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
                                    m_current.pro->fileName(), m_current.line);
     }
     case T_CONFIG: {
-        if (args.count() == 1)
+        if (args.size() == 1)
             return returnBool(isActiveConfig(args.at(0).toQStringView()));
         const auto mutuals = args.at(1).toQStringView().split(QLatin1Char('|'),
                                                              Qt::SkipEmptyParts);
         const ProStringList &configs = values(statics.strCONFIG);
 
         for (int i = configs.size() - 1; i >= 0; i--) {
-            for (int mut = 0; mut < mutuals.count(); mut++) {
+            for (int mut = 0; mut < mutuals.size(); mut++) {
                 if (configs[i].toQStringView() == mutuals[mut].trimmed())
                     return returnBool(configs[i] == args[0]);
             }
@@ -1589,7 +1589,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             }
         }
         const ProStringList &l = values(map(args.at(0)));
-        if (args.count() == 2) {
+        if (args.size() == 2) {
             for (int i = 0; i < l.size(); ++i) {
                 const ProString &val = l[i];
                 if (val == qry)
@@ -1605,7 +1605,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
                                                                  Qt::SkipEmptyParts);
             for (int i = l.size() - 1; i >= 0; i--) {
                 const ProString &val = l[i];
-                for (int mut = 0; mut < mutuals.count(); mut++) {
+                for (int mut = 0; mut < mutuals.size(); mut++) {
                     if (val.toQStringView() == mutuals[mut].trimmed()) {
                         if (val == qry)
                             return ReturnTrue;
@@ -1622,9 +1622,9 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         return ReturnFalse;
     }
     case T_COUNT: {
-        int cnt = values(map(args.at(0))).count();
+        int cnt = values(map(args.at(0))).size();
         int val = args.at(1).toInt();
-        if (args.count() == 3) {
+        if (args.size() == 3) {
             const ProString &comp = args.at(2);
             if (comp == QLatin1String(">") || comp == QLatin1String("greaterThan")) {
                 return returnBool(cnt > val);
@@ -1710,10 +1710,10 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         LoadFlags flags;
         if (m_cumulative)
             flags = LoadSilent;
-        if (args.count() >= 2) {
+        if (args.size() >= 2) {
             if (!args.at(1).isEmpty())
                 parseInto = args.at(1) + QLatin1Char('.');
-            if (args.count() >= 3 && isTrue(args.at(2)))
+            if (args.size() >= 3 && isTrue(args.at(2)))
                 flags = LoadSilent;
         }
         QString fn = filePathEnvArg0(args);
@@ -1745,7 +1745,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         return ok;
     }
     case T_LOAD: {
-        bool ignore_error = (args.count() == 2 && isTrue(args.at(1)));
+        bool ignore_error = (args.size() == 2 && isTrue(args.at(1)));
         VisitReturn ok = evaluateFeatureFile(m_option->expandEnvVars(args.at(0).toQString()),
                                              ignore_error);
         if (ok == ReturnFalse && ignore_error)
@@ -1844,11 +1844,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         QIODevice::OpenMode mode = QIODevice::Truncate;
         QMakeVfs::VfsFlags flags = (m_cumulative ? QMakeVfs::VfsCumulative : QMakeVfs::VfsExact);
         QString contents;
-        if (args.count() >= 2) {
+        if (args.size() >= 2) {
             const ProStringList &vals = values(args.at(1).toKey());
             if (!vals.isEmpty())
                 contents = vals.join(QLatin1Char('\n')) + QLatin1Char('\n');
-            if (args.count() >= 3) {
+            if (args.size() >= 3) {
                 const auto opts = split_value_list(args.at(2).toQStringView());
                 for (const ProString &opt : opts) {
                     if (opt == QLatin1String("append")) {
