@@ -1,14 +1,13 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qloggingcategory.h"
+#include "qloggingcategory_p.h"
 #include "qloggingregistry_p.h"
 
 QT_BEGIN_NAMESPACE
 
-const char qtDefaultCategoryName[] = "default";
-Q_GLOBAL_STATIC(QLoggingCategory, qtDefaultCategory, qtDefaultCategoryName)
-
 /*!
     \class QLoggingCategory
     \inmodule QtCore
@@ -170,17 +169,10 @@ Q_GLOBAL_STATIC(QLoggingCategory, qtDefaultCategory, qtDefaultCategoryName)
     \since 5.4
 */
 QLoggingCategory::QLoggingCategory(const char *category, QtMsgType enableForLevel)
-    : d(nullptr),
-      name(nullptr)
+    : QLoggingCategory(UnregisteredInitialization{},
+                       category ? category : QLoggingRegistry::defaultCategoryName)
 {
-    enabled.storeRelaxed(0x01010101);   // enabledDebug = enabledWarning = enabledCritical = true;
-
-    if (category)
-        name = category;
-    else
-        name = qtDefaultCategoryName;
-
-    if (QLoggingRegistry *reg = QLoggingRegistry::instance())
+   if (QLoggingRegistry *reg = QLoggingRegistry::instance())
         reg->registerCategory(this, enableForLevel);
 }
 
@@ -189,6 +181,9 @@ QLoggingCategory::QLoggingCategory(const char *category, QtMsgType enableForLeve
 */
 QLoggingCategory::~QLoggingCategory()
 {
+    // Note: this destructor is never called for the defaultCategory(), so it
+    // is always registered. If we ever need to free memory, make
+    // QLoggingRegistry do it.
     if (QLoggingRegistry *reg = QLoggingRegistry::instance())
         reg->unregisterCategory(this);
 }
@@ -309,7 +304,7 @@ void QLoggingCategory::setEnabled(QtMsgType type, bool enable)
  */
 QLoggingCategory *QLoggingCategory::defaultCategory()
 {
-    return qtDefaultCategory();
+    return QLoggingRegistry::defaultCategory();
 }
 
 /*!
@@ -400,7 +395,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note Arguments aren't processed if the debug output for that \a category is not
     enabled, so don't rely on any side effects.
 
-    \sa qDebug()
+    \sa QDebug::qDebug()
 */
 
 /*!
@@ -420,7 +415,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note Arguments aren't processed if the debug output for that \a category is not
     enabled, so don't rely on any side effects.
 
-    \sa qDebug()
+    \sa qDebug(const char *, ...)
 */
 
 /*!
@@ -443,7 +438,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the debug output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qInfo()
+    \sa QDebug::qInfo()
 */
 
 /*!
@@ -463,7 +458,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the debug output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qInfo()
+    \sa qInfo(const char *, ...)
 */
 
 /*!
@@ -486,7 +481,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the warning output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qWarning()
+    \sa QDebug::qWarning()
 */
 
 /*!
@@ -506,7 +501,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the warning output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qWarning()
+    \sa qWarning(const char *, ...)
 */
 
 /*!
@@ -530,7 +525,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the critical output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qCritical()
+    \sa QDebug::qCritical()
 */
 
 /*!
@@ -550,7 +545,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     \note If the critical output for a particular category isn't enabled, arguments
     won't be processed, so don't rely on any side effects.
 
-    \sa qCritical()
+    \sa qCritical(const char *, ...)
 */
 
 /*!
@@ -569,7 +564,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
 
     \snippet qloggingcategory/main.cpp 16
 
-    \sa qFatal()
+    \sa QDebug::qFatal()
 */
 
 /*!
@@ -589,7 +584,7 @@ void QLoggingCategory::setFilterRules(const QString &rules)
     to create a core dump. On Windows, for debug builds, this function will
     report a \c _CRT_ERROR enabling you to connect a debugger to the application.
 
-    \sa qFatal()
+    \sa qFatal(const char *, ...)
 */
 
 /*!
