@@ -303,7 +303,7 @@ QRangeModel::QRangeModel(QRangeModelImplBase *impl, QObject *parent)
     tree data structure needs to be homomorphic: on all levels of the tree, the
     list of child rows needs to use the exact same representation as the tree
     itself. In addition, the row type needs be of a static size: either a gadget
-    or QObject type, or a type that implements the {C++ tuple protocol}.
+    or QObject type, or a type that implements \l{the C++ tuple protocol}.
 
     To represent such data as a tree, QRangeModel has to be able to traverse the
     data structure: for any given row, the model needs to be able to retrieve
@@ -337,9 +337,8 @@ QRangeModel::QRangeModel(QRangeModelImplBase *impl, QObject *parent)
 
     The tree traversal protocol can then be implemented as member functions of
     the row data type. A const \c{parentRow()} function has to return a pointer
-    to a const row item; and the \c{childRows()} function has to return a
-    reference to a const \c{std::optional} that can hold the optional child
-    range.
+    to a row item; and the \c{childRows()} function has to return a reference
+    to a const \c{std::optional} that can hold the optional child range.
 
     These two functions are sufficient for the model to navigate the tree as a
     read-only data structure. To allow the user to edit data in a view, and the
@@ -1044,14 +1043,16 @@ QHash<int, QByteArray> QRangeModelImplBase::roleNamesForMetaObject(const QAbstra
 {
     const auto defaults = model.QAbstractItemModel::roleNames();
     QHash<int, QByteArray> result = {{Qt::RangeModelDataRole, "modelData"}};
-    const int offset = metaObject.propertyOffset();
+    int offset = metaObject.propertyOffset();
     for (int i = offset; i < metaObject.propertyCount(); ++i) {
         const auto name = metaObject.property(i).name();
         const int defaultRole = defaults.key(name, -1);
-        if (defaultRole != -1)
+        if (defaultRole != -1) {
+            ++offset;
             result[defaultRole] = name;
-        else
+        } else {
             result[Qt::UserRole + i - offset] = name;
+        }
     }
     return result;
 }
