@@ -11,12 +11,15 @@
 #include <QtCore/qtimer.h>
 #include <QtCore/qset.h>
 
-#if QT_CONFIG(future)
+#if QT_CONFIG(permissions)
 #include <QtCore/qpromise.h>
 #endif
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+using namespace QtJniTypes;
+
 class QAndroidParcelPrivate
 {
 public:
@@ -63,13 +66,7 @@ void QAndroidParcelPrivate::writeData(const QByteArray &data) const
     if (data.isEmpty())
         return;
 
-    QJniEnvironment().checkAndClearExceptions();
-    QJniEnvironment env;
-    jbyteArray array = env->NewByteArray(data.size());
-    env->SetByteArrayRegion(array, 0, data.length(),
-                            reinterpret_cast<const jbyte*>(data.constData()));
-    handle.callMethod<void>("writeByteArray", "([B)V", array);
-    env->DeleteLocalRef(array);
+    handle.callMethod<void>("writeByteArray", data);
 }
 
 void QAndroidParcelPrivate::writeBinder(const QAndroidBinder &binder) const
@@ -126,8 +123,6 @@ QAndroidBinder QAndroidParcelPrivate::readBinder() const
     \l {https://developer.android.com/reference/android/os/Parcel.html}{Android Parcel}
     methods.
 
-    \include qtcore.qdoc qtcoreprivate-usage
-
     \since 6.2
 */
 
@@ -245,8 +240,6 @@ QJniObject QAndroidParcel::handle() const
     \l {https://developer.android.com/reference/android/os/Binder.html}{Android Binder}
     methods.
 
-    \include qtcore.qdoc qtcoreprivate-usage
-
     \since 6.2
 */
 
@@ -387,8 +380,6 @@ QJniObject QAndroidBinder::handle() const
 
     It is useful when you perform a QtAndroidPrivate::bindService operation.
 
-    \include qtcore.qdoc qtcoreprivate-usage
-
     \since 6.2
 */
 
@@ -451,7 +442,7 @@ QJniObject QAndroidServiceConnection::handle() const
  */
 
 
-static QBasicAtomicInteger<uint> nextUniqueActivityRequestCode = Q_BASIC_ATOMIC_INITIALIZER(0);
+Q_CONSTINIT static QBasicAtomicInteger<uint> nextUniqueActivityRequestCode = Q_BASIC_ATOMIC_INITIALIZER(0);
 
 // Get a unique activity request code.
 static int uniqueActivityRequestCode()
@@ -499,7 +490,7 @@ public:
 
     static QAndroidActivityResultReceiverPrivate *get(QAndroidActivityResultReceiver *publicObject)
     {
-        return publicObject->d.data();
+        return publicObject->d.get();
     }
 };
 
@@ -514,7 +505,6 @@ public:
   Create a subclass of this class to be notified of the results when using the
   \c QtAndroidPrivate::startActivity() and \c QtAndroidPrivate::startIntentSender() APIs.
 
-  \include qtcore.qdoc qtcoreprivate-usage
  */
 
 /*!
@@ -524,7 +514,7 @@ QAndroidActivityResultReceiver::QAndroidActivityResultReceiver()
     : d(new QAndroidActivityResultReceiverPrivate)
 {
     d->q = this;
-    QtAndroidPrivate::registerActivityResultListener(d.data());
+    QtAndroidPrivate::registerActivityResultListener(d.get());
 }
 
 /*!
@@ -532,7 +522,7 @@ QAndroidActivityResultReceiver::QAndroidActivityResultReceiver()
 */
 QAndroidActivityResultReceiver::~QAndroidActivityResultReceiver()
 {
-    QtAndroidPrivate::unregisterActivityResultListener(d.data());
+    QtAndroidPrivate::unregisterActivityResultListener(d.get());
 }
 
 /*!
@@ -612,8 +602,6 @@ public:
     \l {https://developer.android.com/reference/android/app/Service.html}{Android Service}
     methods.
 
-    \include qtcore.qdoc qtcoreprivate-usage
-
     \since 6.2
 */
 
@@ -672,6 +660,45 @@ QAndroidBinder* QAndroidService::onBind(const QAndroidIntent &/*intent*/)
     return nullptr;
 }
 
+static jboolean onTransact(JNIEnv */*env*/, jclass /*cls*/, jlong id, jint code, jobject data,
+                           jobject reply, jint flags)
+{
+    if (!id)
+        return false;
+
+    return reinterpret_cast<QAndroidBinder*>(id)->onTransact(
+            code, QAndroidParcel(data), QAndroidParcel(reply), QAndroidBinder::CallType(flags));
+}
+
+static void onServiceConnected(JNIEnv */*env*/, jclass /*cls*/, jlong id, jstring name,
+                               jobject service)
+{
+    if (!id)
+        return;
+
+    return reinterpret_cast<QAndroidServiceConnection *>(id)->onServiceConnected(
+            QJniObject(name).toString(), QAndroidBinder(service));
+}
+
+static void onServiceDisconnected(JNIEnv */*env*/, jclass /*cls*/, jlong id, jstring name)
+{
+    if (!id)
+        return;
+
+    return reinterpret_cast<QAndroidServiceConnection *>(id)->onServiceDisconnected(
+            QJniObject(name).toString());
+}
+
+bool QtAndroidPrivate::registerExtrasNatives(QJniEnvironment &env)
+{
+    static const JNINativeMethod methods[] = {
+        {"onTransact", "(JILandroid/os/Parcel;Landroid/os/Parcel;I)Z", (void *)onTransact},
+        {"onServiceConnected", "(JLjava/lang/String;Landroid/os/IBinder;)V", (void *)onServiceConnected},
+        {"onServiceDisconnected", "(JLjava/lang/String;)V", (void *)onServiceDisconnected}
+    };
+
+    return env.registerNativeMethods("org/qtproject/qt/android/extras/QtNative", methods, 3);
+}
 
 /*!
     \class QAndroidIntent
@@ -684,8 +711,6 @@ QAndroidBinder* QAndroidService::onBind(const QAndroidIntent &/*intent*/)
     \l {https://developer.android.com/reference/android/content/Intent.html}{Android Intent}
     methods.
 
-    \include qtcore.qdoc qtcoreprivate-usage
-
     \since 6.2
 */
 
@@ -740,15 +765,7 @@ QAndroidIntent::QAndroidIntent(const QJniObject &packageContext, const char *cla
  */
 void QAndroidIntent::putExtra(const QString &key, const QByteArray &data)
 {
-    QJniEnvironment().checkAndClearExceptions();
-    QJniEnvironment env;
-    jbyteArray array = env->NewByteArray(data.size());
-    env->SetByteArrayRegion(array, 0, data.length(),
-                            reinterpret_cast<const jbyte*>(data.constData()));
-    m_handle.callObjectMethod("putExtra", "(Ljava/lang/String;[B)Landroid/content/Intent;",
-                              QJniObject::fromString(key).object(), array);
-    env->DeleteLocalRef(array);
-    QJniEnvironment().checkAndClearExceptions();
+    m_handle.callMethod<QtJniTypes::Intent>("putExtra", key, data);
 }
 
 /*!
@@ -756,19 +773,7 @@ void QAndroidIntent::putExtra(const QString &key, const QByteArray &data)
  */
 QByteArray QAndroidIntent::extraBytes(const QString &key)
 {
-    QJniEnvironment().checkAndClearExceptions();
-    auto array = m_handle.callObjectMethod("getByteArrayExtra", "(Ljava/lang/String;)[B",
-                                           QJniObject::fromString(key).object());
-    if (!array.isValid() || !array.object())
-        return QByteArray();
-    QJniEnvironment env;
-    auto sz = env->GetArrayLength(jarray(array.object()));
-    QByteArray res(sz, Qt::Initialization::Uninitialized);
-    env->GetByteArrayRegion(jbyteArray(array.object()), 0, sz,
-                            reinterpret_cast<jbyte *>(res.data()));
-    QJniEnvironment().checkAndClearExceptions();
-
-    return res;
+    return m_handle.callMethod<QByteArray>("getByteArrayExtra", key);
 }
 
 /*!
@@ -811,8 +816,6 @@ QJniObject QAndroidIntent::handle() const
     \brief The QtAndroidPrivate namespace provides miscellaneous functions
            to aid Android development.
     \inheaderfile QtCore/private/qandroidextras_p.h
-
-    \include qtcore.qdoc qtcoreprivate-usage
 */
 
 /*!
@@ -858,8 +861,9 @@ QJniObject QAndroidIntent::handle() const
   Starts the activity given by \a intent and provides the result asynchronously through the
   \a resultReceiver if this is non-null.
 
-  If \a resultReceiver is null, then the \c startActivity() method in the \c androidActivity()
-  will be called. Otherwise \c startActivityForResult() will be called.
+  If \a resultReceiver is null, then the \c startActivity() method of
+  QNativeInterface::QAndroidApplication::context() will be called. Otherwise
+  \c startActivityForResult() will be called.
 
   The \a receiverRequestCode is a request code unique to the \a resultReceiver, and will be
   returned along with the result, making it possible to use the same receiver for more than
@@ -891,8 +895,9 @@ void QtAndroidPrivate::startActivity(const QJniObject &intent,
   Starts the activity given by \a intent and provides the result asynchronously through the
   \a resultReceiver if this is non-null.
 
-  If \a resultReceiver is null, then the \c startActivity() method in the \c androidActivity()
-  will be called. Otherwise \c startActivityForResult() will be called.
+  If \a resultReceiver is null, then the \c startActivity() method of
+  QNativeInterface::QAndroidApplication::context() will be called. Otherwise
+  \c startActivityForResult() will be called.
 
   The \a receiverRequestCode is a request code unique to the \a resultReceiver, and will be
   returned along with the result, making it possible to use the same receiver for more than
@@ -928,8 +933,9 @@ void QtAndroidPrivate::startActivity(const QJniObject &intent,
   Starts the activity given by \a intentSender and provides the result asynchronously through the
   \a resultReceiver if this is non-null.
 
-  If \a resultReceiver is null, then the \c startIntentSender() method in the \c androidActivity()
-  will be called. Otherwise \c startIntentSenderForResult() will be called.
+  If \a resultReceiver is null, then the \c startIntentSender() method of
+  QNativeInterface::QAndroidApplication::context() will be called. Otherwise
+  \c startIntentSenderForResult() will be called.
 
   The \a receiverRequestCode is a request code unique to the \a resultReceiver, and will be
   returned along with the result, making it possible to use the same receiver for more than
@@ -1024,10 +1030,9 @@ void QAndroidActivityCallbackResultReceiver::registerCallback(
     callbackMap.insert(receiverRequestCode, callbackFunc);
 }
 
+#if QT_CONFIG(permissions)
 // Permissions API
 
-static const char qtNativeClassName[] = "org/qtproject/qt/android/QtNative";
-
 QtAndroidPrivate::PermissionResult resultFromAndroid(jint value)
 {
     return value == 0 ? QtAndroidPrivate::Authorized : QtAndroidPrivate::Denied;
@@ -1036,72 +1041,25 @@ QtAndroidPrivate::PermissionResult resultFromAndroid(jint value)
 using PendingPermissionRequestsHash
             = QHash<int, QSharedPointer<QPromise<QtAndroidPrivate::PermissionResult>>>;
 Q_GLOBAL_STATIC(PendingPermissionRequestsHash, g_pendingPermissionRequests);
-static QBasicMutex g_pendingPermissionRequestsMutex;
+Q_CONSTINIT static QBasicMutex g_pendingPermissionRequestsMutex;
 
 static int nextRequestCode()
 {
-    static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(0);
+    Q_CONSTINIT static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(0);
     return counter.fetchAndAddRelaxed(1);
 }
 
-static QStringList nativeStringsFromPermission(QtAndroidPrivate::PermissionType permission)
-{
-    static const auto precisePerm = QStringLiteral("android.permission.ACCESS_FINE_LOCATION");
-    static const auto coarsePerm = QStringLiteral("android.permission.ACCESS_COARSE_LOCATION");
-    static const auto backgroundPerm =
-            QStringLiteral("android.permission.ACCESS_BACKGROUND_LOCATION");
-
-    switch (permission) {
-    case QtAndroidPrivate::Location:
-        return {coarsePerm};
-    case QtAndroidPrivate::PreciseLocation:
-        return {precisePerm};
-    case QtAndroidPrivate::BackgroundLocation:
-        // Keep the background permission first to be able to use .first()
-        // in checkPermission because it takes single permission
-        if (QtAndroidPrivate::androidSdkVersion() >= 29)
-            return {backgroundPerm, coarsePerm};
-        return {coarsePerm};
-    case QtAndroidPrivate::PreciseBackgroundLocation:
-        // Keep the background permission first to be able to use .first()
-        // in checkPermission because it takes single permission
-        if (QtAndroidPrivate::androidSdkVersion() >= 29)
-            return {backgroundPerm, precisePerm};
-        return {precisePerm};
-    case QtAndroidPrivate::Camera:
-        return {QStringLiteral("android.permission.CAMERA")};
-    case QtAndroidPrivate::Microphone:
-        return {QStringLiteral("android.permission.RECORD_AUDIO")};
-    case QtAndroidPrivate::Bluetooth:
-        return { QStringLiteral("android.permission.BLUETOOTH") };
-    case QtAndroidPrivate::BodySensors:
-        return {QStringLiteral("android.permission.BODY_SENSORS")};
-    case QtAndroidPrivate::PhysicalActivity:
-        return {QStringLiteral("android.permission.ACTIVITY_RECOGNITION")};
-    case QtAndroidPrivate::Contacts:
-        return {QStringLiteral("android.permission.READ_CONTACTS"),
-                QStringLiteral("android.permission.WRITE_CONTACTS")};
-    case QtAndroidPrivate::Storage:
-        return {QStringLiteral("android.permission.READ_EXTERNAL_STORAGE"),
-                QStringLiteral("android.permission.WRITE_EXTERNAL_STORAGE")};
-    case QtAndroidPrivate::Calendar:
-        return {QStringLiteral("android.permission.READ_CALENDAR"),
-                QStringLiteral("android.permission.WRITE_CALENDAR")};
-    }
-
-    return {};
-}
-
 /*!
     \internal
 
     This function is called when the result of the permission request is available.
-    Once a permission is requested, the result is braodcast by the OS and listened
+    Once a permission is requested, the result is broadcast by the OS and listened
     to by QtActivity which passes it to C++ through a native JNI method call.
  */
-static void sendRequestPermissionsResult(JNIEnv *env, jobject *obj, jint requestCode,
-                                         jobjectArray permissions, jintArray grantResults)
+static void sendRequestPermissionsResult(JNIEnv *env, jclass obj, jint requestCode,
+                                         const QJniArray<int> &grantResults)
 {
+    Q_UNUSED(env);
     Q_UNUSED(obj);
 
     QMutexLocker locker(&g_pendingPermissionRequestsMutex);
@@ -1115,38 +1073,47 @@ static void sendRequestPermissionsResult(JNIEnv *env, jobject *obj, jint request
     g_pendingPermissionRequests->erase(it);
     locker.unlock();
 
-    const int size = env->GetArrayLength(permissions);
-    std::unique_ptr<jint[]> results(new jint[size]);
-    env->GetIntArrayRegion(grantResults, 0, size, results.get());
-
-    for (int i = 0 ; i < size; ++i) {
-        QtAndroidPrivate::PermissionResult result = resultFromAndroid(results[i]);
-        request->addResult(result, i);
-    }
+    request->addResults([grantResults](){
+        QList<QtAndroidPrivate::PermissionResult> results(grantResults.size(),
+                                                          Qt::Uninitialized);
+        for (qsizetype i = 0; i < grantResults.size(); ++i)
+            results[i] = resultFromAndroid(grantResults.at(i));
+        return results;
+    }());
 
+    QtAndroidPrivate::releaseAndroidDeadlockProtector();
     request->finish();
 }
+Q_DECLARE_JNI_NATIVE_METHOD(sendRequestPermissionsResult)
 
 QFuture<QtAndroidPrivate::PermissionResult>
 requestPermissionsInternal(const QStringList &permissions)
 {
+    // No mechanism to request permission for SDK version below 23, because
+    // permissions defined in the manifest are granted at install time.
+    if (QtAndroidPrivate::androidSdkVersion() < 23) {
+        QList<QtAndroidPrivate::PermissionResult> result;
+        result.reserve(permissions.size());
+        // ### can we kick off all checkPermission()s, and whenAll() collect results?
+        for (const QString &permission : permissions)
+            result.push_back(QtAndroidPrivate::checkPermission(permission).result());
+        return QtFuture::makeReadyRangeFuture(result);
+    }
+
+    QtAndroidPrivate::AndroidDeadlockProtector protector(
+        u"requestPermissionsInternal()"_s);
+    if (!protector.acquire())
+        return QtFuture::makeReadyValueFuture(QtAndroidPrivate::Denied);
+
     QSharedPointer<QPromise<QtAndroidPrivate::PermissionResult>> promise;
     promise.reset(new QPromise<QtAndroidPrivate::PermissionResult>());
     QFuture<QtAndroidPrivate::PermissionResult> future = promise->future();
     promise->start();
 
-    // No mechanism to request permission for SDK version below 23, because
-    // permissions defined in the manifest are granted at install time.
-    if (QtAndroidPrivate::androidSdkVersion() < 23) {
-        for (int i = 0; i < permissions.size(); ++i)
-            promise->addResult(QtAndroidPrivate::checkPermission(permissions.at(i)).result(), i);
-        promise->finish();
-        return future;
-    }
-
     const int requestCode = nextRequestCode();
     QMutexLocker locker(&g_pendingPermissionRequestsMutex);
     g_pendingPermissionRequests->insert(requestCode, promise);
+    locker.unlock();
 
     QNativeInterface::QAndroidApplication::runOnAndroidMainThread([permissions, requestCode] {
         QJniEnvironment env;
@@ -1164,6 +1131,8 @@ requestPermissionsInternal(const QStringList &permissions)
         env->DeleteLocalRef(array);
     });
 
+    QtAndroidPrivate::releaseAndroidDeadlockProtector();
+
     return future;
 }
 
@@ -1172,71 +1141,26 @@ requestPermissionsInternal(const QStringList &permissions)
     Requests the \a permission and returns a QFuture representing the
     result of the request.
 
+    \note QPermission is the recommended API to use for requesting permissions.
+        If QPermission doesn't cover an Android permission you want to request,
+        this preliminary API can still used instead.
+
     \since 6.2
     \sa checkPermission()
 */
 QFuture<QtAndroidPrivate::PermissionResult>
 QtAndroidPrivate::requestPermission(const QString &permission)
 {
-    // avoid the uneccessary call and response to an empty permission string
-    if (permission.size() > 0)
-        return requestPermissionsInternal({permission});
-
-    QPromise<QtAndroidPrivate::PermissionResult> promise;
-    QFuture<QtAndroidPrivate::PermissionResult> future = promise.future();
-    promise.start();
-    promise.addResult(QtAndroidPrivate::Denied);
-    promise.finish();
-    return future;
+    return requestPermissions({permission});
 }
 
-static bool isBackgroundLocationApi29(QtAndroidPrivate::PermissionType permission)
-{
-    return QNativeInterface::QAndroidApplication::sdkVersion() >= 29
-            && (permission == QtAndroidPrivate::BackgroundLocation
-                || permission == QtAndroidPrivate::PreciseBackgroundLocation);
-}
-
-/*!
-    \preliminary
-
-    Requests the \a permission and returns a QFuture representing the
-    result of the request.
-
-    \since 6.2
-    \sa checkPermission()
-*/
 QFuture<QtAndroidPrivate::PermissionResult>
-QtAndroidPrivate::requestPermission(QtAndroidPrivate::PermissionType permission)
+QtAndroidPrivate::requestPermissions(const QStringList &permissions)
 {
-    QSharedPointer<QPromise<QtAndroidPrivate::PermissionResult>> promise;
-    promise.reset(new QPromise<QtAndroidPrivate::PermissionResult>());
-    QFuture<QtAndroidPrivate::PermissionResult> future = promise->future();
-    promise->start();
-    const auto nativePermissions = nativeStringsFromPermission(permission);
-
-    if (nativePermissions.size() > 0 && QtAndroidPrivate::acquireAndroidDeadlockProtector()) {
-        requestPermissionsInternal(nativePermissions).then(
-                    [promise, permission](QFuture<QtAndroidPrivate::PermissionResult> future) {
-            auto AuthorizedCount = future.results().count(QtAndroidPrivate::Authorized);
-            if (AuthorizedCount > 0) {
-                if (isBackgroundLocationApi29(permission))
-                    promise->addResult(future.resultAt(0), 0);
-                else
-                    promise->addResult(QtAndroidPrivate::Authorized, 0);
-            } else {
-                promise->addResult(QtAndroidPrivate::Denied, 0);
-            }
-            QtAndroidPrivate::releaseAndroidDeadlockProtector();
-            promise->finish();
-        });
-
-        return future;
-    }
-
-    promise->addResult(QtAndroidPrivate::Denied);
-    promise->finish();
-    return future;
+    // avoid the uneccessary call and response to an empty permission string
+    if (permissions.isEmpty())
+        return QtFuture::makeReadyValueFuture(QtAndroidPrivate::Denied);
+    return requestPermissionsInternal(permissions);
 }
 
 /*!
@@ -1244,66 +1168,36 @@ QtAndroidPrivate::requestPermission(QtAndroidPrivate::PermissionType permission)
     Checks whether this process has the named \a permission and returns a QFuture
     representing the result of the check.
 
+    \note QPermission is the recommended API to use for requesting permissions.
+        If QPermission doesn't cover an Android permission you want to request,
+        this preliminary API can still used instead.
+
     \since 6.2
     \sa requestPermission()
 */
 QFuture<QtAndroidPrivate::PermissionResult>
 QtAndroidPrivate::checkPermission(const QString &permission)
 {
-    QPromise<QtAndroidPrivate::PermissionResult> promise;
-    QFuture<QtAndroidPrivate::PermissionResult> future = promise.future();
-    promise.start();
-
-    if (permission.size() > 0) {
-        auto res = QJniObject::callStaticMethod<jint>(qtNativeClassName,
-                                                      "checkSelfPermission",
-                                                      "(Ljava/lang/String;)I",
-                                                      QJniObject::fromString(permission).object());
-        promise.addResult(resultFromAndroid(res));
-    } else {
-        promise.addResult(QtAndroidPrivate::Denied);
+    QtAndroidPrivate::PermissionResult result = Denied;
+    if (!permission.isEmpty()) {
+        auto res = QtNative::callStaticMethod<jint>("checkSelfPermission", permission);
+        result = resultFromAndroid(res);
     }
-
-    promise.finish();
-    return future;
+    return QtFuture::makeReadyValueFuture(result);
 }
 
-/*!
-    \preliminary
-    Checks whether this process has the named \a permission and returns a QFuture
-    representing the result of the check.
-
-    \since 6.2
-    \sa requestPermission()
-*/
-QFuture<QtAndroidPrivate::PermissionResult>
-QtAndroidPrivate::checkPermission(QtAndroidPrivate::PermissionType permission)
-{
-    const auto nativePermissions = nativeStringsFromPermission(permission);
-
-    if (nativePermissions.size() > 0)
-        return checkPermission(nativePermissions.first());
-
-    QPromise<QtAndroidPrivate::PermissionResult> promise;
-    QFuture<QtAndroidPrivate::PermissionResult> future = promise.future();
-    promise.start();
-    promise.addResult(QtAndroidPrivate::Denied);
-    promise.finish();
-    return future;
-}
-
-bool QtAndroidPrivate::registerPermissionNatives()
+bool QtAndroidPrivate::registerPermissionNatives(QJniEnvironment &env)
 {
     if (QtAndroidPrivate::androidSdkVersion() < 23)
         return true;
 
-    const JNINativeMethod methods[] = {
-        {"sendRequestPermissionsResult", "(I[Ljava/lang/String;[I)V",
-         reinterpret_cast<void *>(sendRequestPermissionsResult)
-        }};
-
-    QJniEnvironment env;
-    return env.registerNativeMethods(qtNativeClassName, methods, 1);
+    return env.registerNativeMethods<QtNative>({
+        Q_JNI_NATIVE_METHOD(sendRequestPermissionsResult)
+    });
 }
 
+#endif // QT_CONFIG(permissions)
+
 QT_END_NAMESPACE
+
+#include "moc_qandroidextras_p.cpp"
