@@ -48,8 +48,9 @@ QT_DEFINE_NATIVE_INTERFACE(QAndroidApplication);
 /*!
     \fn QJniObject QNativeInterface::QAndroidApplication::context()
 
-    Returns the Android context as a \c QJniObject. The context is an \c Activity
-    if the main activity object is valid. Otherwise, the context is a \c Service.
+    Returns the Android context as a \c QtJniTypes::Context. The context
+    is an \c Activity if the most recently started activity object is valid.
+    Otherwise, the context is a \c Service.
 
     \since 6.2
 */
@@ -153,7 +154,7 @@ void QNativeInterface::QAndroidApplication::hideSplashScreen(int duration)
     also good to use a \l QDeadlineTimer in your \a runnable to manage
     the execution and make sure it doesn't block the UI thread. Usually,
     any operation longer than 5 seconds might block the app's UI. For more
-    information, see \l {Android: Keeping your app responsive}{Keeping your app responsive}.
+    information, see \l {Android: Keep your app responsive}{Keep your app responsive}.
 
     \since 6.2
 */
