@@ -4,11 +4,8 @@
 
 #include "qfactoryloader_p.h"
 
-#ifndef QT_NO_QOBJECT
 #include "private/qcoreapplication_p.h"
-#include "private/qduplicatetracker_p.h"
 #include "private/qloggingregistry_p.h"
-#include "private/qobject_p.h"
 #include "qcborarray.h"
 #include "qcbormap.h"
 #include "qcborstreamreader.h"
@@ -18,20 +15,11 @@
 #include "qjsonarray.h"
 #include "qjsondocument.h"
 #include "qjsonobject.h"
-#include "qmutex.h"
 #include "qplugin.h"
-#include "qplugin_p.h"
 #include "qpluginloader.h"
 
-#if QT_CONFIG(library)
-#  include "qlibrary_p.h"
-#endif
-
 #include <qtcore_tracepoints_p.h>
 
-#include <map>
-#include <vector>
-
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -249,28 +237,6 @@ QJsonObject QPluginParsedMetaData::toJson() const
     return o;
 }
 
-class QFactoryLoaderPrivate : public QObjectPrivate
-{
-    Q_DECLARE_PUBLIC(QFactoryLoader)
-    Q_DISABLE_COPY_MOVE(QFactoryLoaderPrivate)
-public:
-    QFactoryLoaderPrivate() { }
-    QByteArray iid;
-#if QT_CONFIG(library)
-    ~QFactoryLoaderPrivate();
-    mutable QMutex mutex;
-    QDuplicateTracker<QString> loadedPaths;
-    std::vector<QLibraryPrivate::UniquePtr> libraries;
-    std::map<QString, QLibraryPrivate*> keyMap;
-    QString suffix;
-    QString extraSearchPath;
-    Qt::CaseSensitivity cs;
-    QLibrary::LoadHints loadHints;
-
-    void updateSinglePath(const QString &pluginDir);
-#endif
-};
-
 #if QT_CONFIG(library)
 
 Q_STATIC_LOGGING_CATEGORY_WITH_ENV_OVERRIDE(lcFactoryLoader, "QT_DEBUG_PLUGINS",
@@ -288,10 +254,7 @@ struct QFactoryLoaderGlobals
 
 Q_GLOBAL_STATIC(QFactoryLoaderGlobals, qt_factoryloader_global)
 
-QFactoryLoaderPrivate::~QFactoryLoaderPrivate()
-    = default;
-
-inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
+inline void QFactoryLoader::Private::updateSinglePath(const QString &path)
 {
     struct LibraryReleaser {
         void operator()(QLibraryPrivate *library)
@@ -305,13 +268,17 @@ inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
     qCDebug(lcFactoryLoader) << "checking directory path" << path << "...";
 
     QDirListing plugins(path,
-#if defined(Q_OS_WIN)
+#if defined(Q_OS_WIN) || defined(Q_OS_CYGWIN)
                 QStringList(QStringLiteral("*.dll")),
 #elif defined(Q_OS_ANDROID)
                 QStringList("libplugins_%1_*.so"_L1.arg(suffix)),
 #endif
                 QDirListing::IteratorFlag::FilesOnly | QDirListing::IteratorFlag::ResolveSymlinks);
 
+    auto versionFromLib = [](const QLibraryPrivate *lib) {
+        return lib->metaData.value(QtPluginMetaDataKeys::QtVersion).toInteger();
+    };
+
     for (const auto &dirEntry : plugins) {
         const QString &fileName = dirEntry.fileName();
 #if defined(Q_PROCESSOR_X86)
@@ -350,7 +317,7 @@ inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
             continue;
 
         static constexpr qint64 QtVersionNoPatch = QT_VERSION_CHECK(QT_VERSION_MAJOR, QT_VERSION_MINOR, 0);
-        int thisVersion = library->metaData.value(QtPluginMetaDataKeys::QtVersion).toInteger();
+        qint64 thisVersion = versionFromLib(library.get());
         if (iid.startsWith(QStringLiteral("org.qt-project.Qt.QPA"))) {
             // QPA plugins must match Qt Major.Minor
             if (thisVersion != QtVersionNoPatch) {
@@ -374,7 +341,7 @@ inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
                 // If the existing library was built with a future Qt version,
                 // whereas the one we're considering has a Qt version that fits
                 // better, we prioritize the better match.
-                int existingVersion = existingLibrary->metaData.value(QtPluginMetaDataKeys::QtVersion).toInteger();
+                qint64 existingVersion = versionFromLib(existingLibrary);
                 if (existingVersion == QtVersionNoPatch)
                     continue; // Prefer exact Qt version match
                 if (existingVersion < QtVersionNoPatch && thisVersion > QtVersionNoPatch)
@@ -392,19 +359,18 @@ inline void QFactoryLoaderPrivate::updateSinglePath(const QString &path)
             libraries.push_back(std::move(library));
         }
     };
+
+    loadedLibraries.resize(libraries.size());
 }
 
 void QFactoryLoader::setLoadHints(QLibrary::LoadHints loadHints)
 {
-    Q_D(QFactoryLoader);
     d->loadHints = loadHints;
 }
 
 void QFactoryLoader::update()
 {
 #ifdef QT_SHARED
-    Q_D(QFactoryLoader);
-
     if (!d->extraSearchPath.isEmpty())
         d->updateSinglePath(d->extraSearchPath);
 
@@ -418,7 +384,6 @@ void QFactoryLoader::update()
         d->updateSinglePath(path);
     }
 #else
-    Q_D(QFactoryLoader);
     qCDebug(lcFactoryLoader) << "ignoring" << d->iid
                              << "since plugins are disabled in static builds";
 #endif
@@ -430,12 +395,26 @@ QFactoryLoader::~QFactoryLoader()
         QMutexLocker locker(&qt_factoryloader_global->mutex);
         qt_factoryloader_global->loaders.removeOne(this);
     }
+
+#if QT_CONFIG(library)
+    for (qsizetype i = 0; i < d->loadedLibraries.size(); ++i) {
+        if (d->loadedLibraries.at(i)) {
+            auto &plugin = d->libraries[i];
+            delete plugin->inst.data();
+            plugin->unload();
+        }
+    }
+#endif
+
+    for (QtPluginInstanceFunction staticInstance : d->usedStaticInstances) {
+        if (staticInstance)
+            delete staticInstance();
+    }
 }
 
 #if defined(Q_OS_UNIX) && !defined (Q_OS_DARWIN)
 QLibraryPrivate *QFactoryLoader::library(const QString &key) const
 {
-    Q_D(const QFactoryLoader);
     const auto it = d->keyMap.find(d->cs ? key : key.toLower());
     if (it == d->keyMap.cend())
         return nullptr;
@@ -457,13 +436,10 @@ void QFactoryLoader::refreshAll()
 QFactoryLoader::QFactoryLoader(const char *iid,
                                const QString &suffix,
                                Qt::CaseSensitivity cs)
-    : QObject(*new QFactoryLoaderPrivate)
 {
     Q_ASSERT_X(suffix.startsWith(u'/'), "QFactoryLoader",
                "For historical reasons, the suffix must start with '/' (and it can't be empty)");
 
-    moveToThread(QCoreApplicationPrivate::mainThread());
-    Q_D(QFactoryLoader);
     d->iid = iid;
 #if QT_CONFIG(library)
     d->cs = cs;
@@ -485,7 +461,6 @@ QFactoryLoader::QFactoryLoader(const char *iid,
 void QFactoryLoader::setExtraSearchPath(const QString &path)
 {
 #if QT_CONFIG(library)
-    Q_D(QFactoryLoader);
     if (d->extraSearchPath == path)
         return;             // nothing to do
 
@@ -496,6 +471,13 @@ void QFactoryLoader::setExtraSearchPath(const QString &path)
         d->updateSinglePath(d->extraSearchPath);
     } else {
         // must re-scan everything
+        for (qsizetype i = 0; i < d->loadedLibraries.size(); ++i) {
+            if (d->loadedLibraries.at(i)) {
+                auto &plugin = d->libraries[i];
+                delete plugin->inst.data();
+            }
+        }
+        d->loadedLibraries.fill(false);
         d->loadedPaths.clear();
         d->libraries.clear();
         d->keyMap.clear();
@@ -508,7 +490,6 @@ void QFactoryLoader::setExtraSearchPath(const QString &path)
 
 QFactoryLoader::MetaDataList QFactoryLoader::metaData() const
 {
-    Q_D(const QFactoryLoader);
     QList<QPluginParsedMetaData> metaData;
 #if QT_CONFIG(library)
     QMutexLocker locker(&d->mutex);
@@ -535,7 +516,6 @@ QFactoryLoader::MetaDataList QFactoryLoader::metaData() const
 
 QList<QCborArray> QFactoryLoader::metaDataKeys() const
 {
-    Q_D(const QFactoryLoader);
     QList<QCborArray> metaData;
 #if QT_CONFIG(library)
     QMutexLocker locker(&d->mutex);
@@ -565,37 +545,45 @@ QList<QCborArray> QFactoryLoader::metaDataKeys() const
 
 QObject *QFactoryLoader::instance(int index) const
 {
-    Q_D(const QFactoryLoader);
     if (index < 0)
         return nullptr;
 
-#if QT_CONFIG(library)
     QMutexLocker lock(&d->mutex);
+    QObject *obj = instanceHelper_locked(index);
+
+    if (obj && !obj->parent())
+        obj->moveToThread(QCoreApplicationPrivate::mainThread());
+    return obj;
+}
+
+inline QObject *QFactoryLoader::instanceHelper_locked(int index) const
+{
+#if QT_CONFIG(library)
     if (size_t(index) < d->libraries.size()) {
         QLibraryPrivate *library = d->libraries[index].get();
+        d->loadedLibraries[index] = true;
         library->setLoadHints(d->loadHints);
-        if (QObject *obj = library->pluginInstance()) {
-            if (!obj->parent())
-                obj->moveToThread(QCoreApplicationPrivate::mainThread());
-            return obj;
-        }
-        return nullptr;
+        return library->pluginInstance();
     }
     // we know d->libraries.size() <= index <= numeric_limits<decltype(index)>::max() → no overflow
     index -= static_cast<int>(d->libraries.size());
-    lock.unlock();
 #endif
 
     QLatin1StringView iid(d->iid.constData(), d->iid.size());
     const QList<QStaticPlugin> staticPlugins = QPluginLoader::staticPlugins();
+    qsizetype i = 0;
     for (QStaticPlugin plugin : staticPlugins) {
         QByteArrayView pluginData(static_cast<const char *>(plugin.rawMetaData), plugin.rawMetaDataSize);
         if (!isIidMatch(pluginData, iid))
             continue;
 
-        if (index == 0)
+        if (i == index) {
+            if (d->usedStaticInstances.size() <= i)
+                d->usedStaticInstances.resize(i + 1);
+            d->usedStaticInstances[i] = plugin.instance;
             return plugin.instance();
-        --index;
+        }
+        ++i;
     }
 
     return nullptr;
@@ -627,7 +615,3 @@ int QFactoryLoader::indexOf(const QString &needle) const
 }
 
 QT_END_NAMESPACE
-
-#include "moc_qfactoryloader_p.cpp"
-
-#endif // QT_NO_QOBJECT
