@@ -1,5 +1,6 @@
 // Copyright (C) 2018 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qcborarray.h"
 #include "qcborvalue_p.h"
@@ -97,6 +98,17 @@ QCborArray::QCborArray(const QCborArray &other) noexcept
 {
 }
 
+/*!
+    \fn QCborArray::QCborArray(QCborArray &&other)
+    \since 6.10
+
+    Move-constructor.
+
+    The moved-from object \a other is placed in the default-constructed state.
+
+    \sa QCborArray::QCborArray()
+*/
+
 /*!
     \fn QCborArray::QCborArray(std::initializer_list<QCborValue> args)
 
@@ -126,9 +138,17 @@ QCborArray &QCborArray::operator=(const QCborArray &other) noexcept
 }
 
 /*!
-    \fn void QCborArray::swap(QCborArray &other)
+    \fn QCborArray &QCborArray::operator=(QCborArray &&other)
+    \since 6.10
 
-    Swaps the contents of this object and \a other.
+    Move-assignment operator.
+
+    The moved-from object \a other is placed in a valid, but unspecified state.
+*/
+
+/*!
+    \fn void QCborArray::swap(QCborArray &other)
+    \memberswap{array}
  */
 
 /*!
