@@ -29,7 +29,7 @@ static QTimeZonePrivate *newBackendTimeZone()
     return new QMacTimeZonePrivate();
 #elif defined(Q_OS_ANDROID)
     return new QAndroidTimeZonePrivate();
-#elif defined(Q_OS_UNIX)
+#elif defined(Q_OS_UNIX) && !defined(Q_OS_VXWORKS)
     return new QTzTimeZonePrivate();
 #elif QT_CONFIG(icu)
     return new QIcuTimeZonePrivate();
@@ -50,7 +50,7 @@ static QTimeZonePrivate *newBackendTimeZone(const QByteArray &ianaId)
     return new QMacTimeZonePrivate(ianaId);
 #elif defined(Q_OS_ANDROID)
     return new QAndroidTimeZonePrivate(ianaId);
-#elif defined(Q_OS_UNIX)
+#elif defined(Q_OS_UNIX) && !defined(Q_OS_VXWORKS)
     return new QTzTimeZonePrivate(ianaId);
 #elif QT_CONFIG(icu)
     return new QIcuTimeZonePrivate(ianaId);
@@ -1444,6 +1444,7 @@ QTimeZone QTimeZone::systemTimeZone()
 }
 
 /*!
+    \fn QTimeZone QTimeZone::utc()
     \since 5.5
     Returns a QTimeZone object that describes UTC as a time zone.
 
@@ -1454,9 +1455,18 @@ QTimeZone QTimeZone::systemTimeZone()
 
     \sa systemTimeZone(), Initialization, asBackendZone()
 */
+QTimeZone QTimeZonePrivate::utcQTimeZone()
+{
+    return QTimeZone(*new QUtcTimeZonePrivate());
+}
+
+Q_GLOBAL_STATIC(QTimeZone, utcTimeZone, QTimeZonePrivate::utcQTimeZone());
+
 QTimeZone QTimeZone::utc()
 {
-    return QTimeZone(0);
+    if (Q_UNLIKELY(utcTimeZone.isDestroyed()))
+        return QTimeZonePrivate::utcQTimeZone(); // create a new, unshared one
+    return *utcTimeZone; // take a shallow copy
 }
 
 /*!
@@ -1472,7 +1482,8 @@ QTimeZone QTimeZone::utc()
 
 bool QTimeZone::isTimeZoneIdAvailable(const QByteArray &ianaId)
 {
-#if defined(Q_OS_UNIX) && !(defined(Q_OS_ANDROID) || defined(Q_OS_DARWIN))
+#if defined(Q_OS_UNIX) && !(QT_CONFIG(timezone_tzdb) || defined(Q_OS_DARWIN) \
+                            || defined(Q_OS_ANDROID) || defined(Q_OS_VXWORKS))
     // Keep #if-ery consistent with selection of QTzTimeZonePrivate in
     // newBackendTimeZone(). Skip the pre-check, as the TZ backend accepts POSIX
     // zone IDs, which need not be valid IANA IDs. See also QTBUG-112006.
