@@ -284,8 +284,9 @@ qreal QScreen::logicalDotsPerInch() const
 
     Returns the ratio between physical pixels and device-independent pixels for the screen.
 
-    Common values are 1.0 on normal displays and 2.0 on "retina" displays.
-    Higher values are also possible.
+    This function may return a value that differs from QWindow::devicePixelRatio(),
+    for instance on Wayland when using fractional scaling, or if window properties
+    that affect surface resolution are set. Prefer using QWindow::devicePixelRatio().
 
     \note On some platforms the devicePixelRatio of a window and the screen it is on can
     be different. Use this function only when you don't know which window you are targeting.
