@@ -96,7 +96,7 @@ QT_BEGIN_NAMESPACE
     \sa activated()
 */
 
-static bool simpleContextMatcher(QObject *object, Qt::ShortcutContext context)
+bool QShortcutPrivate::simpleContextMatcher(QObject *object, Qt::ShortcutContext context)
 {
     auto guiShortcut = qobject_cast<QShortcut *>(object);
     if (QGuiApplication::applicationState() != Qt::ApplicationActive || guiShortcut == nullptr)
@@ -127,14 +127,14 @@ void QShortcutPrivate::redoGrab(QShortcutMap &map)
         return;
     }
 
-    for (int id : qAsConst(sc_ids))
+    for (int id : std::as_const(sc_ids))
         map.removeShortcut(id, q);
 
     sc_ids.clear();
     if (sc_sequences.isEmpty())
         return;
-    sc_ids.reserve(sc_sequences.count());
-    for (const auto &keySequence : qAsConst(sc_sequences)) {
+    sc_ids.reserve(sc_sequences.size());
+    for (const auto &keySequence : std::as_const(sc_sequences)) {
         if (keySequence.isEmpty())
             continue;
         int id = map.addShortcut(q, keySequence, sc_context, contextMatcher());
@@ -337,7 +337,7 @@ QShortcut::~QShortcut()
 {
     Q_D(QShortcut);
     if (qApp) {
-         for (int id : qAsConst(d->sc_ids))
+         for (int id : std::as_const(d->sc_ids))
              QGuiApplicationPrivate::instance()->shortcutMap.removeShortcut(id, this);
     }
 }
@@ -536,6 +536,8 @@ QString QShortcut::whatsThis() const
 /*!
     Returns the primary key binding's ID.
 
+    \deprecated
+
     \sa QShortcutEvent::shortcutId()
 */
 int QShortcut::id() const
@@ -575,4 +577,6 @@ bool QShortcut::event(QEvent *e)
 
 QT_END_NAMESPACE
 
+#undef QAPP_CHECK
+
 #include "moc_qshortcut.cpp"
