@@ -4,6 +4,8 @@
 #include "qmatrix4x4.h"
 #include <QtCore/qmath.h>
 #include <QtCore/qvariant.h>
+
+#include <QtGui/qquaternion.h>
 #include <QtGui/qtransform.h>
 
 #include <cmath>
@@ -729,8 +731,7 @@ QMatrix4x4 operator/(const QMatrix4x4& matrix, float divisor)
 }
 
 /*!
-    \fn bool qFuzzyCompare(const QMatrix4x4& m1, const QMatrix4x4& m2)
-    \relates QMatrix4x4
+    \fn bool QMatrix4x4::qFuzzyCompare(const QMatrix4x4& m1, const QMatrix4x4& m2)
 
     Returns \c true if \a m1 and \a m2 are equal, allowing for a small
     fuzziness factor for floating-point comparisons; false otherwise.
@@ -1280,7 +1281,7 @@ void QMatrix4x4::projectedRotate(float angle, float x, float y, float z)
 */
 
 /*!
-    \enum QMatrix4x4::Flags
+    \enum QMatrix4x4::Flag
     \internal
     \omitvalue Identity
     \omitvalue Translation
