@@ -180,6 +180,17 @@ struct QD3D11TextureRenderTarget : public QRhiTextureRenderTarget
     friend class QRhiD3D11;
 };
 
+struct RenderTargetUavUpdateState
+{
+    ID3D11RenderTargetView *rtv[QD3D11RenderTargetData::MAX_COLOR_ATTACHMENTS];
+    ID3D11DepthStencilView *dsv = nullptr;
+    std::array<ID3D11UnorderedAccessView *, QD3D11RenderTargetData::MAX_COLOR_ATTACHMENTS> uav;
+    bool update(QD3D11RenderTargetData *data, ID3D11UnorderedAccessView * const *uavs = nullptr, int count = 0);
+};
+
+struct QD3D11GraphicsPipeline;
+struct QD3D11ComputePipeline;
+
 struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
 {
     QD3D11ShaderResourceBindings(QRhiImplementation *rhi);
@@ -191,6 +202,8 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
     bool hasDynamicOffset = false;
     QVarLengthArray<QRhiShaderResourceBinding, 8> sortedBindings;
     uint generation = 0;
+    QD3D11GraphicsPipeline *lastUsedGraphicsPipeline = nullptr;
+    QD3D11ComputePipeline *lastUsedComputePipeline = nullptr;
 
     // Keep track of the generation number of each referenced QRhi* to be able
     // to detect that the batched bindings are out of date.
@@ -270,21 +283,43 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
         }
     };
 
-    StageUniformBufferBatches vsUniformBufferBatches;
-    StageUniformBufferBatches hsUniformBufferBatches;
-    StageUniformBufferBatches dsUniformBufferBatches;
-    StageUniformBufferBatches gsUniformBufferBatches;
-    StageUniformBufferBatches fsUniformBufferBatches;
-    StageUniformBufferBatches csUniformBufferBatches;
+    struct ResourceBatches {
+        StageUniformBufferBatches vsUniformBufferBatches;
+        StageUniformBufferBatches hsUniformBufferBatches;
+        StageUniformBufferBatches dsUniformBufferBatches;
+        StageUniformBufferBatches gsUniformBufferBatches;
+        StageUniformBufferBatches fsUniformBufferBatches;
+        StageUniformBufferBatches csUniformBufferBatches;
 
-    StageSamplerBatches vsSamplerBatches;
-    StageSamplerBatches hsSamplerBatches;
-    StageSamplerBatches dsSamplerBatches;
-    StageSamplerBatches gsSamplerBatches;
-    StageSamplerBatches fsSamplerBatches;
-    StageSamplerBatches csSamplerBatches;
+        StageSamplerBatches vsSamplerBatches;
+        StageSamplerBatches hsSamplerBatches;
+        StageSamplerBatches dsSamplerBatches;
+        StageSamplerBatches gsSamplerBatches;
+        StageSamplerBatches fsSamplerBatches;
+        StageSamplerBatches csSamplerBatches;
 
-    StageUavBatches csUavBatches;
+        StageUavBatches csUavBatches;
+        StageUavBatches fsUavBatches;
+
+        void clear() {
+            vsUniformBufferBatches.clear();
+            hsUniformBufferBatches.clear();
+            dsUniformBufferBatches.clear();
+            gsUniformBufferBatches.clear();
+            fsUniformBufferBatches.clear();
+            csUniformBufferBatches.clear();
+
+            vsSamplerBatches.clear();
+            hsSamplerBatches.clear();
+            dsSamplerBatches.clear();
+            gsSamplerBatches.clear();
+            fsSamplerBatches.clear();
+            csSamplerBatches.clear();
+
+            csUavBatches.clear();
+            fsUavBatches.clear();
+        }
+    } resourceBatches;
 
     friend class QRhiD3D11;
 };
@@ -356,6 +391,8 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
 
     struct Command {
         enum Cmd {
+            BeginFrame,
+            EndFrame,
             ResetShaderResources,
             SetRenderTarget,
             Clear,
@@ -385,6 +422,15 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
         // QRhi*/QD3D11* references should be kept at minimum (so no
         // QRhiTexture/Buffer/etc. pointers).
         union Args {
+            struct {
+                ID3D11Query *tsQuery;
+                ID3D11Query *tsDisjointQuery;
+                QD3D11RenderTargetData *swapchainData;
+            } beginFrame;
+            struct {
+                ID3D11Query *tsQuery;
+                ID3D11Query *tsDisjointQuery;
+            } endFrame;
             struct {
                 QRhiRenderTarget *rt;
             } setRenderTarget;
@@ -418,7 +464,7 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
                 QD3D11GraphicsPipeline *ps;
             } bindGraphicsPipeline;
             struct {
-                QD3D11ShaderResourceBindings *srb;
+                int resourceBatchesIndex;
                 bool offsetOnlyChange;
                 int dynamicOffsetCount;
                 uint dynamicOffsetPairs[MAX_DYNAMIC_OFFSET_COUNT * 2]; // binding, offsetInConstants
@@ -510,10 +556,12 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
     DXGI_FORMAT currentIndexFormat;
     ID3D11Buffer *currentVertexBuffers[D3D11_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT];
     quint32 currentVertexOffsets[D3D11_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT];
+    QD3D11RenderTargetData *prevRtD;
 
     QVarLengthArray<QByteArray, 4> dataRetainPool;
     QVarLengthArray<QRhiBufferData, 4> bufferDataRetainPool;
     QVarLengthArray<QImage, 4> imageRetainPool;
+    QVarLengthArray<QD3D11ShaderResourceBindings::ResourceBatches, 4> resourceBatchRetainPool;
 
     // relies heavily on implicit sharing (no copies of the actual data will be made)
     const uchar *retainData(const QByteArray &data) {
@@ -528,16 +576,23 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
         imageRetainPool.append(image);
         return imageRetainPool.last().constBits();
     }
+    // except this one
+    int retainResourceBatches(const QD3D11ShaderResourceBindings::ResourceBatches &resourceBatches) {
+        resourceBatchRetainPool.append(resourceBatches);
+        return resourceBatchRetainPool.count() - 1;
+    }
     void resetCommands() {
         commands.reset();
         dataRetainPool.clear();
         bufferDataRetainPool.clear();
         imageRetainPool.clear();
+        resourceBatchRetainPool.clear();
     }
     void resetState() {
         recordingPass = NoPass;
         // do not zero lastGpuTime
         currentTarget = nullptr;
+        prevRtD = nullptr;
         resetCommands();
         resetCachedState();
     }
@@ -556,17 +611,15 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
     }
 };
 
-static const int QD3D11_SWAPCHAIN_BUFFER_COUNT = 2;
-
-struct QD3D11Timestamps
+struct QD3D11SwapChainTimestamps
 {
-    static const int MAX_TIMESTAMP_PAIRS = QD3D11_SWAPCHAIN_BUFFER_COUNT;
-    bool active[MAX_TIMESTAMP_PAIRS] = {};
-    ID3D11Query *disjointQuery[MAX_TIMESTAMP_PAIRS] = {};
-    ID3D11Query *query[MAX_TIMESTAMP_PAIRS * 2] = {};
-    int pairCount = 0;
+    static const int TIMESTAMP_PAIRS = 2;
 
-    bool prepare(int pairCount, QRhiD3D11 *rhiD);
+    bool active[TIMESTAMP_PAIRS] = {};
+    ID3D11Query *disjointQuery[TIMESTAMP_PAIRS] = {};
+    ID3D11Query *query[TIMESTAMP_PAIRS * 2] = {};
+
+    bool prepare(QRhiD3D11 *rhiD);
     void destroy();
     bool tryQueryTimestamps(int idx, ID3D11DeviceContext *context, double *elapsedSec);
 };
@@ -579,6 +632,7 @@ struct QD3D11SwapChain : public QRhiSwapChain
 
     QRhiCommandBuffer *currentFrameCommandBuffer() override;
     QRhiRenderTarget *currentFrameRenderTarget() override;
+    QRhiRenderTarget *currentFrameRenderTarget(StereoTargetBuffer targetBuffer) override;
 
     QSize surfacePixelSize() override;
     bool isFormatSupported(Format f) override;
@@ -594,6 +648,7 @@ struct QD3D11SwapChain : public QRhiSwapChain
     QWindow *window = nullptr;
     QSize pixelSize;
     QD3D11SwapChainRenderTarget rt;
+    QD3D11SwapChainRenderTarget rtRight;
     QD3D11CommandBuffer cb;
     DXGI_FORMAT colorFormat;
     DXGI_FORMAT srgbAdjustedColorFormat;
@@ -601,7 +656,8 @@ struct QD3D11SwapChain : public QRhiSwapChain
     UINT swapChainFlags = 0;
     ID3D11Texture2D *backBufferTex;
     ID3D11RenderTargetView *backBufferRtv;
-    static const int BUFFER_COUNT = QD3D11_SWAPCHAIN_BUFFER_COUNT;
+    ID3D11RenderTargetView *backBufferRtvRight = nullptr;
+    static const int BUFFER_COUNT = 2;
     ID3D11Texture2D *msaaTex[BUFFER_COUNT];
     ID3D11RenderTargetView *msaaRtv[BUFFER_COUNT];
     DXGI_SAMPLE_DESC sampleDesc;
@@ -611,7 +667,19 @@ struct QD3D11SwapChain : public QRhiSwapChain
     UINT swapInterval = 1;
     IDCompositionTarget *dcompTarget = nullptr;
     IDCompositionVisual *dcompVisual = nullptr;
-    QD3D11Timestamps timestamps;
+    QD3D11SwapChainTimestamps timestamps;
+    int currentTimestampPairIndex = 0;
+    HANDLE frameLatencyWaitableObject = nullptr;
+    int lastFrameLatencyWaitSlot = -1;
+};
+
+class QD3D11Adapter : public QRhiAdapter
+{
+public:
+    QRhiDriverInfo info() const override;
+
+    LUID luid;
+    QRhiDriverInfo adapterInfo;
 };
 
 class QRhiD3D11 : public QRhiImplementation
@@ -621,6 +689,7 @@ public:
 
     bool create(QRhi::Flags flags) override;
     void destroy() override;
+    QRhi::AdapterList enumerateAdaptersBeforeCreate(QRhiNativeHandles *nativeHandles) const override;
 
     QRhiGraphicsPipeline *createGraphicsPipeline() override;
     QRhiComputePipeline *createComputePipeline() override;
@@ -649,6 +718,8 @@ public:
     QRhiTextureRenderTarget *createTextureRenderTarget(const QRhiTextureRenderTargetDescription &desc,
                                                        QRhiTextureRenderTarget::Flags flags) override;
 
+    QRhiShadingRateMap *createShadingRateMap() override;
+
     QRhiSwapChain *createSwapChain() override;
     QRhi::FrameOpResult beginFrame(QRhiSwapChain *swapChain, QRhi::BeginFrameFlags flags) override;
     QRhi::FrameOpResult endFrame(QRhiSwapChain *swapChain, QRhi::EndFrameFlags flags) override;
@@ -683,6 +754,7 @@ public:
     void setScissor(QRhiCommandBuffer *cb, const QRhiScissor &scissor) override;
     void setBlendConstants(QRhiCommandBuffer *cb, const QColor &c) override;
     void setStencilRef(QRhiCommandBuffer *cb, quint32 refValue) override;
+    void setShadingRate(QRhiCommandBuffer *cb, const QSize &coarsePixelSize) override;
 
     void draw(QRhiCommandBuffer *cb, quint32 vertexCount,
               quint32 instanceCount, quint32 firstVertex, quint32 firstInstance) override;
@@ -708,6 +780,7 @@ public:
     double lastCompletedGpuTime(QRhiCommandBuffer *cb) override;
 
     QList<int> supportedSampleCounts() const override;
+    QList<QSize> supportedShadingRates(int sampleCount) const override;
     int ubufAlignment() const override;
     bool isYUpInFramebuffer() const override;
     bool isYUpInNDC() const override;
@@ -720,6 +793,7 @@ public:
     QRhiDriverInfo driverInfo() const override;
     QRhiStats statistics() override;
     bool makeThreadLocalNativeContextCurrent() override;
+    void setQueueSubmitParams(QRhiNativeHandles *params) override;
     void releaseCachedResources() override;
     bool isDeviceLost() const override;
 
@@ -732,11 +806,12 @@ public:
     void updateShaderResourceBindings(QD3D11ShaderResourceBindings *srbD,
                                       const QShader::NativeResourceBindingMap *nativeResourceBindingMaps[]);
     void executeBufferHostWrites(QD3D11Buffer *bufD);
-    void bindShaderResources(QD3D11ShaderResourceBindings *srbD,
+
+    void bindShaderResources(const QD3D11ShaderResourceBindings::ResourceBatches &allResourceBatches,
                              const uint *dynOfsPairs, int dynOfsPairCount,
-                             bool offsetOnlyChange);
-    void resetShaderResources();
-    void executeCommandBuffer(QD3D11CommandBuffer *cbD, QD3D11SwapChain *timestampSwapChain = nullptr);
+                             bool offsetOnlyChange, QD3D11RenderTargetData *rtD, RenderTargetUavUpdateState &rtUavState);
+    void resetShaderResources(QD3D11RenderTargetData *rtD, RenderTargetUavUpdateState &rtUavState);
+    void executeCommandBuffer(QD3D11CommandBuffer *cbD);
     DXGI_SAMPLE_DESC effectiveSampleDesc(int sampleCount) const;
     void finishActiveReadbacks();
     void reportLiveObjects(ID3D11Device *device);
@@ -747,6 +822,7 @@ public:
 
     QRhi::Flags rhiFlags;
     bool debugLayer = false;
+    UINT maxFrameLatency = 2; // 1-3, use 2 to keep CPU-GPU parallelism while reducing lag compared to tripple buffering
     bool importedDeviceAndContext = false;
     ID3D11Device *dev = nullptr;
     ID3D11DeviceContext1 *context = nullptr;
@@ -772,6 +848,7 @@ public:
         int fsHighestActiveSrvBinding = -1;
         int csHighestActiveSrvBinding = -1;
         int csHighestActiveUavBinding = -1;
+        int fsHighestActiveUavBinding = -1;
         QD3D11SwapChain *currentSwapChain = nullptr;
     } contextState;
 
@@ -779,8 +856,8 @@ public:
         OffscreenFrame(QRhiImplementation *rhi) : cbWrapper(rhi) { }
         bool active = false;
         QD3D11CommandBuffer cbWrapper;
-        QD3D11Timestamps timestamps;
-        int timestampIdx = 0;
+        ID3D11Query *tsQueries[2] = {};
+        ID3D11Query *tsDisjointQuery = nullptr;
     } ofr;
 
     struct TextureReadback {
@@ -845,7 +922,7 @@ inline bool operator!=(const QRhiD3D11::BytecodeCacheKey &a, const QRhiD3D11::By
 
 inline size_t qHash(const QRhiD3D11::BytecodeCacheKey &k, size_t seed = 0) noexcept
 {
-    return qHash(k.sourceHash, seed) ^ qHash(k.target) ^ qHash(k.entryPoint) ^ k.compileFlags;
+    return qHashMulti(seed, k.sourceHash, k.target, k.entryPoint, k.compileFlags);
 }
 
 QT_END_NAMESPACE
