@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qnetworkcookie.h"
 #include "qnetworkcookie_p.h"
@@ -7,16 +8,20 @@
 #include "qnetworkrequest.h"
 #include "qnetworkreply.h"
 #include "QtCore/qbytearray.h"
+#include "QtCore/qdatetime.h"
 #include "QtCore/qdebug.h"
 #include "QtCore/qlist.h"
 #include "QtCore/qlocale.h"
 #include <QtCore/qregularexpression.h>
 #include "QtCore/qstring.h"
 #include "QtCore/qstringlist.h"
+#include "QtCore/qtimezone.h"
 #include "QtCore/qurl.h"
 #include "QtNetwork/qhostaddress.h"
 #include "private/qobject_p.h"
 
+#include <utility>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -115,9 +120,7 @@ QNetworkCookie &QNetworkCookie::operator=(const QNetworkCookie &other)
 /*!
     \fn void QNetworkCookie::swap(QNetworkCookie &other)
     \since 5.0
-
-    Swaps this cookie with \a other. This function is very fast and
-    never fails.
+    \memberswap{cookie}
 */
 
 /*!
@@ -374,7 +377,7 @@ void QNetworkCookie::setValue(const QByteArray &value)
 }
 
 // ### move this to qnetworkcookie_p.h and share with qnetworkaccesshttpbackend
-static QPair<QByteArray, QByteArray> nextField(const QByteArray &text, int &position, bool isNameValue)
+static std::pair<QByteArray, QByteArray> nextField(QByteArrayView text, int &position, bool isNameValue)
 {
     // format is one of:
     //    (1)  token
@@ -390,18 +393,18 @@ static QPair<QByteArray, QByteArray> nextField(const QByteArray &text, int &posi
     int equalsPosition = text.indexOf('=', position);
     if (equalsPosition < 0 || equalsPosition > semiColonPosition) {
         if (isNameValue)
-            return qMakePair(QByteArray(), QByteArray()); //'=' is required for name-value-pair (RFC6265 section 5.2, rule 2)
+            return std::pair(QByteArray(), QByteArray()); //'=' is required for name-value-pair (RFC6265 section 5.2, rule 2)
         equalsPosition = semiColonPosition; //no '=' means there is an attribute-name but no attribute-value
     }
 
-    QByteArray first = text.mid(position, equalsPosition - position).trimmed();
+    QByteArray first = text.mid(position, equalsPosition - position).trimmed().toByteArray();
     QByteArray second;
     int secondLength = semiColonPosition - equalsPosition - 1;
     if (secondLength > 0)
-        second = text.mid(equalsPosition + 1, secondLength).trimmed();
+        second = text.mid(equalsPosition + 1, secondLength).trimmed().toByteArray();
 
     position = semiColonPosition;
-    return qMakePair(first, second);
+    return std::pair(first, second);
 }
 
 /*!
@@ -441,29 +444,33 @@ static QPair<QByteArray, QByteArray> nextField(const QByteArray &text, int &posi
 */
 
 namespace {
-QByteArray sameSiteToRawString(QNetworkCookie::SameSite samesite)
+
+constexpr QByteArrayView sameSiteNone() noexcept { return "None"; }
+constexpr QByteArrayView sameSiteLax() noexcept { return "Lax"; }
+constexpr QByteArrayView sameSiteStrict() noexcept { return "Strict"; }
+
+QByteArrayView sameSiteToRawString(QNetworkCookie::SameSite samesite) noexcept
 {
     switch (samesite) {
     case QNetworkCookie::SameSite::None:
-        return QByteArrayLiteral("None");
+        return sameSiteNone();
     case QNetworkCookie::SameSite::Lax:
-        return QByteArrayLiteral("Lax");
+        return sameSiteLax();
     case QNetworkCookie::SameSite::Strict:
-        return QByteArrayLiteral("Strict");
+        return sameSiteStrict();
     case QNetworkCookie::SameSite::Default:
         break;
     }
-    return QByteArray();
+    return QByteArrayView();
 }
 
-QNetworkCookie::SameSite sameSiteFromRawString(QByteArray str)
+QNetworkCookie::SameSite sameSiteFromRawString(QByteArrayView str) noexcept
 {
-    str = str.toLower();
-    if (str == QByteArrayLiteral("none"))
+    if (str.compare(sameSiteNone(), Qt::CaseInsensitive) == 0)
         return QNetworkCookie::SameSite::None;
-    if (str == QByteArrayLiteral("lax"))
+    if (str.compare(sameSiteLax(), Qt::CaseInsensitive) == 0)
         return QNetworkCookie::SameSite::Lax;
-    if (str == QByteArrayLiteral("strict"))
+    if (str.compare(sameSiteStrict(), Qt::CaseInsensitive) == 0)
         return QNetworkCookie::SameSite::Strict;
     return QNetworkCookie::SameSite::Default;
 }
@@ -574,7 +581,7 @@ static inline bool isValueSeparator(char c)
 static inline bool isWhitespace(char c)
 { return c == ' '  || c == '\t'; }
 
-static bool checkStaticArray(int &val, const QByteArray &dateString, int at, const char *array, int size)
+static bool checkStaticArray(int &val, QByteArrayView dateString, int at, const char *array, int size)
 {
     if (dateString[at] < 'a' || dateString[at] > 'z')
         return false;
@@ -621,7 +628,7 @@ static bool checkStaticArray(int &val, const QByteArray &dateString, int at, con
     Or in their own words:
         "} // else what the hell is this."
 */
-static QDateTime parseDateString(const QByteArray &dateString)
+static QDateTime parseDateString(QByteArrayView dateString)
 {
     QTime time;
     // placeholders for values when we are not sure it is a year, month or day
@@ -684,13 +691,13 @@ static QDateTime parseDateString(const QByteArray &dateString)
             int hours = 0;
             switch (end - 1) {
             case 4:
-                minutes = atoi(dateString.mid(at + 3, 2).constData());
+                minutes = dateString.mid(at + 3, 2).toInt();
                 Q_FALLTHROUGH();
             case 2:
-                hours = atoi(dateString.mid(at + 1, 2).constData());
+                hours = dateString.mid(at + 1, 2).toInt();
                 break;
             case 1:
-                hours = atoi(dateString.mid(at + 1, 1).constData());
+                hours = dateString.mid(at + 1, 1).toInt();
                 break;
             default:
                 at += end;
@@ -741,7 +748,7 @@ static QDateTime parseDateString(const QByteArray &dateString)
             if (isNumber(dateString[at + 1])
                 && isNumber(dateString[at + 2])
                 && isNumber(dateString[at + 3])) {
-                year = atoi(dateString.mid(at, 4).constData());
+                year = dateString.mid(at, 4).toInt();
                 at += 4;
 #ifdef PARSEDATESTRINGDEBUG
                 qDebug() << "Year:" << year;
@@ -757,7 +764,7 @@ static QDateTime parseDateString(const QByteArray &dateString)
             if (dateString.size() > at + 1
                 && isNumber(dateString[at + 1]))
                 ++length;
-            int x = atoi(dateString.mid(at, length).constData());
+            int x = dateString.mid(at, length).toInt();
             if (year == -1 && (x > 31 || x == 0)) {
                 year = x;
             } else {
@@ -906,11 +913,11 @@ static QDateTime parseDateString(const QByteArray &dateString)
     if (!date.isValid())
         date = QDate(day + y2k, month, year);
 
-    QDateTime dateTime(date, time, Qt::UTC);
+    QDateTime dateTime(date, time, QTimeZone::UTC);
 
-    if (zoneOffset != -1) {
+    if (zoneOffset != -1)
         dateTime = dateTime.addSecs(zoneOffset);
-    }
+
     if (!dateTime.isValid())
         return QDateTime();
     return dateTime;
@@ -926,19 +933,19 @@ static QDateTime parseDateString(const QByteArray &dateString)
     cookie that is parsed.
 
     \sa toRawForm()
+    \note In Qt versions prior to 6.7, this function took QByteArray only.
 */
-QList<QNetworkCookie> QNetworkCookie::parseCookies(const QByteArray &cookieString)
+QList<QNetworkCookie> QNetworkCookie::parseCookies(QByteArrayView cookieString)
 {
     // cookieString can be a number of set-cookie header strings joined together
     // by \n, parse each line separately.
     QList<QNetworkCookie> cookies;
-    QList<QByteArray> list = cookieString.split('\n');
-    for (int a = 0; a < list.size(); a++)
-        cookies += QNetworkCookiePrivate::parseSetCookieHeaderLine(list.at(a));
+    for (auto s : QLatin1StringView(cookieString).tokenize('\n'_L1))
+        cookies += QNetworkCookiePrivate::parseSetCookieHeaderLine(s);
     return cookies;
 }
 
-QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByteArray &cookieString)
+QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(QByteArrayView cookieString)
 {
     // According to http://wp.netscape.com/newsref/std/cookie_spec.html,<
     // the Set-Cookie response header is of the format:
@@ -958,7 +965,7 @@ QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByt
         QNetworkCookie cookie;
 
         // The first part is always the "NAME=VALUE" part
-        QPair<QByteArray,QByteArray> field = nextField(cookieString, position, true);
+        std::pair<QByteArray,QByteArray> field = nextField(cookieString, position, true);
         if (field.first.isEmpty())
             // parsing error
             break;
@@ -971,28 +978,27 @@ QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByt
             case ';':
                 // new field in the cookie
                 field = nextField(cookieString, position, false);
-                field.first = field.first.toLower(); // everything but the NAME=VALUE is case-insensitive
 
-                if (field.first == "expires") {
+                if (field.first.compare("expires", Qt::CaseInsensitive) == 0) {
                     position -= field.second.size();
                     int end;
                     for (end = position; end < length; ++end)
                         if (isValueSeparator(cookieString.at(end)))
                             break;
 
-                    QByteArray dateString = cookieString.mid(position, end - position).trimmed();
+                    QByteArray dateString = cookieString.mid(position, end - position).trimmed().toByteArray().toLower();
                     position = end;
-                    QDateTime dt = parseDateString(dateString.toLower());
+                    QDateTime dt = parseDateString(dateString);
                     if (dt.isValid())
                         cookie.setExpirationDate(dt);
                     //if unparsed, ignore the attribute but not the whole cookie (RFC6265 section 5.2.1)
-                } else if (field.first == "domain") {
-                    QByteArray rawDomain = field.second;
+                } else if (field.first.compare("domain", Qt::CaseInsensitive) == 0) {
+                    QByteArrayView rawDomain = field.second;
                     //empty domain should be ignored (RFC6265 section 5.2.3)
                     if (!rawDomain.isEmpty()) {
-                        QString maybeLeadingDot;
+                        QLatin1StringView maybeLeadingDot;
                         if (rawDomain.startsWith('.')) {
-                            maybeLeadingDot = u'.';
+                            maybeLeadingDot = "."_L1;
                             rawDomain = rawDomain.mid(1);
                         }
 
@@ -1007,7 +1013,7 @@ QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByt
                             return result;
                         }
                     }
-                } else if (field.first == "max-age") {
+                } else if (field.first.compare("max-age", Qt::CaseInsensitive) == 0) {
                     bool ok = false;
                     int secs = field.second.toInt(&ok);
                     if (ok) {
@@ -1019,7 +1025,7 @@ QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByt
                         }
                     }
                     //if unparsed, ignore the attribute but not the whole cookie (RFC6265 section 5.2.2)
-                } else if (field.first == "path") {
+                } else if (field.first.compare("path", Qt::CaseInsensitive) == 0) {
                     if (field.second.startsWith('/')) {
                         // ### we should treat cookie paths as an octet sequence internally
                         // However RFC6265 says we should assume UTF-8 for presentation as a string
@@ -1029,11 +1035,11 @@ QList<QNetworkCookie> QNetworkCookiePrivate::parseSetCookieHeaderLine(const QByt
                         // and also IETF test case path0030 which has valid and empty path in the same cookie
                         cookie.setPath(QString());
                     }
-                } else if (field.first == "secure") {
+                } else if (field.first.compare("secure", Qt::CaseInsensitive) == 0) {
                     cookie.setSecure(true);
-                } else if (field.first == "httponly") {
+                } else if (field.first.compare("httponly", Qt::CaseInsensitive) == 0) {
                     cookie.setHttpOnly(true);
-                } else if (field.first == "samesite") {
+                } else if (field.first.compare("samesite", Qt::CaseInsensitive) == 0) {
                     cookie.setSameSitePolicy(sameSiteFromRawString(field.second));
                 } else {
                     // ignore unknown fields in the cookie (RFC6265 section 5.2, rule 6)
