@@ -153,6 +153,9 @@ static void convertToLevelAndOption(QNativeSocketEngine::SocketOption opt,
     case QNativeSocketEngine::MaxStreamsSocketOption:
         Q_UNREACHABLE();
 
+    case QNativeSocketEngine::BindInterfaceIndex:
+        Q_UNREACHABLE(); // handled directly in setOption()
+
     case QNativeSocketEngine::ReceiveBufferSocketOption:
         n = SO_RCVBUF;
         break;
@@ -199,13 +202,7 @@ static void convertToLevelAndOption(QNativeSocketEngine::SocketOption opt,
         }
         break;
     case QNativeSocketEngine::ReceivePacketInformation:
-        if (socketProtocol == QAbstractSocket::IPv6Protocol || socketProtocol == QAbstractSocket::AnyIPProtocol) {
-            level = IPPROTO_IPV6;
-            n = IPV6_PKTINFO;
-        } else if (socketProtocol == QAbstractSocket::IPv4Protocol) {
-            level = IPPROTO_IP;
-            n = IP_PKTINFO;
-        }
+        Q_UNREACHABLE(); // handled in setOption() directly now
         break;
     case QNativeSocketEngine::ReceiveHopLimit:
         if (socketProtocol == QAbstractSocket::IPv6Protocol || socketProtocol == QAbstractSocket::AnyIPProtocol) {
@@ -330,6 +327,13 @@ bool QNativeSocketEnginePrivate::createNewSocket(QAbstractSocket::SocketType soc
                  &sendmsg, sizeof(sendmsg), &bytesReturned, NULL, NULL) == SOCKET_ERROR)
         sendmsg = 0;
 
+    // Attempt to enable dual-stack
+    if (protocol == AF_INET6) {
+        int ipv6only = 0;
+        ::setsockopt(socket, IPPROTO_IPV6, IPV6_V6ONLY,
+                     reinterpret_cast<char *>(&ipv6only), sizeof(ipv6only));
+    }
+
     socketDescriptor = socket;
     this->socketProtocol = socketProtocol;
     this->socketType = socketType;
@@ -418,6 +422,86 @@ bool QNativeSocketEnginePrivate::setOption(QNativeSocketEngine::SocketOption opt
     case QNativeSocketEngine::MaxStreamsSocketOption:
         return false;
 
+    case QNativeSocketEngine::ReceivePacketInformation: {
+        if (socketProtocol == QAbstractSocket::IPv6Protocol
+            || socketProtocol == QAbstractSocket::AnyIPProtocol) {
+            // set the IPv6 option
+            if (::setsockopt(socketDescriptor, IPPROTO_IPV6, IPV6_PKTINFO,
+                             reinterpret_cast<char *>(&v), sizeof(v)) != 0) {
+                WS_ERROR_DEBUG(WSAGetLastError());
+                return false;
+            }
+        }
+        // Try to set the IPv4 option in any case, but fail only if the
+        // protocol is IPv4
+        if (::setsockopt(socketDescriptor, IPPROTO_IP, IP_PKTINFO,
+                         reinterpret_cast<char *>(&v), sizeof(v)) != 0) {
+            if (socketProtocol == QAbstractSocket::IPv4Protocol) {
+                WS_ERROR_DEBUG(WSAGetLastError());
+                return false;
+            }
+        }
+        return true;
+    }
+
+    case QNativeSocketEngine::BindInterfaceIndex: {
+        int ret = 0;
+        if (socketProtocol == QAbstractSocket::IPv6Protocol
+            || socketProtocol == QAbstractSocket::AnyIPProtocol) {
+            // IPv6 - uses host byte order
+            // Bind outgoing datagrams to the interface
+            if (!ret) {
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IPV6, IPV6_UNICAST_IF,
+                                   reinterpret_cast<char *>(&v), sizeof(v));
+            }
+            if (!ret) {
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IPV6, IPV6_MULTICAST_IF,
+                                   reinterpret_cast<char *>(&v), sizeof(v));
+            }
+            // Bind incoming datagrams to the interface
+            if (!ret) {
+                const int enable = 1;
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IPV6, IPV6_IFLIST,
+                                   reinterpret_cast<const char *>(&enable), sizeof(enable));
+                if (!ret) {
+                    ret = ::setsockopt(socketDescriptor, IPPROTO_IPV6, IPV6_ADD_IFLIST,
+                                       reinterpret_cast<char *>(&v), sizeof(v));
+                }
+            }
+        }
+        bool result = !ret;
+        if (result) {
+            // Try to set the IPv4 options unconditionally, but ignore
+            // the result if the protocol is not IPv4-only
+
+            // IPv4 - uses network byte order
+            int netIdx = htonl(v);
+            // Bind outgoing datagrams to the interface
+            if (!ret) {
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IP, IP_UNICAST_IF,
+                                   reinterpret_cast<char *>(&netIdx), sizeof(netIdx));
+            }
+            if (!ret) {
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IP, IP_MULTICAST_IF,
+                                   reinterpret_cast<char *>(&netIdx), sizeof(netIdx));
+            }
+            // Bind incoming datagrams to the interface
+            if (!ret) {
+                const int enable = 1;
+                ret = ::setsockopt(socketDescriptor, IPPROTO_IP, IP_IFLIST,
+                                   reinterpret_cast<const char *>(&enable), sizeof(enable));
+                if (!ret) {
+                    // uses host byte order here
+                    ret = ::setsockopt(socketDescriptor, IPPROTO_IP, IP_ADD_IFLIST,
+                                       reinterpret_cast<char *>(&v), sizeof(v));
+                }
+            }
+            if (socketProtocol == QAbstractSocket::IPv4Protocol)
+                result = !ret;
+        }
+        return result;
+    }
+
     default:
         break;
     }
