@@ -9,13 +9,15 @@
 #include <QtGui/qpalette.h>
 #include <QtCore/qhash.h>
 #include <QtCore/qbytearray.h>
-
+#include <QtCore/qloggingcategory.h>
 #include <QJsonObject>
 
 #include <memory>
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(lcQpaMenus)
+
 struct AndroidStyle
 {
     static QJsonObject loadStyleData();
@@ -30,24 +32,36 @@ class QAndroidPlatformNativeInterface;
 class QAndroidPlatformTheme: public QPlatformTheme
 {
 public:
-    QAndroidPlatformTheme(QAndroidPlatformNativeInterface * androidPlatformNativeInterface);
+    ~QAndroidPlatformTheme();
+    void updateColorScheme();
+    void updateStyle();
     QPlatformMenuBar *createPlatformMenuBar() const override;
     QPlatformMenu *createPlatformMenu() const override;
     QPlatformMenuItem *createPlatformMenuItem() const override;
     void showPlatformMenuBar() override;
-    Appearance appearance() const override;
+    Qt::ColorScheme colorScheme() const override;
+    Qt::ColorScheme colorSchemeOverride() const { return m_colorSchemeOverride; };
+    void requestColorScheme(Qt::ColorScheme scheme) override;
+    Qt::ContrastPreference contrastPreference() const override;
+
     const QPalette *palette(Palette type = SystemPalette) const override;
     const QFont *font(Font type = SystemFont) const override;
+    QIconEngine *createIconEngine(const QString &iconName) const override;
     QVariant themeHint(ThemeHint hint) const override;
     QString standardButtonText(int button) const override;
     bool usePlatformNativeDialog(DialogType type) const override;
     QPlatformDialogHelper *createPlatformDialogHelper(DialogType type) const override;
 
+    static QAndroidPlatformTheme *instance(
+                    QAndroidPlatformNativeInterface * androidPlatformNativeInterface = nullptr);
 
 private:
+    QAndroidPlatformTheme(QAndroidPlatformNativeInterface * androidPlatformNativeInterface);
+    static QAndroidPlatformTheme * m_instance;
     std::shared_ptr<AndroidStyle> m_androidStyleData;
     QPalette m_defaultPalette;
     QFont m_systemFont;
+    Qt::ColorScheme m_colorSchemeOverride = Qt::ColorScheme::Unknown;
 };
 
 QT_END_NAMESPACE
