@@ -25,18 +25,13 @@ QCocoaMenuBar::QCocoaMenuBar()
 {
     static_menubars.append(this);
 
-    // clicks into the menu bar should close all popup windows
-    static QMacNotificationObserver menuBarClickObserver(nil, NSMenuDidBeginTrackingNotification, ^{
-        QGuiApplicationPrivate::instance()->closeAllPopups();
-    });
-
     m_nativeMenu = [[NSMenu alloc] init];
     qCDebug(lcQpaMenus) << "Constructed" << this << "with" << m_nativeMenu;
 }
 
 QCocoaMenuBar::~QCocoaMenuBar()
 {
-    qCDebug(lcQpaMenus) << "Destructing" << this << "with" << m_nativeMenu;;
+    qCDebug(lcQpaMenus) << "Destructing" << this << "with" << m_nativeMenu;
     for (auto menu : std::as_const(m_menus)) {
         if (!menu)
             continue;
