@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Copyright (C) 2012 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author James Turner <james.turner@kdab.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// Copyright (C) 2012 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com, author James Turner <james.turner@kdab.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QCOCOAMENUITEM_H
 #define QCOCOAMENUITEM_H
@@ -44,7 +8,7 @@
 #include <qpa/qplatformmenu.h>
 #include <QtGui/QImage>
 
-//#define QT_COCOA_ENABLE_MENU_DEBUG
+#include <QtCore/qpointer.h>
 
 Q_FORWARD_DECLARE_OBJC_CLASS(NSMenuItem);
 Q_FORWARD_DECLARE_OBJC_CLASS(NSMenu);
@@ -53,6 +17,18 @@ Q_FORWARD_DECLARE_OBJC_CLASS(NSView);
 
 QT_BEGIN_NAMESPACE
 
+enum {
+    AboutAppMenuItem = 0,
+    PreferencesAppMenuItem,
+    ServicesAppMenuItem,
+    HideAppMenuItem,
+    HideOthersAppMenuItem,
+    ShowAllAppMenuItem,
+    QuitAppMenuItem
+};
+
+QString qt_mac_applicationmenu_string(int type);
+
 class QCocoaMenu;
 
 class QCocoaMenuObject
@@ -88,7 +64,7 @@ public:
 #ifndef QT_NO_SHORTCUT
     void setShortcut(const QKeySequence& shortcut) override;
 #endif
-    void setCheckable(bool checkable) override { Q_UNUSED(checkable) }
+    void setCheckable(bool) override {}
     void setChecked(bool isChecked) override;
     void setEnabled(bool isEnabled) override;
     void setIconSize(int size) override;
@@ -105,6 +81,7 @@ public:
     inline bool isMerged() const { return m_merged; }
     inline bool isEnabled() const { return m_enabled && m_parentEnabled; }
     inline bool isSeparator() const { return m_isSeparator; }
+    inline bool isVisible() const { return m_isVisible; }
 
     QCocoaMenu *menu() const { return m_menu; }
     MenuRole effectiveRole() const;
