@@ -15,6 +15,7 @@
 #include <IOKit/graphics/IOGraphicsLib.h>
 
 #include <QtGui/private/qwindow_p.h>
+#include <QtGui/private/qhighdpiscaling_p.h>
 
 #include <QtCore/private/qcore_mac_p.h>
 #include <QtCore/private/qeventdispatcher_cf_p.h>
@@ -63,6 +64,10 @@ void QCocoaScreen::initializeScreens()
         NSApplicationDidChangeScreenParametersNotification, [&]() {
             qCDebug(lcQpaScreen) << "Received screen parameter change notification";
             updateScreens();
+
+            // The notification is posted when the EDR headroom of a display changes,
+            // which might affect the rendering of windows that opt in to EDR.
+            updateHdrWindows();
         });
 }
 
@@ -83,7 +88,7 @@ void QCocoaScreen::updateScreens()
         qCInfo(lcQpaScreen) << "Skipping screen update, already updating";
         return;
     }
-    QBoolBlocker recursionGuard(updatingScreens);
+    QScopedValueRollback recursionGuard(updatingScreens, true);
 
     uint32_t displayCount = 0;
     if (CGGetOnlineDisplayList(0, nullptr, &displayCount) != kCGErrorSuccess)
@@ -196,38 +201,6 @@ QCocoaScreen::~QCocoaScreen()
          dispatch_release(m_displayLinkSource);
 }
 
-static QString displayName(CGDirectDisplayID displayID)
-{
-    QIOType<io_iterator_t> iterator;
-    if (IOServiceGetMatchingServices(kIOMasterPortDefault,
-        IOServiceMatching("IODisplayConnect"), &iterator))
-        return QString();
-
-    QIOType<io_service_t> display;
-    while ((display = IOIteratorNext(iterator)) != 0)
-    {
-        NSDictionary *info = [(__bridge NSDictionary*)IODisplayCreateInfoDictionary(
-            display, kIODisplayOnlyPreferredName) autorelease];
-
-        if ([[info objectForKey:@kDisplayVendorID] unsignedIntValue] != CGDisplayVendorNumber(displayID))
-            continue;
-
-        if ([[info objectForKey:@kDisplayProductID] unsignedIntValue] != CGDisplayModelNumber(displayID))
-            continue;
-
-        if ([[info objectForKey:@kDisplaySerialNumber] unsignedIntValue] != CGDisplaySerialNumber(displayID))
-            continue;
-
-        NSDictionary *localizedNames = [info objectForKey:@kDisplayProductName];
-        if (![localizedNames count])
-            break; // Correct screen, but no name in dictionary
-
-        return QString::fromNSString([localizedNames objectForKey:[[localizedNames allKeys] objectAtIndex:0]]);
-    }
-
-    return QString();
-}
-
 void QCocoaScreen::update(CGDirectDisplayID displayId)
 {
     if (displayId != m_displayId) {
@@ -247,6 +220,7 @@ void QCocoaScreen::update(CGDirectDisplayID displayId)
     const QRect previousGeometry = m_geometry;
     const QRect previousAvailableGeometry = m_availableGeometry;
     const qreal previousRefreshRate = m_refreshRate;
+    const double previousRotation = m_rotation;
 
     // The reference screen for the geometry is always the primary screen
     QRectF primaryScreenGeometry = QRectF::fromCGRect(CGDisplayBounds(CGMainDisplayID()));
@@ -271,14 +245,14 @@ void QCocoaScreen::update(CGDirectDisplayID displayId)
     QCFType<CGDisplayModeRef> displayMode = CGDisplayCopyDisplayMode(m_displayId);
     float refresh = CGDisplayModeGetRefreshRate(displayMode);
     m_refreshRate = refresh > 0 ? refresh : 60.0;
-
-    if (@available(macOS 10.15, *))
-        m_name = QString::fromNSString(nsScreen.localizedName);
-    else
-        m_name = displayName(m_displayId);
+    m_rotation = CGDisplayRotation(displayId);
+    m_name = QString::fromNSString(nsScreen.localizedName);
 
     const bool didChangeGeometry = m_geometry != previousGeometry || m_availableGeometry != previousAvailableGeometry;
 
+    if (m_rotation != previousRotation)
+        QWindowSystemInterface::handleScreenOrientationChange(screen(), orientation());
+
     if (didChangeGeometry)
         QWindowSystemInterface::handleScreenGeometryChange(screen(), geometry(), availableGeometry());
     if (m_refreshRate != previousRefreshRate)
@@ -298,6 +272,11 @@ bool QCocoaScreen::requestUpdate()
         return false;
     }
 
+    // Track how many update requests we have queued, so that we
+    // know whether the display-link thread should try to deliver
+    // update requests, or if it can bail out early.
+    ++m_pendingUpdateRequests;
+
     if (!m_displayLink) {
         qCDebug(lcQpaScreenUpdates) << "Creating display link for" << this;
         if (CVDisplayLinkCreateWithCGDisplay(m_displayId, &m_displayLink) != kCVReturnSuccess) {
@@ -411,10 +390,13 @@ void QCocoaScreen::deliverUpdateRequests()
         // We're explicitly not using the data of the GCD source to track the pending updates,
         // as the data isn't reset to 0 until after the event handler, and also doesn't update
         // during the event handler, both of which we need to track late frames.
-        const int pendingUpdates = ++m_pendingUpdates;
+        const int pendingUpdates = ++m_pendingDisplayLinkUpdates;
+
+        const int pendingUpdateRequests = m_pendingUpdateRequests;
 
         DeferredDebugHelper screenUpdates(lcQpaScreenUpdates());
-        qDeferredDebug(screenUpdates) << "display link callback for screen " << m_displayId;
+        qDeferredDebug(screenUpdates) << "display link callback for screen " << m_displayId
+            << " with " << pendingUpdateRequests << " pending update requests";
 
         if (const int framesAheadOfDelivery = pendingUpdates - 1) {
             // If we have more than one update pending it means that a previous display link callback
@@ -424,6 +406,16 @@ void QCocoaScreen::deliverUpdateRequests()
             qDeferredDebug(screenUpdates) << ", " << framesAheadOfDelivery << " frame(s) ahead";
         }
 
+        if (!pendingUpdateRequests) {
+            // There's a cost to stopping and starting the display link thread,
+            // so once started we always keep it running, to avoid missing frames.
+            // In the case where we don't have any pending update requests we don't
+            // need to signal the main thread.
+            qDeferredDebug(screenUpdates) << "; skipping signaling dispatch source";
+            m_pendingDisplayLinkUpdates = 0;
+            return;
+        }
+
         qDeferredDebug(screenUpdates) << "; signaling dispatch source";
 
         if (!m_displayLinkSource) {
@@ -440,54 +432,97 @@ void QCocoaScreen::deliverUpdateRequests()
         DeferredDebugHelper screenUpdates(lcQpaScreenUpdates());
         qDeferredDebug(screenUpdates) << "gcd event handler on main thread";
 
-        const int pendingUpdates = m_pendingUpdates;
+        const int pendingUpdates = m_pendingDisplayLinkUpdates;
         if (pendingUpdates > 1)
             qDeferredDebug(screenUpdates) << ", " << (pendingUpdates - 1) << " frame(s) behind display link";
 
         screenUpdates.flushOutput();
 
-        bool pauseUpdates = true;
+        int pendingUpdateRequests = 0;
 
         auto windows = QGuiApplication::allWindows();
         for (int i = 0; i < windows.size(); ++i) {
             QWindow *window = windows.at(i);
-            auto *platformWindow = static_cast<QCocoaWindow*>(window->handle());
+            if (window->screen() != screen())
+                continue;
+
+            QPointer<QCocoaWindow> platformWindow = static_cast<QCocoaWindow*>(window->handle());
             if (!platformWindow)
                 continue;
 
             if (!platformWindow->hasPendingUpdateRequest())
                 continue;
 
-            if (window->screen() != screen())
-                continue;
-
             // Skip windows that are not doing update requests via display link
             if (!platformWindow->updatesWithDisplayLink())
                 continue;
 
             platformWindow->deliverUpdateRequest();
 
-            // Another update request was triggered, keep the display link running
+            // platform window can be destroyed in deliverUpdateRequest()
+            if (!platformWindow)
+                continue;
+
+            // The update request delivery could result in another request
+            // from the window, or the platform window could decide to not
+            // deliver the request at this time.
             if (platformWindow->hasPendingUpdateRequest())
-                pauseUpdates = false;
+                ++pendingUpdateRequests;
         }
 
-        if (pauseUpdates) {
-            // Pause the display link if there are no pending update requests
-            qCDebug(lcQpaScreenUpdates) << "Stopping display link for" << this;
-            CVDisplayLinkStop(m_displayLink);
-        }
+        m_pendingUpdateRequests = pendingUpdateRequests;
 
-        if (const int missedUpdates = m_pendingUpdates.fetchAndStoreRelaxed(0) - pendingUpdates) {
+        if (const int missedUpdates = m_pendingDisplayLinkUpdates.fetchAndStoreRelaxed(0) - pendingUpdates) {
             qCWarning(lcQpaScreenUpdates) << "main thread missed" << missedUpdates
                 << "update(s) from display link during update request delivery";
         }
     }
 }
 
-bool QCocoaScreen::isRunningDisplayLink() const
+void QCocoaScreen::maybeStopDisplayLink()
 {
-    return m_displayLink && CVDisplayLinkIsRunning(m_displayLink);
+    if (!CVDisplayLinkIsRunning(m_displayLink))
+        return;
+
+    const auto windows = QGuiApplication::allWindows();
+    for (auto *window : windows) {
+        if (window->screen() != screen())
+            continue;
+
+        QPointer<QCocoaWindow> platformWindow = static_cast<QCocoaWindow*>(window->handle());
+        if (!platformWindow)
+            continue;
+
+        if (window->isExposed())
+            return;
+
+        if (platformWindow->hasPendingUpdateRequest())
+            return;
+    }
+
+    qCDebug(lcQpaScreenUpdates) << "Stopping display link for" << this;
+    CVDisplayLinkStop(m_displayLink);
+}
+
+
+// -----------------------------------------------------------
+
+void QCocoaScreen::updateHdrWindows()
+{
+    if (@available(macOS 14, *)) {
+        for (auto *window : QGuiApplication::allWindows()) {
+            auto *platformWindow = static_cast<QCocoaWindow*>(window->handle());
+            if (!platformWindow)
+                continue;
+
+            NSView *view = platformWindow->view();
+
+            if (!view.layer.wantsExtendedDynamicRangeContent)
+                continue;
+
+            [view setNeedsDisplay:YES];
+        }
+    }
 }
 
 // -----------------------------------------------------------
@@ -502,41 +537,56 @@ QPlatformScreen::SubpixelAntialiasingType QCocoaScreen::subpixelAntialiasingType
     return type;
 }
 
+Qt::ScreenOrientation QCocoaScreen::orientation() const
+{
+    if (m_rotation == 0)
+        return Qt::LandscapeOrientation;
+    if (m_rotation == 90)
+        return Qt::PortraitOrientation;
+    if (m_rotation == 180)
+        return Qt::InvertedLandscapeOrientation;
+    if (m_rotation == 270)
+        return Qt::InvertedPortraitOrientation;
+    return QPlatformScreen::orientation();
+}
+
 QWindow *QCocoaScreen::topLevelAt(const QPoint &point) const
 {
-    NSPoint screenPoint = mapToNative(point);
+    __block QWindow *window = nullptr;
+    [NSApp enumerateWindowsWithOptions:NSWindowListOrderedFrontToBack
+        usingBlock:^(NSWindow *nsWindow, BOOL *stop) {
+            if (!nsWindow)
+                return;
 
-    // Search (hit test) for the top-level window. [NSWidow windowNumberAtPoint:
-    // belowWindowWithWindowNumber] may return windows that are not interesting
-    // to Qt. The search iterates until a suitable window or no window is found.
-    NSInteger topWindowNumber = 0;
-    QWindow *window = nullptr;
-    do {
-        // Get the top-most window, below any previously rejected window.
-        topWindowNumber = [NSWindow windowNumberAtPoint:screenPoint
-                                    belowWindowWithWindowNumber:topWindowNumber];
+            // Continue the search if the window does not belong to Qt
+            if (![nsWindow conformsToProtocol:@protocol(QNSWindowProtocol)])
+                return;
 
-        // Continue the search if the window does not belong to this process.
-        NSWindow *nsWindow = [NSApp windowWithWindowNumber:topWindowNumber];
-        if (!nsWindow)
-            continue;
+            QCocoaWindow *cocoaWindow = qnsview_cast(nsWindow.contentView).platformWindow;
+            if (!cocoaWindow)
+                return;
 
-        // Continue the search if the window does not belong to Qt.
-        if (![nsWindow conformsToProtocol:@protocol(QNSWindowProtocol)])
-            continue;
+            QWindow *w = cocoaWindow->window();
+            if (!w->isVisible())
+                return;
 
-        QCocoaWindow *cocoaWindow = qnsview_cast(nsWindow.contentView).platformWindow;
-        if (!cocoaWindow)
-            continue;
-        window = cocoaWindow->window();
+            auto nativeGeometry = QHighDpi::toNativePixels(w->geometry(), w);
+            if (!nativeGeometry.contains(point))
+                return;
 
-        // Continue the search if the window is not a top-level window.
-        if (!window->isTopLevel())
-             continue;
+            QRegion mask = QHighDpi::toNativeLocalPosition(w->mask(), w);
+            if (!mask.isEmpty() && !mask.contains(point - nativeGeometry.topLeft()))
+                return;
 
-        // Stop searching. The current window is the correct window.
-        break;
-    } while (topWindowNumber > 0);
+            window = w;
+
+            // Continue the search if the window is not a top-level window
+            if (!window->isTopLevel())
+                return;
+
+            *stop = true;
+        }
+    ];
 
     return window;
 }
