@@ -12,9 +12,10 @@
 #include "qcocoaintegration.h"
 
 #include <qpa/qwindowsysteminterface.h>
-#include <qoperatingsystemversion.h>
 
-Q_LOGGING_CATEGORY(lcQpaEvents, "qt.qpa.events");
+#include <QtGui/private/qhighdpiscaling_p.h>
+
+Q_STATIC_LOGGING_CATEGORY(lcQpaEvents, "qt.qpa.events");
 
 static bool isMouseEvent(NSEvent *ev)
 {
@@ -58,6 +59,17 @@ static bool isMouseEvent(NSEvent *ev)
 }
 @end
 
+
+QT_BEGIN_NAMESPACE
+NSWindow<QNSWindowProtocol> *qnswindow_cast(NSWindow *window)
+{
+    if ([window conformsToProtocol:@protocol(QNSWindowProtocol)])
+        return static_cast<QCocoaNSWindow *>(window);
+    else
+        return nil;
+}
+QT_END_NAMESPACE
+
 @implementation QNSWindow
 #define QNSWINDOW_PROTOCOL_IMPLMENTATION 1
 #include "qnswindow.mm"
@@ -98,9 +110,10 @@ static bool isMouseEvent(NSEvent *ev)
             continue;
 
         if ([window conformsToProtocol:@protocol(QNSWindowProtocol)]) {
-            QCocoaWindow *cocoaWindow = static_cast<QCocoaNSWindow *>(window).platformWindow;
-            window.level = notification.name == NSApplicationWillResignActiveNotification ?
-                NSNormalWindowLevel : cocoaWindow->windowLevel(cocoaWindow->window()->flags());
+            if (QCocoaWindow *cocoaWindow = static_cast<QCocoaNSWindow *>(window).platformWindow) {
+                window.level = notification.name == NSApplicationWillResignActiveNotification ?
+                    NSNormalWindowLevel : cocoaWindow->windowLevel(cocoaWindow->window()->flags());
+            }
         }
 
         // The documentation says that "when a window enters a new level, it’s ordered
@@ -167,45 +180,6 @@ static bool isMouseEvent(NSEvent *ev)
 }
 @end
 
-#if !defined(QT_APPLE_NO_PRIVATE_APIS)
-// When creating an NSWindow the worksWhenModal function is queried,
-// and the resulting state is used to set the corresponding window tag,
-// which the window server uses to determine whether or not the window
-// should be allowed to activate via mouse clicks in the title-bar.
-// Unfortunately, prior to macOS 10.15, this window tag was never
-// updated after the initial assignment in [NSWindow _commonAwake],
-// which meant that windows that dynamically change their worksWhenModal
-// state will behave as if they were never allowed to work when modal.
-// We work around this by manually updating the window tag when needed.
-
-typedef uint32_t CGSConnectionID;
-typedef uint32_t CGSWindowID;
-
-extern "C" {
-CGSConnectionID CGSMainConnectionID() __attribute__((weak_import));
-OSStatus CGSSetWindowTags(const CGSConnectionID, const CGSWindowID, int *, int) __attribute__((weak_import));
-OSStatus CGSClearWindowTags(const CGSConnectionID, const CGSWindowID, int *, int) __attribute__((weak_import));
-}
-
-@interface QNSPanel (WorksWhenModalWindowTagWorkaround) @end
-@implementation QNSPanel (WorksWhenModalWindowTagWorkaround)
-- (void)setWorksWhenModal:(BOOL)worksWhenModal
-{
-    [super setWorksWhenModal:worksWhenModal];
-
-    if (QOperatingSystemVersion::current() < QOperatingSystemVersion::MacOSCatalina) {
-        if (CGSMainConnectionID && CGSSetWindowTags && CGSClearWindowTags) {
-            static int kWorksWhenModalWindowTag = 0x40;
-            auto *function = worksWhenModal ? CGSSetWindowTags : CGSClearWindowTags;
-            function(CGSMainConnectionID(), self.windowNumber, &kWorksWhenModalWindowTag, 64);
-        } else {
-            qWarning() << "Missing APIs for window tag handling, can not update worksWhenModal state";
-        }
-    }
-}
-@end
-#endif // QT_APPLE_NO_PRIVATE_APIS
-
 #else // QNSWINDOW_PROTOCOL_IMPLMENTATION
 
 // The following content is mixed in to the QNSWindow and QNSPanel classes via includes
@@ -235,6 +209,40 @@ OSStatus CGSClearWindowTags(const CGSConnectionID, const CGSWindowID, int *, int
     return m_platformWindow;
 }
 
+- (void)setContentView:(NSView*)view
+{
+    [super setContentView:view];
+
+    if (!qnsview_cast(self.contentView))
+        return;
+
+    // Now that we're the content view, we can apply the properties of
+    // the QWindow. We do this here, instead of in init, so that we can
+    // use the same code paths for setting these properties during
+    // NSWindow initialization as we do when setting them later on.
+    const QWindow *window = m_platformWindow->window();
+    qCDebug(lcQpaWindow) << "Reflecting" << window << "state to" << self;
+
+    m_platformWindow->propagateSizeHints();
+    m_platformWindow->setWindowFlags(window->flags());
+    m_platformWindow->setWindowTitle(window->title());
+    m_platformWindow->setWindowFilePath(window->filePath()); // Also sets window icon
+    m_platformWindow->setWindowState(window->windowState());
+    m_platformWindow->setOpacity(window->opacity());
+
+    // At the time of creation the QNSWindow is given a geometry based
+    // on the client geometry of the QWindow. But at that point we don't
+    // know anything about the size of the NSWindow frame, which means
+    // that the logic in QCocoaWindow::setGeometry for adjusting the
+    // client geometry based on the QWindow's positionPolicy is a noop.
+    // Now that we have a NSWindow to read the frame from we re-apply
+    // the QWindow geometry, which will move the NSWindow if needed.
+    m_platformWindow->setGeometry(QHighDpi::toNativeWindowGeometry(window->geometry(), window));
+
+
+    m_platformWindow->setVisible(window->isVisible());
+}
+
 - (NSString *)description
 {
     NSMutableString *description = [NSMutableString stringWithString:[super description]];
@@ -357,7 +365,7 @@ OSStatus CGSClearWindowTags(const CGSConnectionID, const CGSWindowID, int *, int
     // not Qt). However, an active popup is expected to grab any mouse event within the
     // application, so we need to handle those explicitly and trust Qt's isWindowBlocked
     // implementation to eat events that shouldn't be delivered anyway.
-    if (isMouseEvent(theEvent) && QGuiApplicationPrivate::instance()->popupActive()
+    if (isMouseEvent(theEvent) && QGuiApplicationPrivate::instance()->activePopupWindow()
         && QGuiApplicationPrivate::instance()->isWindowBlocked(m_platformWindow->window(), nullptr)) {
         qCDebug(lcQpaWindow) << "Mouse event over modally blocked window" << m_platformWindow->window()
                              << "while popup is open - redirecting";
@@ -369,7 +377,7 @@ OSStatus CGSClearWindowTags(const CGSConnectionID, const CGSWindowID, int *, int
 
 - (void)miniaturize:(id)sender
 {
-    QBoolBlocker miniaturizeTracker(m_isMinimizing, true);
+    QScopedValueRollback miniaturizeTracker(m_isMinimizing, true);
     [super miniaturize:sender];
 }
 
