@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qxcbeglintegration.h"
 
@@ -58,6 +59,7 @@ std::optional<VisualInfo> getVisualInfo(xcb_screen_t *screen,
 QXcbEglIntegration::QXcbEglIntegration()
     : m_connection(nullptr)
     , m_egl_display(EGL_NO_DISPLAY)
+    , m_using_platform_display(false)
 {
     qCDebug(lcQpaGl) << "Xcb EGL gl-integration created";
 }
@@ -80,6 +82,7 @@ bool QXcbEglIntegration::initialize(QXcbConnection *connection)
         m_egl_display = streamFuncs.get_platform_display(EGL_PLATFORM_X11_KHR,
                                                          m_connection->xlib_display(),
                                                          nullptr);
+        m_using_platform_display = true;
     }
 
 #if QT_CONFIG(egl_x11)
@@ -92,16 +95,19 @@ bool QXcbEglIntegration::initialize(QXcbConnection *connection)
         m_egl_display = streamFuncs.get_platform_display(EGL_PLATFORM_XCB_KHR,
                                                          reinterpret_cast<void *>(connection->xcb_connection()),
                                                          nullptr);
+        m_using_platform_display = true;
     }
 #endif
 
     EGLint major, minor;
     bool success = eglInitialize(m_egl_display, &major, &minor);
+#if QT_CONFIG(egl_x11)
     if (!success) {
         m_egl_display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
         qCDebug(lcQpaGl) << "Xcb EGL gl-integration retrying with display" << m_egl_display;
         success = eglInitialize(m_egl_display, &major, &minor);
     }
+#endif
 
     m_native_interface_handler.reset(new QXcbEglNativeInterfaceHandler(connection->nativeInterface()));
 
