@@ -1,44 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
+
 #include "qxcbkeyboard.h"
 #include "qxcbwindow.h"
 #include "qxcbscreen.h"
+#include "qxcbcursor.h"
 
 #include <qpa/qwindowsysteminterface.h>
 #include <qpa/qplatforminputcontext.h>
@@ -49,13 +16,7 @@
 
 #include <private/qguiapplication_p.h>
 
-#if QT_CONFIG(xkb)
-#include <xkbcommon/xkbcommon-x11.h>
-#endif
-
-#if QT_CONFIG(xcb_xinput)
 #include <xcb/xinput.h>
-#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -229,7 +190,7 @@ struct xkb_keymap *QXcbKeyboard::keymapFromCore(const KeysymModifierMap &keysymM
 
     // Generate mapping between symbolic names and keysyms
     {
-        QVector<xcb_keysym_t> xkeymap;
+        QList<xcb_keysym_t> xkeymap;
         int keysymsPerKeycode = 0;
         {
             int keycodeCount = maxKeycode - minKeycode + 1;
@@ -263,7 +224,7 @@ struct xkb_keymap *QXcbKeyboard::keymapFromCore(const KeysymModifierMap &keysymM
             const int maxGroup1 = 4; // We only support 4 shift states anyway
             const int maxGroup2 = 2; // Only 3rd and 4th keysym are group 2
             xcb_keysym_t symbolsGroup1[maxGroup1];
-            xcb_keysym_t symbolsGroup2[maxGroup2];
+            xcb_keysym_t symbolsGroup2[maxGroup2] = { XKB_KEY_NoSymbol, XKB_KEY_NoSymbol };
             for (int i = 0; i < maxGroup1 + maxGroup2; i++) {
                 xcb_keysym_t sym = i < keysymsPerKeycode ? codeMap[i] : XKB_KEY_NoSymbol;
                 if (mapGroup2ToLevel3) {
@@ -392,20 +353,16 @@ void QXcbKeyboard::updateKeymap()
         xkb_context_set_log_level(m_xkbContext.get(), logLevel);
     }
 
-#if QT_CONFIG(xkb)
     if (connection()->hasXKB()) {
         m_xkbKeymap.reset(xkb_x11_keymap_new_from_device(m_xkbContext.get(), xcb_connection(),
                                                          core_device_id, XKB_KEYMAP_COMPILE_NO_FLAGS));
         if (m_xkbKeymap)
             m_xkbState.reset(xkb_x11_state_new_from_device(m_xkbKeymap.get(), xcb_connection(), core_device_id));
     } else {
-#endif
         m_xkbKeymap.reset(keymapFromCore(keysymMods));
         if (m_xkbKeymap)
             m_xkbState.reset(xkb_state_new(m_xkbKeymap.get()));
-#if QT_CONFIG(xkb)
     }
-#endif
 
     if (!m_xkbKeymap) {
         qCWarning(lcQpaKeyboard, "failed to compile a keymap");
@@ -423,12 +380,20 @@ void QXcbKeyboard::updateKeymap()
     QXkbCommon::verifyHasLatinLayout(m_xkbKeymap.get());
 }
 
-QList<int> QXcbKeyboard::possibleKeys(const QKeyEvent *event) const
+QList<QKeyCombination> QXcbKeyboard::possibleKeyCombinations(const QKeyEvent *event) const
 {
-    return QXkbCommon::possibleKeys(m_xkbState.get(), event, m_superAsMeta, m_hyperAsMeta);
+    return QXkbCommon::possibleKeyCombinations(
+        m_xkbState.get(), event, m_superAsMeta, m_hyperAsMeta);
+}
+
+Qt::KeyboardModifiers QXcbKeyboard::queryKeyboardModifiers() const
+{
+    // FIXME: Should we base this on m_xkbState?
+    int stateMask = 0;
+    QXcbCursor::queryPointer(connection(), nullptr, nullptr, &stateMask);
+    return translateModifiers(stateMask);
 }
 
-#if QT_CONFIG(xkb)
 void QXcbKeyboard::updateXKBState(xcb_xkb_state_notify_event_t *state)
 {
     if (m_config && connection()->hasXKB()) {
@@ -444,7 +409,6 @@ void QXcbKeyboard::updateXKBState(xcb_xkb_state_notify_event_t *state)
         handleStateChanges(changedComponents);
     }
 }
-#endif
 
 static xkb_layout_index_t lockedGroup(quint16 state)
 {
@@ -453,7 +417,7 @@ static xkb_layout_index_t lockedGroup(quint16 state)
 
 void QXcbKeyboard::updateXKBStateFromCore(quint16 state)
 {
-    if (m_config && !connection()->hasXKB()) {
+    if (m_config) {
         struct xkb_state *xkbState = m_xkbState.get();
         xkb_mod_mask_t modsDepressed = xkb_state_serialize_mods(xkbState, XKB_STATE_MODS_DEPRESSED);
         xkb_mod_mask_t modsLatched = xkb_state_serialize_mods(xkbState, XKB_STATE_MODS_LATCHED);
@@ -473,10 +437,9 @@ void QXcbKeyboard::updateXKBStateFromCore(quint16 state)
     }
 }
 
-#if QT_CONFIG(xcb_xinput)
 void QXcbKeyboard::updateXKBStateFromXI(void *modInfo, void *groupInfo)
 {
-    if (m_config && !connection()->hasXKB()) {
+    if (m_config) {
         auto *mods = static_cast<xcb_input_modifier_info_t *>(modInfo);
         auto *group = static_cast<xcb_input_group_info_t *>(groupInfo);
         const xkb_state_component changedComponents
@@ -491,7 +454,6 @@ void QXcbKeyboard::updateXKBStateFromXI(void *modInfo, void *groupInfo)
         handleStateChanges(changedComponents);
     }
 }
-#endif
 
 void QXcbKeyboard::handleStateChanges(xkb_state_component changedComponents)
 {
@@ -541,7 +503,6 @@ void QXcbKeyboard::updateXKBMods()
 QXcbKeyboard::QXcbKeyboard(QXcbConnection *connection)
     : QXcbObject(connection)
 {
-#if QT_CONFIG(xkb)
     core_device_id = 0;
     if (connection->hasXKB()) {
         selectEvents();
@@ -551,11 +512,9 @@ QXcbKeyboard::QXcbKeyboard(QXcbConnection *connection)
             return;
         }
     } else {
-#endif
         m_key_symbols = xcb_key_symbols_alloc(xcb_connection());
-#if QT_CONFIG(xkb)
     }
-#endif
+
     updateKeymap();
 }
 
@@ -573,7 +532,6 @@ void QXcbKeyboard::initialize()
 
 void QXcbKeyboard::selectEvents()
 {
-#if QT_CONFIG(xkb)
     const uint16_t required_map_parts = (XCB_XKB_MAP_PART_KEY_TYPES |
         XCB_XKB_MAP_PART_KEY_SYMS |
         XCB_XKB_MAP_PART_MODIFIER_MAP |
@@ -597,19 +555,17 @@ void QXcbKeyboard::selectEvents()
                 required_events,
                 required_map_parts,
                 required_map_parts,
-                0);
+                nullptr);
 
     xcb_generic_error_t *error = xcb_request_check(xcb_connection(), select);
     if (error) {
         free(error);
         qCWarning(lcQpaXcb, "failed to select notify events from XKB");
     }
-#endif
 }
 
 void QXcbKeyboard::updateVModMapping()
 {
-#if QT_CONFIG(xkb)
     xcb_xkb_get_names_value_list_t names_list;
 
     memset(&vmod_masks, 0, sizeof(vmod_masks));
@@ -640,7 +596,7 @@ void QXcbKeyboard::updateVModMapping()
     vmod_mask = name_reply->virtualMods;
     // find the virtual modifiers for which names are defined.
     for (bit = 1; vmod_mask; bit <<= 1) {
-        vmod_name = 0;
+        vmod_name = nullptr;
 
         if (!(vmod_mask & bit))
             continue;
@@ -667,12 +623,10 @@ void QXcbKeyboard::updateVModMapping()
         else if (qstrcmp(vmod_name, "Hyper") == 0)
             vmod_masks.hyper = bit;
     }
-#endif
 }
 
 void QXcbKeyboard::updateVModToRModMapping()
 {
-#if QT_CONFIG(xkb)
     xcb_xkb_get_map_map_t map;
 
     memset(&rmod_masks, 0, sizeof(rmod_masks));
@@ -729,7 +683,6 @@ void QXcbKeyboard::updateVModToRModMapping()
         else if (vmod_masks.hyper == bit)
             rmod_masks.hyper = modmap;
     }
-#endif
 }
 
 // Small helper: set modifier bit, if modifier position is valid
