@@ -1,5 +1,6 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 //
 //  W A R N I N G
@@ -289,15 +290,17 @@ QImage QGtk3Interface::qt_convert_gdk_pixbuf(GdkPixbuf *buf) const
     const guint8 *gdata = gdk_pixbuf_read_pixels(buf);
     static_assert(std::is_same<decltype(gdata), const uchar *>::value,
             "guint8 has diverted from uchar. Code needs fixing.");
-    Q_ASSUME(gdk_pixbuf_get_bits_per_sample(buf) == 8);
-    Q_ASSUME(gdk_pixbuf_get_n_channels(buf) == 4);
+    Q_ASSERT(gdk_pixbuf_get_bits_per_sample(buf) == 8);
+    Q_ASSERT(gdk_pixbuf_get_n_channels(buf) == 4);
     const uchar *data = static_cast<const uchar *>(gdata);
 
     const int width = gdk_pixbuf_get_width(buf);
     const int height = gdk_pixbuf_get_height(buf);
     const int bpl = gdk_pixbuf_get_rowstride(buf);
-    QImage converted(data, width, height, bpl, QImage::Format_ARGB32);
-    return converted.copy(); // detatch to survive lifetime of buf
+    QImage converted(data, width, height, bpl, QImage::Format_RGBA8888);
+
+    // convert to more optimal format and detach to survive lifetime of buf
+    return converted.convertToFormat(QImage::Format_ARGB32_Premultiplied);
 }
 
 /*!
@@ -475,15 +478,18 @@ QBrush QGtk3Interface::brush(QGtkWidget wtype, QGtkColorSource source, GtkStateF
     \internal
     \brief Returns the name of the current GTK theme.
  */
-const QString QGtk3Interface::themeName() const
+QString QGtk3Interface::themeName() const
 {
-    gchar *theme_name;
-    GtkSettings *settings = gtk_settings_get_default();
-    if (!settings)
-        return QString();
+    QString name;
 
-    g_object_get(settings, "gtk-theme-name", &theme_name, nullptr);
-    return QLatin1StringView(theme_name);
+    if (GtkSettings *settings = gtk_settings_get_default()) {
+        gchar *theme_name;
+        g_object_get(settings, "gtk-theme-name", &theme_name, nullptr);
+        name = QLatin1StringView(theme_name);
+        g_free(theme_name);
+    }
+
+    return name;
 }
 
 /*!
@@ -537,13 +543,13 @@ inline constexpr QGtk3Interface::QGtkWidget QGtk3Interface::toWidgetType(QPlatfo
     case QPlatformTheme::ToolButtonFont: return QGtkWidget::gtk_button;
     case QPlatformTheme::ItemViewFont: return QGtkWidget::gtk_entry;
     case QPlatformTheme::ListViewFont: return QGtkWidget::gtk_tree_view;
-    case QPlatformTheme::HeaderViewFont: return QGtkWidget::gtk_fixed;
+    case QPlatformTheme::HeaderViewFont: return QGtkWidget::gtk_combo_box;
     case QPlatformTheme::ListBoxFont: return QGtkWidget::gtk_Default;
     case QPlatformTheme::ComboMenuItemFont: return QGtkWidget::gtk_combo_box;
     case QPlatformTheme::ComboLineEditFont: return QGtkWidget::gtk_combo_box_text;
     case QPlatformTheme::SmallFont: return QGtkWidget::gtk_Default;
     case QPlatformTheme::MiniFont: return QGtkWidget::gtk_Default;
-    case QPlatformTheme::FixedFont: return QGtkWidget::gtk_fixed;
+    case QPlatformTheme::FixedFont: return QGtkWidget::gtk_Default;
     case QPlatformTheme::GroupBoxTitleFont: return QGtkWidget::gtk_Default;
     case QPlatformTheme::TabButtonFont: return QGtkWidget::gtk_button;
     case QPlatformTheme::EditorFont: return QGtkWidget::gtk_entry;
@@ -604,6 +610,25 @@ QFont QGtk3Interface::font(QPlatformTheme::Font type) const
     if (!con)
         return QFont();
 
+    // explicitly add provider for fixed font
+    GtkCssProvider *cssProvider = nullptr;
+    if (type == QPlatformTheme::FixedFont) {
+        cssProvider = gtk_css_provider_new();
+        gtk_style_context_add_class (con, GTK_STYLE_CLASS_MONOSPACE);
+        const char *fontSpec = "* {font-family: monospace;}";
+        gtk_css_provider_load_from_data(cssProvider, fontSpec, -1, NULL);
+        gtk_style_context_add_provider(con, GTK_STYLE_PROVIDER(cssProvider),
+                                       GTK_STYLE_PROVIDER_PRIORITY_USER);
+    }
+
+    // remove monospace provider from style context and unref it
+    QScopeGuard guard([&](){
+        if (cssProvider) {
+            gtk_style_context_remove_provider(con, GTK_STYLE_PROVIDER(cssProvider));
+            g_object_unref(cssProvider);
+        }
+    });
+
     const PangoFontDescription *gtkFont = gtk_style_context_get_font(con, GTK_STATE_FLAG_NORMAL);
     if (!gtkFont)
         return QFont();
@@ -630,6 +655,7 @@ QFont QGtk3Interface::font(QPlatformTheme::Font type) const
             font.setFamily("monospace"_L1);
         }
     }
+
     return font;
 }
 
@@ -643,7 +669,7 @@ QIcon QGtk3Interface::fileIcon(const QFileInfo &fileInfo) const
     if (!file)
         return QIcon();
 
-    GFileInfo *info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+    GFileInfo *info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_ICON,
                                          G_FILE_QUERY_INFO_NONE, nullptr, nullptr);
     if (!info) {
         g_object_unref(file);
@@ -658,12 +684,11 @@ QIcon QGtk3Interface::fileIcon(const QFileInfo &fileInfo) const
     }
 
     GtkIconTheme *theme = gtk_icon_theme_get_default();
-    GtkIconInfo *iconInfo = gtk_icon_theme_lookup_by_gicon(theme, icon, GTK_ICON_SIZE_BUTTON,
+    GtkIconInfo *iconInfo = gtk_icon_theme_lookup_by_gicon(theme, icon, 16,
                                                                   GTK_ICON_LOOKUP_FORCE_SIZE);
     if (!iconInfo) {
         g_object_unref(file);
         g_object_unref(info);
-        g_object_unref(icon);
         return QIcon();
     }
 
@@ -671,7 +696,6 @@ QIcon QGtk3Interface::fileIcon(const QFileInfo &fileInfo) const
     QImage image = qt_convert_gdk_pixbuf(buf);
     g_object_unref(file);
     g_object_unref(info);
-    g_object_unref(icon);
     g_object_unref(buf);
     return QIcon(QPixmap::fromImage(image));
 }
