@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qcolordialog.h"
 
@@ -500,6 +501,12 @@ public:
         return false;
     }
 
+    void applicationStateChanged(Qt::ApplicationState state)
+    {
+        if (state != Qt::ApplicationActive)
+            m_dp->releaseColorPicking();
+    }
+
 private:
     QColorDialogPrivate *m_dp;
 };
@@ -1610,7 +1617,7 @@ void QColorDialogPrivate::pickScreenColor()
     Q_Q(QColorDialog);
 
     auto *platformServices = QGuiApplicationPrivate::platformIntegration()->services();
-    if (platformServices->hasCapability(QPlatformServices::Capability::ColorPicking)) {
+    if (platformServices && platformServices->hasCapability(QPlatformServices::Capability::ColorPicking)) {
         if (auto *colorPicker = platformServices->colorPicker(q->windowHandle())) {
             q->connect(colorPicker, &QPlatformServiceColorPicker::colorPicked, q,
                        [q, colorPicker](const QColor &color) {
@@ -1625,6 +1632,8 @@ void QColorDialogPrivate::pickScreenColor()
     if (!colorPickingEventFilter)
         colorPickingEventFilter = new QColorPickingEventFilter(this, q);
     q->installEventFilter(colorPickingEventFilter);
+    QObject::connect(qApp, &QGuiApplication::applicationStateChanged,
+                     colorPickingEventFilter, &QColorPickingEventFilter::applicationStateChanged);
     // If user pushes Escape, the last color before picking will be restored.
     beforeScreenColorPicking = cs->currentColor();
 #ifndef QT_NO_CURSOR
@@ -1671,6 +1680,8 @@ void QColorDialogPrivate::releaseColorPicking()
     Q_Q(QColorDialog);
     cp->setCrossVisible(true);
     q->removeEventFilter(colorPickingEventFilter);
+    QObject::disconnect(qApp, &QGuiApplication::applicationStateChanged,
+                        colorPickingEventFilter, &QColorPickingEventFilter::applicationStateChanged);
     q->releaseMouse();
 #ifdef Q_OS_WIN32
     updateTimer->stop();
@@ -2151,7 +2162,9 @@ void QColorDialog::setVisible(bool visible)
 */
 void QColorDialogPrivate::setVisible(bool visible)
 {
-    Q_Q(QColorDialog);
+    // Don't use Q_Q here! This function is called from ~QDialog,
+    // so Q_Q calling q_func() invokes undefined behavior (invalid cast in q_func()).
+    const auto q = static_cast<QDialog *>(q_ptr);
 
     if (visible)
         selectedQColor = QColor();
@@ -2161,7 +2174,7 @@ void QColorDialogPrivate::setVisible(bool visible)
             // Set WA_DontShowOnScreen so that QDialog::setVisible(visible) below
             // updates the state correctly, but skips showing the non-native version:
             q->setAttribute(Qt::WA_DontShowOnScreen);
-        } else {
+        } else if (visible) {
             initWidgets();
         }
     } else {
@@ -2197,13 +2210,19 @@ void QColorDialog::open(QObject *receiver, const char *member)
 QColor QColorDialog::getColor(const QColor &initial, QWidget *parent, const QString &title,
                               ColorDialogOptions options)
 {
-    QColorDialog dlg(parent);
+    QAutoPointer<QColorDialog> dlg(new QColorDialog(parent));
     if (!title.isEmpty())
-        dlg.setWindowTitle(title);
-    dlg.setOptions(options);
-    dlg.setCurrentColor(initial);
-    dlg.exec();
-    return dlg.selectedColor();
+        dlg->setWindowTitle(title);
+    dlg->setOptions(options);
+    dlg->setCurrentColor(initial);
+
+    // If the dlg was deleted with a parent window,
+    // dlg == nullptr after leaving the exec().
+    dlg->exec();
+    if (bool(dlg))
+        return dlg->selectedColor();
+    else
+        return QColor();
 }
 
 /*!
