@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qinputdialog.h"
 
@@ -1149,7 +1150,7 @@ QString QInputDialog::getText(QWidget *parent, const QString &title, const QStri
     const int ret = dialog->exec();
     if (ok)
         *ok = !!ret;
-    if (ret) {
+    if (bool(dialog) && ret) {
         return dialog->textValue();
     } else {
         return QString();
@@ -1197,7 +1198,7 @@ QString QInputDialog::getMultiLineText(QWidget *parent, const QString &title, co
     const int ret = dialog->exec();
     if (ok)
         *ok = !!ret;
-    if (ret) {
+    if (bool(dialog) && ret) {
         return dialog->textValue();
     } else {
         return QString();
@@ -1242,7 +1243,7 @@ int QInputDialog::getInt(QWidget *parent, const QString &title, const QString &l
     const int ret = dialog->exec();
     if (ok)
         *ok = !!ret;
-    if (ret) {
+    if (bool(dialog) && ret) {
         return dialog->intValue();
     } else {
         return value;
@@ -1291,7 +1292,7 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
     const int ret = dialog->exec();
     if (ok)
         *ok = !!ret;
-    if (ret) {
+    if (bool(dialog) && ret) {
         return dialog->doubleValue();
     } else {
         return value;
@@ -1344,7 +1345,7 @@ QString QInputDialog::getItem(QWidget *parent, const QString &title, const QStri
     const int ret = dialog->exec();
     if (ok)
         *ok = !!ret;
-    if (ret) {
+    if (bool(dialog) && ret) {
         return dialog->textValue();
     } else {
         return text;
