@@ -1,45 +1,9 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qlistwidget.h"
 
-#include <qitemdelegate.h>
 #include <private/qlistview_p.h>
 #include <private/qwidgetitemdata_p.h>
 #include <private/qlistwidget_p.h>
@@ -72,7 +36,7 @@ QListModel::~QListModel()
 void QListModel::clear()
 {
     beginResetModel();
-    for (int i = 0; i < items.count(); ++i) {
+    for (int i = 0; i < items.size(); ++i) {
         if (items.at(i)) {
             items.at(i)->d->theid = -1;
             items.at(i)->view = nullptr;
@@ -106,18 +70,18 @@ void QListModel::insert(int row, QListWidgetItem *item)
     if (!item)
         return;
 
-    item->view = qobject_cast<QListWidget*>(QObject::parent());
+    item->view = this->view();
     if (item->view && item->view->isSortingEnabled()) {
         // sorted insertion
         QList<QListWidgetItem*>::iterator it;
         it = sortedInsertionIterator(items.begin(), items.end(),
                                      item->view->sortOrder(), item);
-        row = qMax(it - items.begin(), 0);
+        row = qMax<qsizetype>(it - items.begin(), 0);
     } else {
         if (row < 0)
             row = 0;
-        else if (row > items.count())
-            row = items.count();
+        else if (row > items.size())
+            row = items.size();
     }
     beginInsertRows(QModelIndex(), row, row);
     items.insert(row, item);
@@ -127,10 +91,10 @@ void QListModel::insert(int row, QListWidgetItem *item)
 
 void QListModel::insert(int row, const QStringList &labels)
 {
-    const int count = labels.count();
+    const int count = labels.size();
     if (count <= 0)
         return;
-    QListWidget *view = qobject_cast<QListWidget*>(QObject::parent());
+    QListWidget *view = this->view();
     if (view && view->isSortingEnabled()) {
         // sorted insertion
         for (int i = 0; i < count; ++i) {
@@ -140,13 +104,13 @@ void QListModel::insert(int row, const QStringList &labels)
     } else {
         if (row < 0)
             row = 0;
-        else if (row > items.count())
-            row = items.count();
+        else if (row > items.size())
+            row = items.size();
         beginInsertRows(QModelIndex(), row, row + count - 1);
         for (int i = 0; i < count; ++i) {
             QListWidgetItem *item = new QListWidgetItem(labels.at(i));
             item->d->theid = row;
-            item->view = qobject_cast<QListWidget*>(QObject::parent());
+            item->view = this->view();
             items.insert(row++, item);
         }
         endInsertRows();
@@ -155,7 +119,7 @@ void QListModel::insert(int row, const QStringList &labels)
 
 QListWidgetItem *QListModel::take(int row)
 {
-    if (row < 0 || row >= items.count())
+    if (row < 0 || row >= items.size())
         return nullptr;
 
     beginRemoveRows(QModelIndex(), row, row);
@@ -169,8 +133,8 @@ QListWidgetItem *QListModel::take(int row)
 void QListModel::move(int srcRow, int dstRow)
 {
     if (srcRow == dstRow
-        || srcRow < 0 || srcRow >= items.count()
-        || dstRow < 0 || dstRow > items.count())
+        || srcRow < 0 || srcRow >= items.size()
+        || dstRow < 0 || dstRow > items.size())
         return;
 
     if (!beginMoveRows(QModelIndex(), srcRow, srcRow, QModelIndex(), dstRow))
@@ -183,7 +147,7 @@ void QListModel::move(int srcRow, int dstRow)
 
 int QListModel::rowCount(const QModelIndex &parent) const
 {
-    return parent.isValid() ? 0 : items.count();
+    return parent.isValid() ? 0 : items.size();
 }
 
 QModelIndex QListModel::index(const QListWidgetItem *item_) const
@@ -194,7 +158,7 @@ QModelIndex QListModel::index(const QListWidgetItem *item_) const
         return QModelIndex();
     int row;
     const int theid = item->d->theid;
-    if (theid >= 0 && theid < items.count() && items.at(theid) == item) {
+    if (theid >= 0 && theid < items.size() && items.at(theid) == item) {
         row = theid;
     } else { // we need to search for the item
         row = items.lastIndexOf(item);  // lastIndexOf is an optimization in favor of indexOf
@@ -214,20 +178,19 @@ QModelIndex QListModel::index(int row, int column, const QModelIndex &parent) co
 
 QVariant QListModel::data(const QModelIndex &index, int role) const
 {
-    if (!index.isValid() || index.row() >= items.count())
+    if (!index.isValid() || index.row() >= items.size())
         return QVariant();
     return items.at(index.row())->data(role);
 }
 
 bool QListModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-    if (!index.isValid() || index.row() >= items.count())
+    if (!index.isValid() || index.row() >= items.size())
         return false;
     items.at(index.row())->setData(role, value);
     return true;
 }
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 bool QListModel::clearItemData(const QModelIndex &index)
 {
     if (!checkIndex(index, CheckIndexOption::IndexIsValid))
@@ -238,18 +201,17 @@ bool QListModel::clearItemData(const QModelIndex &index)
     if (std::all_of(beginIter, endIter, [](const QWidgetItemData& data) -> bool { return !data.value.isValid(); }))
         return true; //it's already cleared
     item->d->values.clear();
-    emit dataChanged(index, index, QVector<int>{});
+    emit dataChanged(index, index, QList<int> {});
     return true;
 }
-#endif
 
 QMap<int, QVariant> QListModel::itemData(const QModelIndex &index) const
 {
     QMap<int, QVariant> roles;
-    if (!index.isValid() || index.row() >= items.count())
+    if (!index.isValid() || index.row() >= items.size())
         return roles;
     QListWidgetItem *itm = items.at(index.row());
-    for (int i = 0; i < itm->d->values.count(); ++i) {
+    for (int i = 0; i < itm->d->values.size(); ++i) {
         roles.insert(itm->d->values.at(i).role,
                      itm->d->values.at(i).value);
     }
@@ -262,7 +224,7 @@ bool QListModel::insertRows(int row, int count, const QModelIndex &parent)
         return false;
 
     beginInsertRows(QModelIndex(), row, row + count - 1);
-    QListWidget *view = qobject_cast<QListWidget*>(QObject::parent());
+    QListWidget *view = this->view();
     QListWidgetItem *itm = nullptr;
 
     for (int r = row; r < row + count; ++r) {
@@ -326,7 +288,7 @@ bool QListModel::moveRows(const QModelIndex &sourceParent, int sourceRow, int co
 
 Qt::ItemFlags QListModel::flags(const QModelIndex &index) const
 {
-    if (!index.isValid() || index.row() >= items.count() || index.model() != this)
+    if (!index.isValid() || index.row() >= items.size() || index.model() != this)
         return Qt::ItemIsDropEnabled; // we allow drops outside the items
     return items.at(index.row())->flags();
 }
@@ -338,18 +300,18 @@ void QListModel::sort(int column, Qt::SortOrder order)
 
     emit layoutAboutToBeChanged({}, QAbstractItemModel::VerticalSortHint);
 
-    QVector < QPair<QListWidgetItem*,int> > sorting(items.count());
-    for (int i = 0; i < items.count(); ++i) {
+    QList<std::pair<QListWidgetItem *, int>> sorting(items.size());
+    for (int i = 0; i < items.size(); ++i) {
         QListWidgetItem *item = items.at(i);
         sorting[i].first = item;
         sorting[i].second = i;
     }
 
     const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
-    std::sort(sorting.begin(), sorting.end(), compare);
+    std::stable_sort(sorting.begin(), sorting.end(), compare);
     QModelIndexList fromIndexes;
     QModelIndexList toIndexes;
-    const int sortingCount = sorting.count();
+    const int sortingCount = sorting.size();
     fromIndexes.reserve(sortingCount);
     toIndexes.reserve(sortingCount);
     for (int r = 0; r < sortingCount; ++r) {
@@ -366,82 +328,44 @@ void QListModel::sort(int column, Qt::SortOrder order)
 /**
  * This function assumes that all items in the model except the items that are between
  * (inclusive) start and end are sorted.
- * With these assumptions, this function can ensure that the model is sorted in a
- * much more efficient way than doing a naive 'sort everything'.
- * (provided that the range is relatively small compared to the total number of items)
  */
 void QListModel::ensureSorted(int column, Qt::SortOrder order, int start, int end)
 {
     if (column != 0)
         return;
 
-    int count = end - start + 1;
-    QVector < QPair<QListWidgetItem*,int> > sorting(count);
-    for (int i = 0; i < count; ++i) {
-        sorting[i].first = items.at(start + i);
-        sorting[i].second = start + i;
-    }
+    const auto compareLt = [](const QListWidgetItem *left, const QListWidgetItem *right) -> bool {
+        return *left < *right;
+    };
 
-    const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
-    std::sort(sorting.begin(), sorting.end(), compare);
+    const auto compareGt = [](const QListWidgetItem *left, const QListWidgetItem *right) -> bool {
+        return *right < *left;
+    };
 
-    QModelIndexList oldPersistentIndexes = persistentIndexList();
-    QModelIndexList newPersistentIndexes = oldPersistentIndexes;
-    QList<QListWidgetItem*> tmp = items;
-    QList<QListWidgetItem*>::iterator lit = tmp.begin();
-    bool changed = false;
-    for (int i = 0; i < count; ++i) {
-        int oldRow = sorting.at(i).second;
-        int tmpitepos = lit - tmp.begin();
-        QListWidgetItem *item = tmp.takeAt(oldRow);
-        if (tmpitepos > tmp.size())
-            --tmpitepos;
-        lit = tmp.begin() + tmpitepos;
-        lit = sortedInsertionIterator(lit, tmp.end(), order, item);
-        int newRow = qMax(lit - tmp.begin(), 0);
-        lit = tmp.insert(lit, item);
-        if (newRow != oldRow) {
-            changed = true;
-            for (int j = i + 1; j < count; ++j) {
-                int otherRow = sorting.at(j).second;
-                if (oldRow < otherRow && newRow >= otherRow)
-                    --sorting[j].second;
-                else if (oldRow > otherRow && newRow <= otherRow)
-                    ++sorting[j].second;
-            }
-            for (int k = 0; k < newPersistentIndexes.count(); ++k) {
-                QModelIndex pi = newPersistentIndexes.at(k);
-                int oldPersistentRow = pi.row();
-                int newPersistentRow = oldPersistentRow;
-                if (oldPersistentRow == oldRow)
-                    newPersistentRow = newRow;
-                else if (oldRow < oldPersistentRow && newRow >= oldPersistentRow)
-                    newPersistentRow = oldPersistentRow - 1;
-                else if (oldRow > oldPersistentRow && newRow <= oldPersistentRow)
-                    newPersistentRow = oldPersistentRow + 1;
-                if (newPersistentRow != oldPersistentRow)
-                    newPersistentIndexes[k] = createIndex(newPersistentRow,
-                                                          pi.column(), pi.internalPointer());
-            }
-        }
-    }
+    /** Check if range [start,end] is already in sorted position in list.
+     *  Take for this the assumption, that outside [start,end] the list
+     *  is already sorted. Therefore the sorted check has to be extended
+     *  to the first element that is known to be sorted before the range
+     *  [start, end], which is (start-1) and the first element after the
+     *  range [start, end], which is (end+2) due to end being included.
+    */
+    const auto beginChangedIterator = items.constBegin() + qMax(start - 1, 0);
+    const auto endChangedIterator = items.constBegin() + qMin(end + 2, items.size());
+    const bool needsSorting = !std::is_sorted(beginChangedIterator, endChangedIterator,
+                                              order == Qt::AscendingOrder ? compareLt : compareGt);
 
-    if (changed) {
-        emit layoutAboutToBeChanged();
-        items = tmp;
-        changePersistentIndexList(oldPersistentIndexes, newPersistentIndexes);
-        emit layoutChanged();
-    }
+    if (needsSorting)
+        sort(column, order);
 }
 
-bool QListModel::itemLessThan(const QPair<QListWidgetItem*,int> &left,
-                              const QPair<QListWidgetItem*,int> &right)
+bool QListModel::itemLessThan(const std::pair<QListWidgetItem*,int> &left,
+                              const std::pair<QListWidgetItem*,int> &right)
 {
     return (*left.first) < (*right.first);
 }
 
-bool QListModel::itemGreaterThan(const QPair<QListWidgetItem*,int> &left,
-                                 const QPair<QListWidgetItem*,int> &right)
+bool QListModel::itemGreaterThan(const std::pair<QListWidgetItem*,int> &left,
+                                 const std::pair<QListWidgetItem*,int> &right)
 {
     return (*right.first) < (*left.first);
 }
@@ -456,7 +380,7 @@ QList<QListWidgetItem*>::iterator QListModel::sortedInsertionIterator(
     return std::lower_bound(begin, end, item, QListModelGreaterThan());
 }
 
-void QListModel::itemChanged(QListWidgetItem *item, const QVector<int> &roles)
+void QListModel::itemChanged(QListWidgetItem *item, const QList<int> &roles)
 {
     const QModelIndex idx = index(item);
     emit dataChanged(idx, idx, roles);
@@ -464,8 +388,10 @@ void QListModel::itemChanged(QListWidgetItem *item, const QVector<int> &roles)
 
 QStringList QListModel::mimeTypes() const
 {
-    const QListWidget *view = qobject_cast<const QListWidget*>(QObject::parent());
-    return view->mimeTypes();
+    const QListWidget *view = this->view();
+    if (view)
+        return view->mimeTypes();
+    return {};
 }
 
 QMimeData *QListModel::internalMimeData()  const
@@ -476,14 +402,13 @@ QMimeData *QListModel::internalMimeData()  const
 QMimeData *QListModel::mimeData(const QModelIndexList &indexes) const
 {
     QList<QListWidgetItem*> itemlist;
-    const int indexesCount = indexes.count();
+    const int indexesCount = indexes.size();
     itemlist.reserve(indexesCount);
     for (int i = 0; i < indexesCount; ++i)
         itemlist << at(indexes.at(i).row());
-    const QListWidget *view = qobject_cast<const QListWidget*>(QObject::parent());
 
     cachedIndexes = indexes;
-    QMimeData *mimeData = view->mimeData(itemlist);
+    QMimeData *mimeData = view()->mimeData(itemlist);
     cachedIndexes.clear();
     return mimeData;
 }
@@ -493,20 +418,24 @@ bool QListModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
                               int row, int column, const QModelIndex &index)
 {
     Q_UNUSED(column);
-    QListWidget *view = qobject_cast<QListWidget*>(QObject::parent());
     if (index.isValid())
         row = index.row();
     else if (row == -1)
-        row = items.count();
+        row = items.size();
 
-    return view->dropMimeData(row, data, action);
+    return view()->dropMimeData(row, data, action);
 }
 
 Qt::DropActions QListModel::supportedDropActions() const
 {
-    const QListWidget *view = qobject_cast<const QListWidget*>(QObject::parent());
-    return view->supportedDropActions();
+    return view()->supportedDropActions();
 }
+
+Qt::DropActions QListModel::supportedDragActions() const
+{
+    return view()->supportedDragActions();
+}
+
 #endif // QT_CONFIG(draganddrop)
 
 /*!
@@ -591,7 +520,6 @@ Qt::DropActions QListModel::supportedDropActions() const
 
 /*!
     \fn void QListWidgetItem::setHidden(bool hide)
-    \since 4.2
 
     Hides the item if \a hide is true; otherwise shows the item.
 
@@ -600,7 +528,6 @@ Qt::DropActions QListModel::supportedDropActions() const
 
 /*!
     \fn bool QListWidgetItem::isHidden() const
-    \since 4.2
 
     Returns \c true if the item is hidden; otherwise returns \c false.
 
@@ -734,7 +661,7 @@ void QListWidgetItem::setData(int role, const QVariant &value)
 {
     bool found = false;
     role = (role == Qt::EditRole ? Qt::DisplayRole : role);
-    for (int i = 0; i < d->values.count(); ++i) {
+    for (int i = 0; i < d->values.size(); ++i) {
         if (d->values.at(i).role == role) {
             if (d->values.at(i).value == value)
                 return;
@@ -746,9 +673,9 @@ void QListWidgetItem::setData(int role, const QVariant &value)
     if (!found)
         d->values.append(QWidgetItemData(role, value));
     if (QListModel *model = listModel()) {
-        const QVector<int> roles((role == Qt::DisplayRole) ?
-                                    QVector<int>({Qt::DisplayRole, Qt::EditRole}) :
-                                    QVector<int>({role}));
+        const QList<int> roles((role == Qt::DisplayRole)
+                                       ? QList<int>({ Qt::DisplayRole, Qt::EditRole })
+                                       : QList<int>({ role }));
         model->itemChanged(this, roles);
     }
 }
@@ -762,7 +689,7 @@ void QListWidgetItem::setData(int role, const QVariant &value)
 QVariant QListWidgetItem::data(int role) const
 {
     role = (role == Qt::EditRole ? Qt::DisplayRole : role);
-    for (int i = 0; i < d->values.count(); ++i)
+    for (int i = 0; i < d->values.size(); ++i)
         if (d->values.at(i).role == role)
             return d->values.at(i).value;
     return QVariant();
@@ -802,8 +729,6 @@ void QListWidgetItem::write(QDataStream &out) const
 #endif // QT_NO_DATASTREAM
 
 /*!
-    \since 4.1
-
     Constructs a copy of \a other. Note that type() and listWidget() are not
     copied.
 
@@ -930,45 +855,36 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
 */
 
 /*!
+    \if defined(qt7)
+
+    \fn Qt::Alignment QListWidgetItem::textAlignment() const
+
+    Returns the text alignment for the list item.
+
+    \else
+
     \fn int QListWidgetItem::textAlignment() const
 
     Returns the text alignment for the list item.
 
-    \sa Qt::AlignmentFlag
-*/
+    \note This function returns an int for historical reasons. It will
+    be corrected to return Qt::Alignment in Qt 7.
 
-#if QT_DEPRECATED_SINCE(5, 13)
-/*!
-    \fn QColor QListWidgetItem::backgroundColor() const
-    \obsolete
+    \sa Qt::Alignment
 
-    This function is deprecated. Use background() instead.
+    \endif
 */
-#endif
 
 /*!
     \fn QBrush QListWidgetItem::background() const
-    \since 4.2
 
     Returns the brush used to display the list item's background.
 
     \sa setBackground(), foreground()
 */
 
-#if QT_DEPRECATED_SINCE(5, 13)
-/*!
-    \fn QColor QListWidgetItem::textColor() const
-    \obsolete
-
-    Returns the color used to display the list item's text.
-
-    This function is deprecated. Use foreground() instead.
-*/
-#endif
-
 /*!
     \fn QBrush QListWidgetItem::foreground() const
-    \since 4.2
 
     Returns the brush used to display the list item's foreground (e.g. text).
 
@@ -985,14 +901,12 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
 
 /*!
     \fn QSize QListWidgetItem::sizeHint() const
-    \since 4.1
 
     Returns the size hint set for the list item.
 */
 
 /*!
     \fn void QListWidgetItem::setSizeHint(const QSize &size)
-    \since 4.1
 
     Sets the size hint for the list item to be \a size.
     If no size hint is set or \a size is invalid, the item
@@ -1001,7 +915,6 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
 
 /*!
     \fn void QListWidgetItem::setSelected(bool select)
-    \since 4.2
 
     Sets the selected state of the item to \a select.
 
@@ -1028,7 +941,6 @@ void QListWidgetItem::setSelected(bool select)
 
 /*!
     \fn bool QListWidgetItem::isSelected() const
-    \since 4.2
 
     Returns \c true if the item is selected; otherwise returns \c false.
 
@@ -1108,23 +1020,30 @@ void QListWidgetItem::setFlags(Qt::ItemFlags aflags)
 */
 
 /*!
+    \obsolete [6.4] Use the overload that takes a Qt::Alignment argument.
+
     \fn void QListWidgetItem::setTextAlignment(int alignment)
 
     Sets the list item's text alignment to \a alignment.
 
-    \sa Qt::AlignmentFlag
+    \sa Qt::Alignment
 */
 
 /*!
-    \fn void QListWidgetItem::setBackgroundColor(const QColor &color)
-    \obsolete
+    \since 6.4
 
-    This function is deprecated. Use setBackground() instead.
+    \fn void QListWidgetItem::setTextAlignment(Qt::Alignment alignment)
+
+    Sets the list item's text alignment to \a alignment.
+*/
+
+/*!
+    \fn void QListWidgetItem::setTextAlignment(Qt::AlignmentFlag alignment)
+    \internal
 */
 
 /*!
     \fn void QListWidgetItem::setBackground(const QBrush &brush)
-    \since 4.2
 
     Sets the background brush of the list item to the given \a brush.
     Setting a default-constructed brush will let the view use the
@@ -1133,18 +1052,8 @@ void QListWidgetItem::setFlags(Qt::ItemFlags aflags)
     \sa background(), setForeground()
 */
 
-#if QT_DEPRECATED_SINCE(5, 13)
-/*!
-    \fn void QListWidgetItem::setTextColor(const QColor &color)
-    \obsolete
-
-    This function is deprecated. Use setForeground() instead.
-*/
-#endif
-
 /*!
     \fn void QListWidgetItem::setForeground(const QBrush &brush)
-    \since 4.2
 
     Sets the foreground brush of the list item to the given \a brush.
     Setting a default-constructed brush will let the view use the
@@ -1166,57 +1075,71 @@ void QListWidgetPrivate::setup()
     Q_Q(QListWidget);
     q->QListView::setModel(new QListModel(q));
     // view signals
-    QObject::connect(q, SIGNAL(pressed(QModelIndex)), q, SLOT(_q_emitItemPressed(QModelIndex)));
-    QObject::connect(q, SIGNAL(clicked(QModelIndex)), q, SLOT(_q_emitItemClicked(QModelIndex)));
-    QObject::connect(q, SIGNAL(doubleClicked(QModelIndex)),
-                     q, SLOT(_q_emitItemDoubleClicked(QModelIndex)));
-    QObject::connect(q, SIGNAL(activated(QModelIndex)),
-                     q, SLOT(_q_emitItemActivated(QModelIndex)));
-    QObject::connect(q, SIGNAL(entered(QModelIndex)), q, SLOT(_q_emitItemEntered(QModelIndex)));
-    QObject::connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                     q, SLOT(_q_emitItemChanged(QModelIndex)));
-    QObject::connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                     q, SLOT(_q_dataChanged(QModelIndex,QModelIndex)));
-    QObject::connect(model, SIGNAL(columnsRemoved(QModelIndex,int,int)), q, SLOT(_q_sort()));
+    connections = {
+        QObjectPrivate::connect(q, &QListWidget::pressed,
+                                this, &QListWidgetPrivate::emitItemPressed),
+        QObjectPrivate::connect(q, &QListWidget::clicked,
+                                this, &QListWidgetPrivate::emitItemClicked),
+        QObjectPrivate::connect(q, &QListWidget::doubleClicked,
+                                this, &QListWidgetPrivate::emitItemDoubleClicked),
+        QObjectPrivate::connect(q, &QListWidget::activated,
+                                this, &QListWidgetPrivate::emitItemActivated),
+        QObjectPrivate::connect(q, &QListWidget::entered,
+                                this, &QListWidgetPrivate::emitItemEntered),
+        QObjectPrivate::connect(model, &QAbstractItemModel::dataChanged,
+                                this, &QListWidgetPrivate::emitItemChanged),
+        QObjectPrivate::connect(model, &QAbstractItemModel::dataChanged,
+                                this, &QListWidgetPrivate::dataChanged),
+        QObjectPrivate::connect(model, &QAbstractItemModel::columnsRemoved,
+                                this, &QListWidgetPrivate::sort)
+    };
 }
 
-void QListWidgetPrivate::_q_emitItemPressed(const QModelIndex &index)
+void QListWidgetPrivate::clearConnections()
+{
+    for (const QMetaObject::Connection &connection : connections)
+        QObject::disconnect(connection);
+    for (const QMetaObject::Connection &connection : selectionModelConnections)
+        QObject::disconnect(connection);
+}
+
+void QListWidgetPrivate::emitItemPressed(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemPressed(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitItemClicked(const QModelIndex &index)
+void QListWidgetPrivate::emitItemClicked(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemClicked(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitItemDoubleClicked(const QModelIndex &index)
+void QListWidgetPrivate::emitItemDoubleClicked(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemDoubleClicked(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitItemActivated(const QModelIndex &index)
+void QListWidgetPrivate::emitItemActivated(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemActivated(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitItemEntered(const QModelIndex &index)
+void QListWidgetPrivate::emitItemEntered(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemEntered(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitItemChanged(const QModelIndex &index)
+void QListWidgetPrivate::emitItemChanged(const QModelIndex &index)
 {
     Q_Q(QListWidget);
     emit q->itemChanged(listModel()->at(index.row()));
 }
 
-void QListWidgetPrivate::_q_emitCurrentItemChanged(const QModelIndex &current,
+void QListWidgetPrivate::emitCurrentItemChanged(const QModelIndex &current,
                                                 const QModelIndex &previous)
 {
     Q_Q(QListWidget);
@@ -1234,14 +1157,14 @@ void QListWidgetPrivate::_q_emitCurrentItemChanged(const QModelIndex &current,
     emit q->currentRowChanged(persistentCurrent.row());
 }
 
-void QListWidgetPrivate::_q_sort()
+void QListWidgetPrivate::sort()
 {
     if (sortingEnabled)
         model->sort(0, sortOrder);
 }
 
-void QListWidgetPrivate::_q_dataChanged(const QModelIndex &topLeft,
-                                        const QModelIndex &bottomRight)
+void QListWidgetPrivate::dataChanged(const QModelIndex &topLeft,
+                                     const QModelIndex &bottomRight)
 {
     if (sortingEnabled && topLeft.isValid() && bottomRight.isValid())
         listModel()->ensureSorted(topLeft.column(), sortOrder,
@@ -1255,7 +1178,7 @@ void QListWidgetPrivate::_q_dataChanged(const QModelIndex &topLeft,
     \ingroup model-view
     \inmodule QtWidgets
 
-    \image windows-listview.png
+    \image fusion-listview.png {List of weather icons}
 
     QListWidget is a convenience class that provides a list view similar to the
     one supplied by QListView, but with a classic item-based interface for
@@ -1418,8 +1341,6 @@ void QListWidgetPrivate::_q_dataChanged(const QModelIndex &topLeft,
 */
 
 /*!
-    \since 4.3
-
     \fn void QListWidget::removeItemWidget(QListWidgetItem *item)
 
     Removes the widget set on the given \a item.
@@ -1447,6 +1368,8 @@ QListWidget::QListWidget(QWidget *parent)
 
 QListWidget::~QListWidget()
 {
+    Q_D(QListWidget);
+    d->clearConnections();
 }
 
 /*!
@@ -1457,20 +1380,18 @@ void QListWidget::setSelectionModel(QItemSelectionModel *selectionModel)
 {
     Q_D(QListWidget);
 
-    if (d->selectionModel) {
-        QObject::disconnect(d->selectionModel, SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-                            this, SLOT(_q_emitCurrentItemChanged(QModelIndex,QModelIndex)));
-        QObject::disconnect(d->selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-                            this, SIGNAL(itemSelectionChanged()));
-    }
+    for (const QMetaObject::Connection &connection : d->selectionModelConnections)
+        disconnect(connection);
 
     QListView::setSelectionModel(selectionModel);
 
     if (d->selectionModel) {
-        QObject::connect(d->selectionModel, SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-                         this, SLOT(_q_emitCurrentItemChanged(QModelIndex,QModelIndex)));
-        QObject::connect(d->selectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-                         this, SIGNAL(itemSelectionChanged()));
+        d->selectionModelConnections = {
+            QObjectPrivate::connect(d->selectionModel, &QItemSelectionModel::currentChanged,
+                                    d, &QListWidgetPrivate::emitCurrentItemChanged),
+            QObject::connect(d->selectionModel, &QItemSelectionModel::selectionChanged,
+                             this, &QListWidget::itemSelectionChanged)
+        };
     }
 }
 
@@ -1592,7 +1513,6 @@ void QListWidget::setCurrentItem(QListWidgetItem *item)
 }
 
 /*!
-    \since 4.4
     Set the current item to \a item, using the given \a command.
 */
 void QListWidget::setCurrentItem(QListWidgetItem *item, QItemSelectionModel::SelectionFlags command)
@@ -1625,8 +1545,6 @@ void QListWidget::setCurrentRow(int row)
 }
 
 /*!
-    \since 4.4
-
     Sets the current row to be the given \a row, using the given \a command,
 */
 void QListWidget::setCurrentRow(int row, QItemSelectionModel::SelectionFlags command)
@@ -1679,7 +1597,6 @@ void QListWidget::sortItems(Qt::SortOrder order)
 }
 
 /*!
-    \since 4.2
     \property QListWidget::sortingEnabled
     \brief whether sorting is enabled
 
@@ -1759,8 +1676,6 @@ bool QListWidget::isPersistentEditorOpen(QListWidgetItem *item) const
 }
 
 /*!
-    \since 4.1
-
     Returns the widget displayed in the given \a item.
 
     \sa setItemWidget(), removeItemWidget()
@@ -1773,8 +1688,6 @@ QWidget *QListWidget::itemWidget(QListWidgetItem *item) const
 }
 
 /*!
-    \since 4.1
-
     Sets the \a widget to be displayed in the given \a item.
 
     This function should only be used to display static content in the place of
@@ -1782,6 +1695,8 @@ QWidget *QListWidget::itemWidget(QListWidgetItem *item) const
     implement a custom editor widget, use QListView and subclass QStyledItemDelegate
     instead.
 
+    \note The list takes ownership of the \a widget.
+
     \sa itemWidget(), removeItemWidget(), {Delegate Classes}
 */
 void QListWidget::setItemWidget(QListWidgetItem *item, QWidget *widget)
@@ -1791,34 +1706,6 @@ void QListWidget::setItemWidget(QListWidgetItem *item, QWidget *widget)
     QAbstractItemView::setIndexWidget(index, widget);
 }
 
-#if QT_DEPRECATED_SINCE(5, 13)
-/*!
-    Returns \c true if \a item is selected; otherwise returns \c false.
-
-    \obsolete
-
-    This function is deprecated. Use QListWidgetItem::isSelected() instead.
-*/
-bool QListWidget::isItemSelected(const QListWidgetItem *item) const
-{
-    return ((item && item->listWidget() == this) ? item->isSelected() : false);
-}
-
-/*!
-    Selects or deselects the given \a item depending on whether \a select is
-    true of false.
-
-    \obsolete
-
-    This function is deprecated. Use QListWidgetItem::setSelected() instead.
-*/
-void QListWidget::setItemSelected(const QListWidgetItem *item, bool select)
-{
-    if (item && item->listWidget() == this)
-        const_cast<QListWidgetItem*>(item)->setSelected(select);
-}
-#endif
-
 /*!
     Returns a list of all selected items in the list widget.
 */
@@ -1828,7 +1715,7 @@ QList<QListWidgetItem*> QListWidget::selectedItems() const
     Q_D(const QListWidget);
     QModelIndexList indexes = selectionModel()->selectedIndexes();
     QList<QListWidgetItem*> items;
-    const int numIndexes = indexes.count();
+    const int numIndexes = indexes.size();
     items.reserve(numIndexes);
     for (int i = 0; i < numIndexes; ++i)
         items.append(d->listModel()->at(indexes.at(i).row()));
@@ -1853,32 +1740,6 @@ QList<QListWidgetItem*> QListWidget::findItems(const QString &text, Qt::MatchFla
     return items;
 }
 
-#if QT_DEPRECATED_SINCE(5, 13)
-/*!
-    Returns \c true if the \a item is explicitly hidden; otherwise returns \c false.
-
-    \obsolete
-
-    This function is deprecated. Use QListWidgetItem::isHidden() instead.
-*/
-bool QListWidget::isItemHidden(const QListWidgetItem *item) const
-{
-    return isRowHidden(row(item));
-}
-
-/*!
-    If \a hide is true, the \a item will be hidden; otherwise it will be shown.
-
-    \obsolete
-
-    This function is deprecated. Use QListWidgetItem::setHidden() instead.
-*/
-void QListWidget::setItemHidden(const QListWidgetItem *item, bool hide)
-{
-    setRowHidden(row(item), hide);
-}
-#endif
-
 /*!
     Scrolls the view if necessary to ensure that the \a item is visible.
 
@@ -1923,11 +1784,7 @@ QStringList QListWidget::mimeTypes() const
     If the list of items is empty, \nullptr is returned instead of a
     serialized empty list.
 */
-#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
 QMimeData *QListWidget::mimeData(const QList<QListWidgetItem *> &items) const
-#else
-QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*> items) const
-#endif
 {
     Q_D(const QListWidget);
 
@@ -1935,7 +1792,7 @@ QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*> items) const
 
     // if non empty, it's called from the model's own mimeData
     if (cachedIndexes.isEmpty()) {
-        cachedIndexes.reserve(items.count());
+        cachedIndexes.reserve(items.size());
         for (QListWidgetItem *item : items)
             cachedIndexes << indexFromItem(item);
 
@@ -1949,12 +1806,13 @@ QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*> items) const
 }
 
 #if QT_CONFIG(draganddrop)
+
 /*!
     Handles \a data supplied by an external drag and drop operation that ended
     with the given \a action in the given \a index. Returns \c true if \a data and
     \a action can be handled by the model; otherwise returns \c false.
 
-    \sa supportedDropActions()
+    \sa supportedDropActions(), supportedDragActions
 */
 bool QListWidget::dropMimeData(int index, const QMimeData *data, Qt::DropAction action)
 {
@@ -1971,58 +1829,41 @@ bool QListWidget::dropMimeData(int index, const QMimeData *data, Qt::DropAction
 }
 
 /*! \reimp */
-void QListWidget::dropEvent(QDropEvent *event) {
-    Q_D(QListWidget);
-    if (event->source() == this && d->movement != Static) {
-        QListView::dropEvent(event);
-        return;
-    }
-
-    if (event->source() == this && (event->dropAction() == Qt::MoveAction ||
-                                    dragDropMode() == QAbstractItemView::InternalMove)) {
-        QModelIndex topIndex;
-        int col = -1;
-        int row = -1;
-        if (d->dropOn(event, &row, &col, &topIndex)) {
-            QList<QModelIndex> selIndexes = selectedIndexes();
-            QList<QPersistentModelIndex> persIndexes;
-            const int selIndexesCount = selIndexes.count();
-            persIndexes.reserve(selIndexesCount);
-            for (int i = 0; i < selIndexesCount; i++)
-                persIndexes.append(selIndexes.at(i));
-
-            if (persIndexes.contains(topIndex))
-                return;
-            std::sort(persIndexes.begin(), persIndexes.end()); // The dropped items will remain in the same visual order.
-
-            QPersistentModelIndex dropRow = model()->index(row, col, topIndex);
-
-            int r = row == -1 ? count() : (dropRow.row() >= 0 ? dropRow.row() : row);
-            for (int i = 0; i < persIndexes.count(); ++i) {
-                const QPersistentModelIndex &pIndex = persIndexes.at(i);
-                d->listModel()->move(pIndex.row(), r);
-                r = pIndex.row() + 1;   // Dropped items are inserted contiguously and in the right order.
-            }
-
-            event->accept();
-            // Don't want QAbstractItemView to delete it because it was "moved" we already did it
-            event->setDropAction(Qt::CopyAction);
-        }
-    }
-
+void QListWidget::dropEvent(QDropEvent *event)
+{
     QListView::dropEvent(event);
 }
 
 /*!
     Returns the drop actions supported by this view.
 
-    \sa Qt::DropActions
+    \sa Qt::DropActions, supportedDragActions, dropMimeData()
 */
 Qt::DropActions QListWidget::supportedDropActions() const
 {
     Q_D(const QListWidget);
     return d->listModel()->QAbstractListModel::supportedDropActions() | Qt::MoveAction;
 }
+
+/*!
+    \property QListWidget::supportedDragActions
+    \brief the drag actions supported by this view
+
+    \since 6.10
+    \sa Qt::DropActions, supportedDropActions()
+*/
+Qt::DropActions QListWidget::supportedDragActions() const
+{
+    Q_D(const QListWidget);
+    return d->supportedDragActions.value_or(supportedDropActions());
+}
+
+void QListWidget::setSupportedDragActions(Qt::DropActions actions)
+{
+    Q_D(QListWidget);
+    d->supportedDragActions = actions;
+}
+
 #endif // QT_CONFIG(draganddrop)
 
 /*!
@@ -2050,18 +1891,6 @@ QModelIndex QListWidget::indexFromItem(const QListWidgetItem *item) const
     return d->listModel()->index(item);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-/*!
-    \internal
-    \obsolete
-    \overload
-*/
-QModelIndex QListWidget::indexFromItem(QListWidgetItem *item) const
-{
-    return indexFromItem(const_cast<const QListWidgetItem *>(item));
-}
-#endif
-
 /*!
     Returns a pointer to the QListWidgetItem associated with the given \a index.
 */
