@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
 
@@ -16,7 +17,9 @@
 #include <qdebug.h>
 #include <qpa/qplatformscreen.h>
 #include <qpa/qplatformcursor.h>
+#if QT_CONFIG(style_stylesheet)
 #include <private/qstylesheetstyle_p.h>
+#endif
 
 #include <qlabel.h>
 #include <QtWidgets/private/qlabel_p.h>
@@ -120,7 +123,7 @@ protected:
     void mouseMoveEvent(QMouseEvent *e) override;
     void resizeEvent(QResizeEvent *e) override;
 
-#ifndef QT_NO_STYLE_STYLESHEET
+#if QT_CONFIG(style_stylesheet)
 public slots:
     /** \internal
       Cleanup the _q_stylesheet_parent property.
@@ -143,7 +146,7 @@ QTipLabel *QTipLabel::instance = nullptr;
 
 QTipLabel::QTipLabel(const QString &text, const QPoint &pos, QWidget *w, int msecDisplayTime)
     : QLabel(w, Qt::ToolTip | Qt::BypassGraphicsProxyWidget)
-#ifndef QT_NO_STYLE_STYLESHEET
+#if QT_CONFIG(style_stylesheet)
     , styleSheetParent(nullptr)
 #endif
     , widget(nullptr)
@@ -178,7 +181,7 @@ void QTipLabel::restartExpireTimer(int msecDisplayTime)
 
 void QTipLabel::reuseTip(const QString &text, int msecDisplayTime, const QPoint &pos)
 {
-#ifndef QT_NO_STYLE_STYLESHEET
+#if QT_CONFIG(style_stylesheet)
     if (styleSheetParent){
         disconnect(styleSheetParent, &QWidget::destroyed,
                    this, &QTipLabel::styleSheetParentDestroyed);
@@ -346,7 +349,7 @@ QScreen *QTipLabel::getTipScreen(const QPoint &pos, QWidget *w)
 
 void QTipLabel::placeTip(const QPoint &pos, QWidget *w)
 {
-#ifndef QT_NO_STYLE_STYLESHEET
+#if QT_CONFIG(style_stylesheet)
     if (testAttribute(Qt::WA_StyleSheet) || (w && qt_styleSheet(w->style()))) {
         //the stylesheet need to know the real parent
         QTipLabel::instance->setProperty("_q_stylesheet_parent", QVariant::fromValue(w));
@@ -365,7 +368,7 @@ void QTipLabel::placeTip(const QPoint &pos, QWidget *w)
         // correct content margin.
         QTipLabel::instance->updateSize(pos);
     }
-#endif //QT_NO_STYLE_STYLESHEET
+#endif //QT_CONFIG(style_stylesheet)
 
     QPoint p = pos;
     const QScreen *screen = getTipScreen(pos, w);
@@ -385,9 +388,9 @@ void QTipLabel::placeTip(const QPoint &pos, QWidget *w)
 
         QRect screenRect = screen->geometry();
         if (p.x() + this->width() > screenRect.x() + screenRect.width())
-        p.rx() -= 4 + this->width();
+            p.rx() -= 4 + this->width();
         if (p.y() + this->height() > screenRect.y() + screenRect.height())
-        p.ry() -= 24 + this->height();
+            p.ry() -= 24 + this->height();
         if (p.y() < screenRect.y())
             p.setY(screenRect.y());
         if (p.x() + this->width() > screenRect.x() + screenRect.width())
