@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qgesture.h"
 #include "qapplication.h"
@@ -16,6 +17,7 @@
 #include "private/qapplication_p.h"
 #include "private/qevent_p.h"
 #include "private/qflickgesture_p.h"
+#include "qbasictimer.h"
 #include "qdebug.h"
 
 #ifndef QT_NO_GESTURES
@@ -63,7 +65,6 @@ class PressDelayHandler : public QObject
 private:
     PressDelayHandler(QObject *parent = nullptr)
         : QObject(parent)
-        , pressDelayTimer(0)
         , sendingEvent(false)
         , mouseButton(Qt::NoButton)
         , mouseTarget(nullptr)
@@ -98,7 +99,8 @@ public:
     {
         if (!pressDelayEvent) {
             pressDelayEvent.reset(copyMouseEvent(e));
-            pressDelayTimer = startTimer(delay);
+            using namespace std::chrono_literals;
+            pressDelayTimer.start(delay * 1ms, this);
             mouseTarget = QApplication::widgetAt(pressDelayEvent->globalPosition().toPoint());
             mouseButton = pressDelayEvent->button();
             mouseEventSource = pressDelayEvent->source();
@@ -115,10 +117,9 @@ public:
         bool result = scrollerWasActive || scrollerIsActive;
 
         // stop the timer
-        if (pressDelayTimer) {
-            killTimer(pressDelayTimer);
-            pressDelayTimer = 0;
-        }
+        if (pressDelayTimer.isActive())
+            pressDelayTimer.stop();
+
         // we still haven't even sent the press, so do it now
         if (pressDelayEvent && mouseTarget && !scrollerIsActive) {
             QScopedPointer<QMouseEvent> releaseEvent(copyMouseEvent(e));
@@ -144,10 +145,8 @@ public:
         qFGDebug("QFG: deleting delayed mouse press, since scroller was only intercepted");
         if (pressDelayEvent) {
             // we still haven't even sent the press, so just throw it away now
-            if (pressDelayTimer) {
-                killTimer(pressDelayTimer);
-                pressDelayTimer = 0;
-            }
+            if (pressDelayTimer.isActive())
+                pressDelayTimer.stop();
             pressDelayEvent.reset(nullptr);
         }
         mouseTarget = nullptr;
@@ -158,10 +157,8 @@ public:
         if (pressDelayEvent) {
             // we still haven't even sent the press, so just throw it away now
             qFGDebug("QFG: deleting delayed mouse press, since scroller is active now");
-            if (pressDelayTimer) {
-                killTimer(pressDelayTimer);
-                pressDelayTimer = 0;
-            }
+            if (pressDelayTimer.isActive())
+                pressDelayTimer.stop();
             pressDelayEvent.reset(nullptr);
             mouseTarget = nullptr;
         } else if (mouseTarget) {
@@ -180,17 +177,15 @@ public:
 protected:
     void timerEvent(QTimerEvent *e) override
     {
-        if (e->timerId() == pressDelayTimer) {
+        if (e->id() == pressDelayTimer.id()) {
             if (pressDelayEvent && mouseTarget) {
                 qFGDebug() << "QFG: timer event: re-sending mouse press to " << mouseTarget;
                 sendMouseEvent(pressDelayEvent.data(), UngrabMouseBefore);
             }
             pressDelayEvent.reset(nullptr);
 
-            if (pressDelayTimer) {
-                killTimer(pressDelayTimer);
-                pressDelayTimer = 0;
-            }
+            if (pressDelayTimer.isActive())
+                pressDelayTimer.stop();
         }
     }
 
@@ -246,7 +241,7 @@ protected:
 
 
 private:
-    int pressDelayTimer;
+    QBasicTimer pressDelayTimer;
     QScopedPointer<QMouseEvent> pressDelayEvent;
     bool sendingEvent;
     Qt::MouseButton mouseButton;
@@ -324,7 +319,7 @@ QGestureRecognizer::Result QFlickGestureRecognizer::recognize(QGesture *state,
 {
     Q_UNUSED(watched);
 
-    static QElapsedTimer monotonicTimer;
+    Q_CONSTINIT static QElapsedTimer monotonicTimer;
     if (!monotonicTimer.isValid())
         monotonicTimer.start();
 
