@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qpainter.h"
 #include "qevent.h"
@@ -51,7 +52,9 @@ QLabelPrivate::~QLabelPrivate()
     \ingroup basicwidgets
     \inmodule QtWidgets
 
-    \image fusion-label.png
+    \image fusion-label.png {Label}
+
+
 
     QLabel is used for displaying text or an image. No user
     interaction functionality is provided. The visual appearance of
@@ -1057,7 +1060,12 @@ void QLabel::paintEvent(QPaintEvent *)
         const QSize size = d->scaledcontents ? cr.size() : d->pixmapSize;
         const auto mode = isEnabled() ? QIcon::Normal : QIcon::Disabled;
         QPixmap pix = d->icon->pixmap(size, dpr, mode);
-        if (d->scaledcontents && pix.size() != size * dpr) {
+        // the size of the returned pixmap might not match when
+        //  - scaledContents is enabled
+        //  - the dpr does not match the one from the pixmap in QIcon
+        // since QStyle::drawItemPixmap() stretches without Qt::SmoothTransformation
+        // we do it here
+        if (pix.size() != size * dpr) {
             const QString key = "qt_label_"_L1 % HexString<quint64>(pix.cacheKey())
                                                % HexString<quint8>(mode)
                                                % HexString<uint>(size.width())
