@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qmainwindow.h"
 #include "qmainwindowlayout_p.h"
@@ -130,6 +131,7 @@ void QMainWindowPrivate::init()
     layout below.
 
     \image mainwindowlayout.png
+           {Diagram of main window and the position of its components}
 
     \section1 Creating Main Window Components
 
