@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "private/qlayoutengine_p.h"
 #if QT_CONFIG(itemviews)
@@ -35,6 +36,7 @@
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
+using namespace std::chrono_literals;
 
 namespace {
 class CloseButton : public QAbstractButton
@@ -105,14 +107,16 @@ void QTabBarPrivate::updateMacBorderMetrics()
     if (!function)
         return; // Not Cocoa platform plugin.
     typedef void (*RegisterContentBorderAreaFunction)(QWindow *window, quintptr identifier, int upper, int lower);
-    (reinterpret_cast<RegisterContentBorderAreaFunction>(function))(q->window()->windowHandle(), identifier, upper, lower);
+    (reinterpret_cast<RegisterContentBorderAreaFunction>(QFunctionPointer(function)))(
+        q->window()->windowHandle(), identifier, upper, lower);
 
     // Set visibility state
     function = nativeInterface->nativeResourceFunctionForIntegration("setContentBorderAreaEnabled");
     if (!function)
         return;
     typedef void (*SetContentBorderAreaEnabledFunction)(QWindow *window, quintptr identifier, bool enable);
-    (reinterpret_cast<SetContentBorderAreaEnabledFunction>(function))(q->window()->windowHandle(), identifier, q->isVisible());
+    (reinterpret_cast<SetContentBorderAreaEnabledFunction>(QFunctionPointer(function)))(
+        q->window()->windowHandle(), identifier, q->isVisible());
 #endif
 }
 
@@ -189,6 +193,8 @@ void QTabBarPrivate::initBasicStyleOption(QStyleOptionTab *option, int tabIndex)
             option->cornerWidgets |= QStyleOptionTab::RightCornerWidget;
     }
 #endif
+    if (tab.measuringMinimum)
+        option->features |= QStyleOptionTab::MinimumSizeHint;
     option->tabIndex = tabIndex;
 }
 
@@ -275,9 +281,9 @@ void QTabBar::initStyleOption(QStyleOptionTab *option, int tabIndex) const
     returns the visual geometry of a single tab.
 
     \table 100%
-    \row \li \inlineimage fusion-tabbar.png Screenshot of a Fusion style tab bar
+    \row \li \inlineimage {fusion-tabbar.png} {Tab bar with three tabs}
          \li A tab bar shown in the \l{Qt Widget Gallery}{Fusion widget style}.
-    \row \li \inlineimage fusion-tabbar-truncated.png Screenshot of a truncated Fusion tab bar
+    \row \li \inlineimage {fusion-tabbar-truncated.png} {Truncated tab bar}
          \li A truncated tab bar shown in the Fusion widget style.
     \endtable
 
@@ -366,12 +372,14 @@ void QTabBarPrivate::init()
     leftB = new QToolButton(q);
     leftB->setObjectName(u"ScrollLeftButton"_s);
     leftB->setAutoRepeat(true);
-    QObject::connect(leftB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
+    QObjectPrivate::connect(leftB, &QToolButton::clicked,
+                            this, &QTabBarPrivate::scrollTabs);
     leftB->hide();
     rightB = new QToolButton(q);
     rightB->setObjectName(u"ScrollRightButton"_s);
     rightB->setAutoRepeat(true);
-    QObject::connect(rightB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
+    QObjectPrivate::connect(rightB, &QToolButton::clicked,
+                            this, &QTabBarPrivate::scrollTabs);
     rightB->hide();
 #ifdef QT_KEYPAD_NAVIGATION
     if (QApplicationPrivate::keypadNavigationEnabled()) {
@@ -611,8 +619,11 @@ QRect QTabBarPrivate::normalizedScrollRect(int index)
             bottomEdge = scrollButtonRightRect.top();
         }
 
+        const auto lastTab = lastVisibleTab();
+        if (!lastTab)
+            return {};
         bool tearTopVisible = index != 0 && topEdge != -scrollOffset;
-        bool tearBottomVisible = index != tabList.size() - 1 && bottomEdge != tabList.constLast()->rect.bottom() + 1 - scrollOffset;
+        bool tearBottomVisible = index != tabList.size() - 1 && bottomEdge != lastTab->rect.bottom() + 1 - scrollOffset;
         if (tearTopVisible && !tearLeftRect.isNull())
             topEdge = tearLeftRect.bottom() + 1;
         if (tearBottomVisible && !tearRightRect.isNull())
@@ -642,8 +653,11 @@ QRect QTabBarPrivate::normalizedScrollRect(int index)
             rightEdge = scrollButtonRightRect.left();
         }
 
+        const auto lastTab = lastVisibleTab();
+        if (!lastTab)
+            return {};
         bool tearLeftVisible = index != 0 && leftEdge != -scrollOffset;
-        bool tearRightVisible = index != tabList.size() - 1 && rightEdge != tabList.constLast()->rect.right() + 1 - scrollOffset;
+        bool tearRightVisible = index != tabList.size() - 1 && rightEdge != lastTab->rect.right() + 1 - scrollOffset;
         if (tearLeftVisible && !tearLeftRect.isNull())
             leftEdge = tearLeftRect.right() + 1;
         if (tearRightVisible && !tearRightRect.isNull())
@@ -668,19 +682,23 @@ void QTabBarPrivate::makeVisible(int index)
     if (!validIndex(index))
         return;
 
+    const auto lastTab = lastVisibleTab();
     const QRect tabRect = tabList.at(index)->rect;
     const int oldScrollOffset = scrollOffset;
     const bool horiz = !verticalTabs(shape);
     const int available = horiz ? q->width() : q->height();
     const int tabStart = horiz ? tabRect.left() : tabRect.top();
     const int tabEnd = horiz ? tabRect.right() : tabRect.bottom();
-    const int lastTabEnd = horiz ? tabList.constLast()->rect.right() : tabList.constLast()->rect.bottom();
+    const int lastTabEnd = lastTab ? (horiz ? lastTab->rect.right() : lastTab->rect.bottom()) : 0;
     const QRect scrollRect = normalizedScrollRect(index);
     const QRect entireScrollRect = normalizedScrollRect(0); // ignore tears
     const int scrolledTabBarStart = qMax(1, scrollRect.left() + scrollOffset);
     const int scrolledTabBarEnd = qMin(lastTabEnd - 1, scrollRect.right() + scrollOffset);
 
-    if (tabStart < scrolledTabBarStart) {
+     if (available >= lastTabEnd) {
+        // the entire tabbar fits, reset scroll
+        scrollOffset = 0;
+    } else if (tabStart < scrolledTabBarStart) {
         // Tab is outside on the left, so scroll left.
         scrollOffset = tabStart - scrollRect.left();
     } else if (tabEnd > scrolledTabBarEnd) {
@@ -689,9 +707,6 @@ void QTabBarPrivate::makeVisible(int index)
     } else if (scrollOffset + entireScrollRect.width() > lastTabEnd + 1) {
         // fill any free space on the right without overshooting
         scrollOffset = qMax(0, lastTabEnd - entireScrollRect.width() + 1);
-    } else if (available >= lastTabEnd) {
-        // the entire tabbar fits, reset scroll
-        scrollOffset = 0;
     }
 
     leftB->setEnabled(scrollOffset > -scrollRect.left());
@@ -705,11 +720,7 @@ void QTabBarPrivate::makeVisible(int index)
 
 void QTabBarPrivate::killSwitchTabTimer()
 {
-    Q_Q(QTabBar);
-    if (switchTabTimerId) {
-        q->killTimer(switchTabTimerId);
-        switchTabTimerId = 0;
-    }
+    switchTabTimer.stop();
     switchTabCurrentIndex = -1;
 }
 
@@ -765,7 +776,7 @@ void QTabBarPrivate::autoHideTabs()
         q->setVisible(q->count() > 1);
 }
 
-void QTabBarPrivate::_q_closeTab()
+void QTabBarPrivate::closeTab()
 {
     Q_Q(QTabBar);
     QObject *object = q->sender();
@@ -788,26 +799,30 @@ void QTabBarPrivate::_q_closeTab()
         emit q->tabCloseRequested(tabToClose);
 }
 
-void QTabBarPrivate::_q_scrollTabs()
+void QTabBarPrivate::scrollTabs()
 {
     Q_Q(QTabBar);
     const QObject *sender = q->sender();
     const bool horizontal = !verticalTabs(shape);
     const QRect scrollRect = normalizedScrollRect().translated(scrollOffset, 0);
 
-    int i = -1;
-
     if (sender == leftB) {
-        for (i = tabList.size() - 1; i >= 0; --i) {
-            int start = horizontal ? tabList.at(i)->rect.left() : tabList.at(i)->rect.top();
+        for (qsizetype i = tabList.size() - 1; i >= 0; --i) {
+            const auto *tab = tabList.at(i);
+            if (!tab->visible)
+                continue;
+            int start = horizontal ? tab->rect.left() : tab->rect.top();
             if (start < scrollRect.left()) {
                 makeVisible(i);
                 return;
             }
         }
     } else if (sender == rightB) {
-        for (i = 0; i < tabList.size(); ++i) {
-            const auto tabRect = tabList.at(i)->rect;
+        for (qsizetype i = 0; i < tabList.size(); ++i) {
+            const auto *tab = tabList.at(i);
+            if (!tab->visible)
+                continue;
+            const auto &tabRect = tab->rect;
             int start = horizontal ? tabRect.left() : tabRect.top();
             int end = horizontal ? tabRect.right() : tabRect.bottom();
             if (end > scrollRect.right() && start > scrollOffset) {
@@ -978,7 +993,8 @@ int QTabBar::insertTab(int index, const QIcon& icon, const QString &text)
         initStyleOption(&opt, index);
         ButtonPosition closeSide = (ButtonPosition)style()->styleHint(QStyle::SH_TabBar_CloseButtonPosition, nullptr, this);
         QAbstractButton *closeButton = new CloseButton(this);
-        connect(closeButton, SIGNAL(clicked()), this, SLOT(_q_closeTab()));
+        QObjectPrivate::connect(closeButton, &CloseButton::clicked,
+                                d, &QTabBarPrivate::closeTab);
         setTabButton(index, closeSide, closeButton);
     }
 
@@ -987,7 +1003,7 @@ int QTabBar::insertTab(int index, const QIcon& icon, const QString &text)
             ++tab->lastTab;
     }
 
-    if (tabAt(d->mousePosition) == index) {
+    if (isVisible() && tabAt(d->mousePosition) == index) {
         d->hoverIndex = index;
         d->hoverRect = tabRect(index);
     }
@@ -1551,8 +1567,10 @@ QSize QTabBar::minimumTabSizeHint(int index) const
     QTabBarPrivate::Tab *tab = d->tabList.at(index);
     QString oldText = tab->text;
     tab->text = computeElidedText(d->elideMode, oldText);
+    tab->measuringMinimum = true;
     QSize size = tabSizeHint(index);
     tab->text = oldText;
+    tab->measuringMinimum = false;
     return size;
 }
 
@@ -1753,9 +1771,8 @@ bool QTabBar::event(QEvent *event)
             const int tabIndex = tabAt(static_cast<QDragMoveEvent *>(event)->position().toPoint());
             if (isTabEnabled(tabIndex) && d->switchTabCurrentIndex != tabIndex) {
                 d->switchTabCurrentIndex = tabIndex;
-                if (d->switchTabTimerId)
-                    killTimer(d->switchTabTimerId);
-                d->switchTabTimerId = startTimer(style()->styleHint(QStyle::SH_TabBar_ChangeCurrentDelay));
+                d->switchTabTimer.start(
+                    style()->styleHint(QStyle::SH_TabBar_ChangeCurrentDelay, nullptr, this) * 1ms, this);
             }
             event->ignore();
         }
@@ -2375,7 +2392,7 @@ void QTabBar::keyPressEvent(QKeyEvent *event)
 void QTabBar::wheelEvent(QWheelEvent *event)
 {
     Q_D(QTabBar);
-    if (style()->styleHint(QStyle::SH_TabBar_AllowWheelScrolling)) {
+    if (style()->styleHint(QStyle::SH_TabBar_AllowWheelScrolling, nullptr, this)) {
         const bool wheelVertical = qAbs(event->angleDelta().y()) > qAbs(event->angleDelta().x());
         const bool tabsVertical = verticalTabs(d->shape);
         if (event->device()->capabilities().testFlag(QInputDevice::Capability::PixelScroll)) {
@@ -2472,9 +2489,8 @@ void QTabBar::changeEvent(QEvent *event)
 void QTabBar::timerEvent(QTimerEvent *event)
 {
     Q_D(QTabBar);
-    if (event->timerId() == d->switchTabTimerId) {
-        killTimer(d->switchTabTimerId);
-        d->switchTabTimerId = 0;
+    if (event->id() == d->switchTabTimer.id()) {
+        d->switchTabTimer.stop();
         setCurrentIndex(d->switchTabCurrentIndex);
         d->switchTabCurrentIndex = -1;
     }
@@ -2582,7 +2598,8 @@ void QTabBar::setTabsClosable(bool closable)
                 continue;
             newButtons = true;
             QAbstractButton *closeButton = new CloseButton(this);
-            connect(closeButton, SIGNAL(clicked()), this, SLOT(_q_closeTab()));
+            QObjectPrivate::connect(closeButton, &CloseButton::clicked,
+                                    d, &QTabBarPrivate::closeTab);
             setTabButton(i, closeSide, closeButton);
         }
         if (newButtons)
