@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qtoolbutton.h"
 
@@ -39,14 +40,15 @@ class QToolButtonPrivate : public QAbstractButtonPrivate
 public:
     void init();
 #if QT_CONFIG(menu)
-    void _q_buttonPressed();
-    void _q_buttonReleased();
+    void onButtonPressed();
+    void onButtonReleased();
     void popupTimerDone();
-    void _q_updateButtonDown();
-    void _q_menuTriggered(QAction *);
+    void updateButtonDown();
+    void onMenuTriggered(QAction *);
+    void onDefaultActionChanged();
 #endif
     bool updateHoverControl(const QPoint &pos);
-    void _q_actionTriggered();
+    void onActionTriggered();
     QStyle::SubControl newHoverControl(const QPoint &pos);
     QStyle::SubControl hoverControl;
     QRect hoverRect;
@@ -56,7 +58,8 @@ public:
     Qt::ArrowType arrowType;
     Qt::ToolButtonStyle toolButtonStyle;
     QToolButton::ToolButtonPopupMode popupMode;
-    enum { NoButtonPressed=0, MenuButtonPressed=1, ToolButtonPressed=2 };
+    uint popupModeSetByUser : 1; // true if popupMode was set through setPopupMode
+    enum { NoButtonPressed = 0, MenuButtonPressed = 1, ToolButtonPressed = 2 };
     uint buttonPressed : 2;
     uint menuButtonDown          : 1;
     uint autoRaise             : 1;
@@ -129,11 +132,10 @@ bool QToolButtonPrivate::hasMenu() const
     of possible pages to jump to. The timeout is style dependent,
     see QStyle::SH_ToolButton_PopupDelay.
 
-    \table 100%
-    \row \li \inlineimage assistant-toolbar.png Qt Assistant's toolbar with tool buttons
-    \row \li Qt Assistant's toolbar contains tool buttons that are associated
+    \image assistant-toolbar.png {Qt Assistant's toolbar with tool buttons}
+    \caption Qt Assistant's toolbar contains tool buttons that are associated
          with actions used in other parts of the main window.
-    \endtable
+
 
     \sa QPushButton, QToolBar, QMainWindow, QAction
 */
@@ -177,6 +179,7 @@ void QToolButtonPrivate::init()
     arrowType = Qt::NoArrow;
     menuButtonDown = false;
     popupMode = QToolButton::DelayedPopup;
+    popupModeSetByUser = false;
     buttonPressed = QToolButtonPrivate::NoButtonPressed;
 
     toolButtonStyle = Qt::ToolButtonIconOnly;
@@ -187,8 +190,10 @@ void QToolButtonPrivate::init()
                                  QSizePolicy::ToolButton));
 
 #if QT_CONFIG(menu)
-    QObject::connect(q, SIGNAL(pressed()), q, SLOT(_q_buttonPressed()));
-    QObject::connect(q, SIGNAL(released()), q, SLOT(_q_buttonReleased()));
+    QObjectPrivate::connect(q, &QAbstractButton::pressed,
+                            this, &QToolButtonPrivate::onButtonPressed);
+    QObjectPrivate::connect(q, &QAbstractButton::released,
+                            this, &QToolButtonPrivate::onButtonReleased);
 #endif
 
     setLayoutItemMargins(QStyle::SE_ToolButtonLayoutItem);
@@ -435,7 +440,8 @@ void QToolButton::actionEvent(QActionEvent *event)
             setDefaultAction(action); // update button state
         break;
     case QEvent::ActionAdded:
-        connect(action, SIGNAL(triggered()), this, SLOT(_q_actionTriggered()));
+        QObjectPrivate::connect(action, &QAction::triggered, d,
+                                &QToolButtonPrivate::onActionTriggered);
         break;
     case QEvent::ActionRemoved:
         if (d->defaultAction == action)
@@ -480,7 +486,7 @@ bool QToolButtonPrivate::updateHoverControl(const QPoint &pos)
     return !doesHover;
 }
 
-void QToolButtonPrivate::_q_actionTriggered()
+void QToolButtonPrivate::onActionTriggered()
 {
     Q_Q(QToolButton);
     if (QAction *action = qobject_cast<QAction *>(q->sender()))
@@ -581,8 +587,10 @@ void QToolButton::mousePressEvent(QMouseEvent *e)
 void QToolButton::mouseReleaseEvent(QMouseEvent *e)
 {
     Q_D(QToolButton);
+    QPointer<QAbstractButton> guard(this);
     QAbstractButton::mouseReleaseEvent(e);
-    d->buttonPressed = QToolButtonPrivate::NoButtonPressed;
+    if (guard)
+        d->buttonPressed = QToolButtonPrivate::NoButtonPressed;
 }
 
 /*!
@@ -667,7 +675,7 @@ void QToolButton::showMenu()
     d->popupTimerDone();
 }
 
-void QToolButtonPrivate::_q_buttonPressed()
+void QToolButtonPrivate::onButtonPressed()
 {
     Q_Q(QToolButton);
     if (!hasMenu())
@@ -680,7 +688,7 @@ void QToolButtonPrivate::_q_buttonPressed()
         q->showMenu();
 }
 
-void QToolButtonPrivate::_q_buttonReleased()
+void QToolButtonPrivate::onButtonReleased()
 {
     popupTimer.stop();
 }
@@ -722,8 +730,13 @@ static QPoint positionMenu(const QToolButton *q, bool horizontal,
             }
         }
     }
+
+    // QTBUG-118695 Force point inside the current screen. If the returned point
+    // is not found inside any screen, QMenu's positioning logic kicks in without
+    // taking the QToolButton's screen into account. This can cause the menu to
+    // end up on primary monitor, even if the QToolButton is on a non-primary monitor.
     p.rx() = qMax(screen.left(), qMin(p.x(), screen.right() - sh.width()));
-    p.ry() += 1;
+    p.ry() = qMax(screen.top(), qMin(p.y() + 1, screen.bottom()));
     return p;
 }
 
@@ -757,9 +770,12 @@ void QToolButtonPrivate::popupTimerDone()
 #endif
     QPointer<QToolButton> that = q;
     actualMenu->setNoReplayFor(q);
-    if (!mustDeleteActualMenu) //only if action are not in this widget
-        QObject::connect(actualMenu, SIGNAL(triggered(QAction*)), q, SLOT(_q_menuTriggered(QAction*)));
-    QObject::connect(actualMenu, SIGNAL(aboutToHide()), q, SLOT(_q_updateButtonDown()));
+    if (!mustDeleteActualMenu) { //only if action are not in this widget
+        QObjectPrivate::connect(actualMenu, &QMenu::triggered,
+                                this, &QToolButtonPrivate::onMenuTriggered);
+    }
+    QObjectPrivate::connect(actualMenu, &QMenu::aboutToHide,
+                            this, &QToolButtonPrivate::updateButtonDown);
     actualMenu->d_func()->causedPopup.widget = q;
     actualMenu->d_func()->causedPopup.action = defaultAction;
     actionsCopy = q->actions(); //(the list of action may be modified in slots)
@@ -773,11 +789,19 @@ void QToolButtonPrivate::popupTimerDone()
     if (!that)
         return;
 
-    QObject::disconnect(actualMenu, SIGNAL(aboutToHide()), q, SLOT(_q_updateButtonDown()));
-    if (mustDeleteActualMenu)
+    QObjectPrivate::disconnect(actualMenu, &QMenu::aboutToHide,
+                               this, &QToolButtonPrivate::updateButtonDown);
+    if (menuButtonDown) {
+        // The menu was empty, it didn't actually show up, so it was never hidden either
+        updateButtonDown();
+    }
+
+    if (mustDeleteActualMenu) {
         delete actualMenu;
-    else
-        QObject::disconnect(actualMenu, SIGNAL(triggered(QAction*)), q, SLOT(_q_menuTriggered(QAction*)));
+    } else {
+        QObjectPrivate::disconnect(actualMenu, &QMenu::triggered,
+                                   this, &QToolButtonPrivate::onMenuTriggered);
+    }
 
     actionsCopy.clear();
 
@@ -785,7 +809,7 @@ void QToolButtonPrivate::popupTimerDone()
         q->setAutoRepeat(true);
 }
 
-void QToolButtonPrivate::_q_updateButtonDown()
+void QToolButtonPrivate::updateButtonDown()
 {
     Q_Q(QToolButton);
     menuButtonDown = false;
@@ -795,13 +819,20 @@ void QToolButtonPrivate::_q_updateButtonDown()
         q->repaint();
 }
 
-void QToolButtonPrivate::_q_menuTriggered(QAction *action)
+void QToolButtonPrivate::onMenuTriggered(QAction *action)
 {
     Q_Q(QToolButton);
     if (action && !actionsCopy.contains(action))
         emit q->triggered(action);
 }
 
+void QToolButtonPrivate::onDefaultActionChanged()
+{
+    Q_Q(QToolButton);
+    if (defaultAction && defaultAction->menu() && !popupModeSetByUser)
+        q->setPopupMode(QToolButton::MenuButtonPopup);
+}
+
 /*! \enum QToolButton::ToolButtonPopupMode
 
     Describes how a menu should be popped up for tool buttons that has
@@ -835,6 +866,7 @@ void QToolButtonPrivate::_q_menuTriggered(QAction *action)
 void QToolButton::setPopupMode(ToolButtonPopupMode mode)
 {
     Q_D(QToolButton);
+    d->popupModeSetByUser = true;
     d->popupMode = mode;
 }
 
@@ -893,8 +925,11 @@ void QToolButton::setDefaultAction(QAction *action)
 {
     Q_D(QToolButton);
 #if QT_CONFIG(menu)
-    bool hadMenu = false;
-    hadMenu = d->hasMenu();
+    if (d->defaultAction && d->defaultAction != action) {
+        QObjectPrivate::disconnect(d->defaultAction, &QAction::changed, d,
+                                   &QToolButtonPrivate::onDefaultActionChanged);
+    }
+    const bool hadMenu = d->hasMenu();
 #endif
     d->defaultAction = action;
     if (!action)
@@ -918,11 +953,17 @@ void QToolButton::setDefaultAction(QAction *action)
     setWhatsThis(action->whatsThis());
 #endif
 #if QT_CONFIG(menu)
-    if (action->menu() && !hadMenu) {
+    if (!hadMenu && !d->popupModeSetByUser) {
+        // ### Qt7 Fixme
         // new 'default' popup mode defined introduced by tool bar. We
         // should have changed QToolButton's default instead. Do that
         // in 4.2.
-        setPopupMode(QToolButton::MenuButtonPopup);
+        if (action->menu()) {
+            setPopupMode(QToolButton::MenuButtonPopup);
+        } else {
+            QObjectPrivate::connect(d->defaultAction, &QAction::changed, d,
+                                    &QToolButtonPrivate::onDefaultActionChanged);
+        }
     }
 #endif
     setCheckable(action->isCheckable());
