@@ -514,8 +514,8 @@ void tst_QAbstractItemModel::match()
     QCOMPARE(res.size(), 3);
 }
 
-typedef QPair<int, int> Position;
-typedef QList<QPair<int, int> > Selection;
+using Position = std::pair<int, int>;
+using Selection = QList<Position>;
 typedef QList<QList<QString> > StringTable;
 typedef QList<QString> StringTableRow;
 
