@@ -3,7 +3,9 @@
 
 #include <QTest>
 #include <qtimezone.h>
-#include <private/qtimezoneprivate_p.h>
+#if QT_CONFIG(timezone)
+#  include <private/qtimezoneprivate_p.h>
+#endif
 #include <private/qcomparisontesthelper_p.h>
 
 #include <qlocale.h>
@@ -71,8 +73,8 @@ private Q_SLOTS:
 #endif // timezone backends
 
 private:
-    void printTimeZone(const QTimeZone &tz);
 #if QT_CONFIG(timezone)
+    void printTimeZone(const QTimeZone &tz);
 #  if defined(QT_BUILD_INTERNAL)
     // Generic tests of privates, called by implementation-specific private tests:
     void testCetPrivate(const QTimeZonePrivate &tzp);
@@ -102,6 +104,7 @@ private:
     static constexpr bool debug = false;
 };
 
+#if QT_CONFIG(timezone)
 void tst_QTimeZone::printTimeZone(const QTimeZone &tz)
 {
     QDateTime now = QDateTime::currentDateTime();
@@ -167,9 +170,11 @@ void tst_QTimeZone::printTimeZone(const QTimeZone &tz)
     qDebug() << "Transition before 1 Jun = " << tz.previousTransition(jun).atUtc;
     qDebug() << "";
 }
+#endif // feature timezone
 
 void tst_QTimeZone::createTest()
 {
+#if QT_CONFIG(timezone)
     const QTimeZone tz("Pacific/Auckland");
 
     if constexpr (debug)
@@ -265,13 +270,16 @@ void tst_QTimeZone::createTest()
             QCOMPARE(result.at(i).daylightTimeOffset, expected.at(i).daylightTimeOffset);
         }
     }
+#else
+    QSKIP("Test depends on backends, enabled by feature timezone");
+#endif // feature timezone
 }
 
 void tst_QTimeZone::nullTest()
 {
     QTimeZone nullTz1;
     QTimeZone nullTz2;
-    QTimeZone utc("UTC");
+    QTimeZone utc(QTimeZone::UTC);
 
     // Validity tests
     QCOMPARE(nullTz1.isValid(), false);
@@ -290,6 +298,7 @@ void tst_QTimeZone::nullTest()
     utc = nullTz1;
     QCOMPARE(utc.isValid(), false);
 
+#if QT_CONFIG(timezone)
     QCOMPARE(nullTz1.id(), QByteArray());
     QCOMPARE(nullTz1.territory(), QLocale::AnyTerritory);
     QCOMPARE(nullTz1.comment(), QString());
@@ -335,6 +344,7 @@ void tst_QTimeZone::nullTest()
     QCOMPARE(data.offsetFromUtc, invalidOffset);
     QCOMPARE(data.standardTimeOffset, invalidOffset);
     QCOMPARE(data.daylightTimeOffset, invalidOffset);
+#endif // feature timezone
 }
 
 void tst_QTimeZone::assign()
@@ -487,7 +497,7 @@ void tst_QTimeZone::offset()
 
 void tst_QTimeZone::dataStreamTest()
 {
-#ifndef QT_NO_DATASTREAM
+#if QT_CONFIG(timezone) && !defined(QT_NO_DATASTREAM)
     // Test the OffsetFromUtc backend serialization. First with a custom timezone:
     QTimeZone tz1("QST"_ba, 23456,
                   u"Qt Standard Time"_s, u"QST"_s, QLocale::Norway, u"Qt Testing"_s);
@@ -546,7 +556,7 @@ void tst_QTimeZone::dataStreamTest()
         QCOMPARE(ds.status(), QDataStream::Ok);
     }
     QCOMPARE(tz2.id(), tz1.id());
-#endif
+#endif // feature timezone and enabled datastream
 }
 
 #if QT_CONFIG(timezone)
@@ -791,7 +801,7 @@ void tst_QTimeZone::hasAlternativeName()
 
 void tst_QTimeZone::specificTransition_data()
 {
-#if QT_CONFIG(timezone) && QT_CONFIG(timezone_tzdb) && defined(__GLIBCXX__)
+#if QT_CONFIG(timezone_tzdb) && defined(__GLIBCXX__)
     QSKIP("libstdc++'s C++20 misreads the IANA DB for Moscow's transitions (among others).");
 #endif
 #if defined Q_OS_ANDROID && !QT_CONFIG(timezone_tzdb)
@@ -1408,7 +1418,8 @@ void tst_QTimeZone::utcTest()
 
 void tst_QTimeZone::icuTest()
 {
-#if defined(QT_BUILD_INTERNAL) && QT_CONFIG(icu) && !QT_CONFIG(timezone_tzdb) && !defined(Q_OS_UNIX)
+#if defined(QT_BUILD_INTERNAL) && QT_CONFIG(icu) && !QT_CONFIG(timezone_tzdb) \
+    && (defined(Q_OS_VXWORKS) || !defined(Q_OS_UNIX))
     // Known datetimes
     qint64 std = QDateTime(QDate(2012, 1, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
     qint64 dst = QDateTime(QDate(2012, 6, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
@@ -1456,8 +1467,9 @@ void tst_QTimeZone::icuTest()
 
 void tst_QTimeZone::tzTest()
 {
-#if defined QT_BUILD_INTERNAL && defined Q_OS_UNIX \
-        && !QT_CONFIG(timezone_tzdb) && !defined Q_OS_DARWIN && !defined Q_OS_ANDROID
+#if defined QT_BUILD_INTERNAL && defined Q_OS_UNIX  \
+    && !(QT_CONFIG(timezone_tzdb) || defined(Q_OS_DARWIN) \
+         || defined(Q_OS_ANDROID) || defined(Q_OS_VXWORKS))
     const auto UTC = QTimeZone::UTC;
     // Known datetimes
     qint64 std = QDateTime(QDate(2012, 1, 1), QTime(0, 0), UTC).toMSecsSinceEpoch();
