@@ -1,35 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QSignalSpy>
 
-#include <qstandarditemmodel.h>
+#include <QStandardItem>
 
 class tst_QStandardItem : public QObject
 {
@@ -48,8 +24,6 @@ private slots:
     void parent();
     void insertColumn_data();
     void insertColumn();
-    void insertColumns_data();
-    void insertColumns();
     void insertRow_data();
     void insertRow();
     void insertRows_data();
@@ -138,13 +112,17 @@ void tst_QStandardItem::getSetData()
             item.setToolTip(toolTip);
             QCOMPARE(item.toolTip(), toolTip);
 
+#ifndef QT_NO_STATUSTIP
             QString statusTip = QLatin1String("statusTip ") + iS;
             item.setStatusTip(statusTip);
             QCOMPARE(item.statusTip(), statusTip);
+#endif
 
+#if QT_CONFIG(whatsthis)
             QString whatsThis = QLatin1String("whatsThis ") + iS;
             item.setWhatsThis(whatsThis);
             QCOMPARE(item.whatsThis(), whatsThis);
+#endif
 
             QSize sizeHint(64*i, 48*i);
             item.setSizeHint(sizeHint);
@@ -183,8 +161,12 @@ void tst_QStandardItem::getSetData()
             QCOMPARE(item.text(), text);
             QCOMPARE(item.icon(), icon);
             QCOMPARE(item.toolTip(), toolTip);
+#ifndef QT_NO_STATUSTIP
             QCOMPARE(item.statusTip(), statusTip);
+#endif
+#if QT_CONFIG(whatsthis)
             QCOMPARE(item.whatsThis(), whatsThis);
+#endif
             QCOMPARE(item.sizeHint(), sizeHint);
             QCOMPARE(item.font(), font);
             QCOMPARE(item.textAlignment(), textAlignment);
@@ -197,8 +179,12 @@ void tst_QStandardItem::getSetData()
             QCOMPARE(qvariant_cast<QString>(item.data(Qt::DisplayRole)), text);
             QCOMPARE(qvariant_cast<QIcon>(item.data(Qt::DecorationRole)), icon);
             QCOMPARE(qvariant_cast<QString>(item.data(Qt::ToolTipRole)), toolTip);
+#ifndef QT_NO_STATUSTIP
             QCOMPARE(qvariant_cast<QString>(item.data(Qt::StatusTipRole)), statusTip);
+#endif
+#if QT_CONFIG(whatsthis)
             QCOMPARE(qvariant_cast<QString>(item.data(Qt::WhatsThisRole)), whatsThis);
+#endif
             QCOMPARE(qvariant_cast<QSize>(item.data(Qt::SizeHintRole)), sizeHint);
             QCOMPARE(qvariant_cast<QFont>(item.data(Qt::FontRole)), font);
             QCOMPARE(qvariant_cast<int>(item.data(Qt::TextAlignmentRole)), int(textAlignment));
@@ -310,15 +296,6 @@ void tst_QStandardItem::getSetFlags()
     item.setCheckState(Qt::Checked);
     item.setCheckable(true);
     QCOMPARE(item.checkState(), Qt::Checked);
-
-#if QT_DEPRECATED_SINCE(5, 6)
-    // deprecated API
-    item.setTristate(true);
-    QVERIFY(item.isTristate());
-    QVERIFY(item.flags() & Qt::ItemIsTristate);
-    item.setTristate(false);
-    QVERIFY(!(item.flags() & Qt::ItemIsTristate));
-#endif
 }
 
 void tst_QStandardItem::getSetRowAndColumnCount()
@@ -382,7 +359,7 @@ void tst_QStandardItem::getSetChild()
     QStandardItem item(rows, columns);
     bool shouldHaveChildren = (rows > 0) && (columns > 0);
     QCOMPARE(item.hasChildren(), shouldHaveChildren);
-    QCOMPARE(item.child(row, column), static_cast<QStandardItem*>(0));
+    QCOMPARE(item.child(row, column), nullptr);
 
     QStandardItem *child = new QStandardItem;
     item.setChild(row, column, child);
@@ -399,11 +376,11 @@ void tst_QStandardItem::getSetChild()
         QCOMPARE(item.child(row, column), anotherChild);
         QCOMPARE(anotherChild->row(), row);
         QCOMPARE(anotherChild->column(), column);
-        item.setChild(row, column, 0);
+        item.setChild(row, column, nullptr);
     } else {
         delete child;
     }
-    QCOMPARE(item.child(row, column), static_cast<QStandardItem*>(0));
+    QCOMPARE(item.child(row, column), nullptr);
 }
 
 void tst_QStandardItem::parent()
@@ -411,7 +388,7 @@ void tst_QStandardItem::parent()
     {
         QStandardItem item;
         QStandardItem *child = new QStandardItem;
-        QCOMPARE(child->parent(), static_cast<QStandardItem*>(0));
+        QCOMPARE(child->parent(), nullptr);
         item.setChild(0, 0, child);
         QCOMPARE(child->parent(), &item);
 
@@ -425,7 +402,7 @@ void tst_QStandardItem::parent()
         QStandardItem *item = new QStandardItem;
         model.appendRow(item);
         // parent of a top-level item should be 0
-        QCOMPARE(item->parent(), static_cast<QStandardItem*>(0));
+        QCOMPARE(item->parent(), nullptr);
     }
 }
 
@@ -485,7 +462,7 @@ void tst_QStandardItem::insertColumn()
         for (int i = 0; i < count; ++i)
             QCOMPARE(item.child(i, column), columnItems.at(i));
         for (int i = count; i < item.rowCount(); ++i)
-            QCOMPARE(item.child(i, column), static_cast<QStandardItem*>(0));
+            QCOMPARE(item.child(i, column), nullptr);
     } else {
         QCOMPARE(item.columnCount(), columns);
         QCOMPARE(item.rowCount(), rows);
@@ -493,14 +470,6 @@ void tst_QStandardItem::insertColumn()
     }
 }
 
-void tst_QStandardItem::insertColumns_data()
-{
-}
-
-void tst_QStandardItem::insertColumns()
-{
-}
-
 void tst_QStandardItem::insertRow_data()
 {
     QTest::addColumn<int>("rows");
@@ -557,7 +526,7 @@ void tst_QStandardItem::insertRow()
         for (int i = 0; i < count; ++i)
             QCOMPARE(item.child(row, i), rowItems.at(i));
         for (int i = count; i < item.columnCount(); ++i)
-            QCOMPARE(item.child(row, i), static_cast<QStandardItem*>(0));
+            QCOMPARE(item.child(row, i), nullptr);
     } else {
         QCOMPARE(item.columnCount(), columns);
         QCOMPARE(item.rowCount(), rows);
@@ -585,9 +554,8 @@ void tst_QStandardItem::insertRows()
     QStandardItem item(rows, columns);
 
     QList<QStandardItem*> items;
-    for (int i = 0; i < insertCount; ++i) {
+    for (int i = 0; i < insertCount; ++i)
         items.append(new QStandardItem());
-    }
     item.insertRows(insertAt, items);
 
     QCOMPARE(item.rowCount(), rows + insertCount);
@@ -659,7 +627,7 @@ void tst_QStandardItem::appendColumn()
     for (int i = 0; i < count; ++i)
         QCOMPARE(item.child(i, columns), columnItems.at(i));
     for (int i = count; i < item.rowCount(); ++i)
-        QCOMPARE(item.child(i, columns), static_cast<QStandardItem*>(0));
+        QCOMPARE(item.child(i, columns), nullptr);
 
     // make sure original children remained unchanged
     for (int i = 0; i < rows; ++i) {
@@ -734,7 +702,7 @@ void tst_QStandardItem::appendRow()
     for (int i = 0; i < count; ++i)
         QCOMPARE(item.child(rows, i), rowItems.at(i));
     for (int i = count; i < item.columnCount(); ++i)
-        QCOMPARE(item.child(rows, i), static_cast<QStandardItem*>(0));
+        QCOMPARE(item.child(rows, i), nullptr);
 
     // make sure original children remained unchanged
     for (int i = 0; i < rows; ++i) {
@@ -753,7 +721,7 @@ void tst_QStandardItem::takeChild()
 
     for (int i = 0; i < item.rowCount(); ++i) {
         QCOMPARE(item.takeChild(i), itemList.at(i));
-        QCOMPARE(item.takeChild(0, 0), static_cast<QStandardItem*>(0));
+        QCOMPARE(item.takeChild(0, 0), nullptr);
         for (int j = i + 1; j < item.rowCount(); ++j)
             QCOMPARE(item.child(j), itemList.at(j));
     }
@@ -801,10 +769,10 @@ void tst_QStandardItem::takeColumn()
 
     QList<QStandardItem *> taken = item.takeColumn(column);
     if (expectSuccess) {
-        QCOMPARE(taken.count(), item.rowCount());
+        QCOMPARE(taken.size(), item.rowCount());
         QCOMPARE(item.columnCount(), columns - 1);
         int index = column;
-        for (int i = 0; i < taken.count(); ++i) {
+        for (int i = 0; i < taken.size(); ++i) {
             QCOMPARE(taken.at(i), originalChildren.takeAt(index));
             index += item.columnCount();
         }
@@ -862,10 +830,10 @@ void tst_QStandardItem::takeRow()
 
     QList<QStandardItem *> taken = item.takeRow(row);
     if (expectSuccess) {
-        QCOMPARE(taken.count(), item.columnCount());
+        QCOMPARE(taken.size(), item.columnCount());
         QCOMPARE(item.rowCount(), rows - 1);
         int index = row * columns;
-        for (int i = 0; i < taken.count(); ++i) {
+        for (int i = 0; i < taken.size(); ++i) {
             QCOMPARE(taken.at(i), originalChildren.takeAt(index));
         }
         index = 0;
@@ -888,7 +856,9 @@ void tst_QStandardItem::streamItem()
     item.setText(QLatin1String("text"));
     item.setToolTip(QLatin1String("toolTip"));
     item.setStatusTip(QLatin1String("statusTip"));
+#if QT_CONFIG(whatsthis)
     item.setWhatsThis(QLatin1String("whatsThis"));
+#endif
     item.setSizeHint(QSize(64, 48));
     item.setFont(QFont());
     item.setTextAlignment(Qt::AlignLeft|Qt::AlignVCenter);
@@ -910,7 +880,9 @@ void tst_QStandardItem::streamItem()
         QCOMPARE(streamedItem.text(), item.text());
         QCOMPARE(streamedItem.toolTip(), item.toolTip());
         QCOMPARE(streamedItem.statusTip(), item.statusTip());
+#if QT_CONFIG(whatsthis)
         QCOMPARE(streamedItem.whatsThis(), item.whatsThis());
+#endif
         QCOMPARE(streamedItem.sizeHint(), item.sizeHint());
         QCOMPARE(streamedItem.font(), item.font());
         QCOMPARE(streamedItem.textAlignment(), item.textAlignment());
@@ -938,7 +910,7 @@ void tst_QStandardItem::deleteItem()
         for (int j = 0; j < model.columnCount(); ++j) {
             QStandardItem *item = model.item(i, j);
             delete item;
-            QCOMPARE(model.item(i, j), static_cast<QStandardItem*>(0));
+            QCOMPARE(model.item(i, j), nullptr);
         }
     }
 }
@@ -949,7 +921,9 @@ void tst_QStandardItem::clone()
     item.setText(QLatin1String("text"));
     item.setToolTip(QLatin1String("toolTip"));
     item.setStatusTip(QLatin1String("statusTip"));
+#if QT_CONFIG(whatsthis)
     item.setWhatsThis(QLatin1String("whatsThis"));
+#endif
     item.setSizeHint(QSize(64, 48));
     item.setFont(QFont());
     item.setTextAlignment(Qt::AlignLeft|Qt::AlignVCenter);
@@ -964,7 +938,9 @@ void tst_QStandardItem::clone()
     QCOMPARE(clone->text(), item.text());
     QCOMPARE(clone->toolTip(), item.toolTip());
     QCOMPARE(clone->statusTip(), item.statusTip());
+#if QT_CONFIG(whatsthis)
     QCOMPARE(clone->whatsThis(), item.whatsThis());
+#endif
     QCOMPARE(clone->sizeHint(), item.sizeHint());
     QCOMPARE(clone->font(), item.font());
     QCOMPARE(clone->textAlignment(), item.textAlignment());
@@ -995,9 +971,9 @@ void tst_QStandardItem::sortChildren()
         item->appendRow(two);
 
         QSignalSpy layoutAboutToBeChangedSpy(
-            model, SIGNAL(layoutAboutToBeChanged()));
+            model, &QAbstractItemModel::layoutAboutToBeChanged);
         QSignalSpy layoutChangedSpy(
-            model, SIGNAL(layoutChanged()));
+            model, &QAbstractItemModel::layoutChanged);
 
         one->sortChildren(0, Qt::DescendingOrder);
         // verify sorted
@@ -1028,8 +1004,8 @@ void tst_QStandardItem::sortChildren()
         QCOMPARE(two->child(1)->text(), QLatin1String("e"));
         QCOMPARE(two->child(2)->text(), QLatin1String("f"));
 
-        QCOMPARE(layoutAboutToBeChangedSpy.count(), (x == 0) ? 0 : 3);
-        QCOMPARE(layoutChangedSpy.count(), (x == 0) ? 0 : 3);
+        QCOMPARE(layoutAboutToBeChangedSpy.size(), (x == 0) ? 0 : 3);
+        QCOMPARE(layoutChangedSpy.size(), (x == 0) ? 0 : 3);
 
         if (x == 0)
             delete item;
@@ -1040,18 +1016,41 @@ void tst_QStandardItem::sortChildren()
 class CustomItem : public QStandardItem
 {
 public:
-    CustomItem(const QString &text) : QStandardItem(text) { }
-    CustomItem() { }
-    virtual ~CustomItem() { }
+    using QStandardItem::QStandardItem;
 
-    virtual int type() const { return QStandardItem::UserType + 1; }
+    int type() const override { return QStandardItem::UserType + 1; }
 
-    virtual QStandardItem *clone() const { return QStandardItem::clone(); }
+    bool operator<(const QStandardItem &other) const override {
+        return text().size() < other.text().size();
+    }
 
-    void emitDataChanged() { QStandardItem::emitDataChanged(); }
+    using QStandardItem::clone;
+    using QStandardItem::emitDataChanged;
 
-    virtual bool operator<(const QStandardItem &other) const {
-        return text().length() < other.text().length();
+    void setData(const QVariant &value, int role) override
+    {
+        switch (role) {
+        case Qt::DisplayRole:
+            QStandardItem::setData(value, role);
+            break;
+        default:
+            // setFlags() uses Qt::StandardItemFlagsRole to store the flags,
+            // which is an implementation detail not exposed in the docs.
+            QStandardItem::setData(value, role);
+            break;
+        }
+    }
+
+    QVariant data(int role) const override
+    {
+        switch (role) {
+        case Qt::DisplayRole:
+            return QStandardItem::data(role);
+        default:
+            // flags() uses Qt::StandardItemFlagsRole to get the flags, which is
+            // an implementation detail not exposed in the docs.
+            return QStandardItem::data(role);
+        }
     }
 };
 
@@ -1072,11 +1071,11 @@ void tst_QStandardItem::subclassing()
     QStandardItemModel model;
     model.appendRow(item);
 
-    QSignalSpy itemChangedSpy(&model, SIGNAL(itemChanged(QStandardItem*)));
+    QSignalSpy itemChangedSpy(&model, &QStandardItemModel::itemChanged);
     item->emitDataChanged();
-    QCOMPARE(itemChangedSpy.count(), 1);
-    QCOMPARE(itemChangedSpy.at(0).count(), 1);
-    QCOMPARE(qvariant_cast<QStandardItem*>(itemChangedSpy.at(0).at(0)), (QStandardItem*)item);
+    QCOMPARE(itemChangedSpy.size(), 1);
+    QCOMPARE(itemChangedSpy.at(0).size(), 1);
+    QCOMPARE(qvariant_cast<QStandardItem*>(itemChangedSpy.at(0).at(0)), item);
 
     CustomItem *child0 = new CustomItem("cc");
     CustomItem *child1 = new CustomItem("bbb");
@@ -1085,9 +1084,15 @@ void tst_QStandardItem::subclassing()
     item->appendRow(child1);
     item->appendRow(child2);
     item->sortChildren(0);
-    QCOMPARE(item->child(0), (QStandardItem*)child2);
-    QCOMPARE(item->child(1), (QStandardItem*)child0);
-    QCOMPARE(item->child(2), (QStandardItem*)child1);
+    QCOMPARE(item->child(0), child2);
+    QCOMPARE(item->child(1), child0);
+    QCOMPARE(item->child(2), child1);
+
+    item->setFlags(Qt::ItemFlags{0});
+    QCOMPARE(item->flags(), Qt::ItemFlags{0});
+
+    item->setFlags(Qt::ItemFlags{Qt::ItemIsEditable | Qt::ItemIsSelectable});
+    QCOMPARE(item->flags(), Qt::ItemFlags{Qt::ItemIsEditable | Qt::ItemIsSelectable});
 }
 
 void tst_QStandardItem::lessThan()
