@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <private/qfontengine_p.h>
@@ -1094,11 +1094,12 @@ void tst_QTextScriptEngine::combiningMarks_qtbug15675_data()
 
     bool hasTests = false;
 
-    QStringList families;
-    families << QStringLiteral("Monaco");
-    families << QStringLiteral("DejaVu Sans Mono");
+    const QString families[] = {
+        QStringLiteral("Monaco"),
+        QStringLiteral("DejaVu Sans Mono"),
+    };
 
-    foreach (const QString &family, families) {
+    for (const QString &family : families) {
         QFont font(family);
         font.setStyleStrategy(QFont::NoFontMerging);
         if (QFontInfo(font).family() != family)
