@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 
@@ -109,6 +84,12 @@ Http2Server::~Http2Server()
 {
 }
 
+void Http2Server::setInformationalStatusCode(int code)
+{
+    if (code == 100 || (102 <= code && code <= 199))
+        informationalStatusCode = code;
+}
+
 void Http2Server::enablePushPromise(bool pushEnabled, const QByteArray &path)
 {
     pushPromiseEnabled = pushEnabled;
@@ -120,6 +101,11 @@ void Http2Server::setResponseBody(const QByteArray &body)
     responseBody = body;
 }
 
+void Http2Server::enableSendEarlyError(bool enable)
+{
+    sendEarlyError = enable;
+}
+
 void Http2Server::setContentEncoding(const QByteArray &encoding)
 {
     contentEncoding = encoding;
@@ -130,17 +116,29 @@ void Http2Server::setAuthenticationHeader(const QByteArray &authentication)
     authenticationHeader = authentication;
 }
 
+void Http2Server::setAuthenticationRequired(bool enable)
+{
+    Q_ASSERT(!enable || authenticationHeader.isEmpty());
+    authenticationRequired = enable;
+}
+
 void Http2Server::setRedirect(const QByteArray &url, int count)
 {
     redirectUrl = url;
     redirectCount = count;
 }
 
-void Http2Server::emulateGOAWAY(int timeout)
+void Http2Server::setSendTrailingHEADERS(bool enable)
+{
+    sendTrailingHEADERS = enable;
+}
+
+void Http2Server::emulateGOAWAY(int code, int timeout)
 {
     Q_ASSERT(timeout >= 0);
     testingGOAWAY = true;
     goawayTimeout = timeout;
+    goawayCode = code;
 }
 
 void Http2Server::redirectOpenStream(quint16 port)
@@ -199,6 +197,9 @@ void Http2Server::sendServerSettings()
         writer.append(it.value());
         if (it.key() == Settings::INITIAL_WINDOW_SIZE_ID)
             streamRecvWindowSize = it.value();
+        if (it.key() == Settings::HEADER_TABLE_SIZE_ID) {
+            pendingMaxTableSizeUpdate = it.value();
+        }
     }
     writer.write(*socket);
     // Now, let's update our peer on a session recv window size:
@@ -273,9 +274,20 @@ void Http2Server::sendDATA(quint32 streamID, quint32 windowSize)
         return;
 
     if (last) {
-        writer.start(FrameType::DATA, FrameFlag::END_STREAM, streamID);
-        writer.setPayloadSize(0);
-        writer.write(*socket);
+        if (sendTrailingHEADERS) {
+            writer.start(FrameType::HEADERS,
+                    FrameFlag::PRIORITY | FrameFlag::END_HEADERS | FrameFlag::END_STREAM, streamID);
+            const quint32 maxFrameSize(clientSetting(Settings::MAX_FRAME_SIZE_ID,
+                    Http2::maxPayloadSize));
+            // 5 bytes for PRIORITY data:
+            writer.append(quint32(0)); // streamID 0 (32-bit)
+            writer.append(quint8(0)); // + weight 0 (8-bit)
+            writer.writeHEADERS(*socket, maxFrameSize);
+        } else {
+            writer.start(FrameType::DATA, FrameFlag::END_STREAM, streamID);
+            writer.setPayloadSize(0);
+            writer.write(*socket);
+        }
         suspendedStreams.erase(it);
         activeRequests.erase(streamID);
 
@@ -325,7 +337,8 @@ void Http2Server::incomingConnection(qintptr socketDescriptor)
         connect(sslSocket, SIGNAL(sslErrors(QList<QSslError>)),
                 this, SLOT(ignoreErrorSlot()));
         QFile file(QT_TESTCASE_SOURCEDIR "/certs/fluke.key");
-        file.open(QIODevice::ReadOnly);
+        if (!file.open(QIODevice::ReadOnly))
+            qFatal("Cannot open certificate file %s", qPrintable(file.fileName()));
         QSslKey key(file.readAll(), QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey);
         sslSocket->setPrivateKey(key);
         auto localCert = QSslCertificate::fromPath(QT_TESTCASE_SOURCEDIR "/certs/fluke.cert");
@@ -386,16 +399,12 @@ bool Http2Server::verifyProtocolUpgradeRequest()
     bool settingsOk = false;
 
     QHttpNetworkReplyPrivate *firstRequestReader = protocolUpgradeHandler->d_func();
+    const auto headers = firstRequestReader->headers();
 
     // That's how we append them, that's what I expect to find:
-    for (const auto &header : firstRequestReader->headers()) {
-        if (header.first == "Connection")
-            connectionOk = header.second.contains("Upgrade, HTTP2-Settings");
-        else if (header.first == "Upgrade")
-            upgradeOk = header.second.contains("h2c");
-        else if (header.first == "HTTP2-Settings")
-            settingsOk = true;
-    }
+    connectionOk = headers.combinedValue(QHttpHeaders::WellKnownHeader::Connection).contains("Upgrade, HTTP2-Settings");
+    upgradeOk = headers.combinedValue(QHttpHeaders::WellKnownHeader::Upgrade).contains("h2c");
+    settingsOk = headers.contains("HTTP2-Settings");
 
     return connectionOk && upgradeOk && settingsOk;
 }
@@ -406,7 +415,7 @@ void Http2Server::triggerGOAWAYEmulation()
     auto timer = new QTimer(this);
     timer->setSingleShot(true);
     connect(timer, &QTimer::timeout, [this]() {
-        sendGOAWAY(quint32(connectionStreamID), quint32(INTERNAL_ERROR), 0);
+        sendGOAWAY(quint32(connectionStreamID), quint32(goawayCode), 0);
     });
     timer->start(goawayTimeout);
 }
@@ -669,6 +678,13 @@ void Http2Server::handleSETTINGS()
             return;
         }
 
+        // The client ACKed our setting, including the new decoder table size,
+        // so we can update it now:
+        if (pendingMaxTableSizeUpdate) {
+            decoder.setMaxDynamicTableSize(*pendingMaxTableSizeUpdate);
+            pendingMaxTableSizeUpdate.reset();
+        }
+
         waitingClientAck = false;
         emit serverSettingsAcked();
         return;
@@ -714,8 +730,12 @@ void Http2Server::handleDATA()
 
     const auto streamID = inboundFrame.streamID();
 
+    // We need to allow this in the `sendEarlyError` case because it mirrors how
+    // we are required to allow some incoming frames in a grace-period after
+    // sending the peer a RST frame. We don't care about the grace period
+    // though.
     if (!is_valid_client_stream(streamID) ||
-        closedStreams.find(streamID) != closedStreams.end()) {
+        (closedStreams.find(streamID) != closedStreams.end() && !sendEarlyError)) {
         emit invalidFrame();
         connectionError = true;
         sendGOAWAY(connectionStreamID, PROTOCOL_ERROR, connectionStreamID);
@@ -758,6 +778,14 @@ void Http2Server::handleDATA()
         sessionCurrRecvWindow += sessionRecvWindowSize / 2;
     }
 
+    if (sendEarlyError) {
+        if (activeRequests.find(streamID) != activeRequests.end()) {
+            responseBody = "not allowed";
+            sendResponse(streamID, false);
+        }
+        return;
+    }
+
     if (inboundFrame.flags().testFlag(FrameFlag::END_STREAM)) {
         if (responseBody.isEmpty()) {
             closedStreams.insert(streamID); // Enter "half-closed remote" state.
@@ -844,6 +872,25 @@ void Http2Server::sendResponse(quint32 streamID, bool emptyBody)
         // Now we'll continue with _normal_ response.
     }
 
+    // Create a header with an informational status code and some random header
+    // fields. The setter ensures that the value is 100 or is between 102 and 199
+    // (inclusive) if set - otherwise it is 0
+
+    if (informationalStatusCode > 0) {
+        writer.start(FrameType::HEADERS, FrameFlag::END_HEADERS, streamID);
+
+        HttpHeader informationalHeader;
+        informationalHeader.push_back({":status", QByteArray::number(informationalStatusCode)});
+        informationalHeader.push_back(HeaderField("a_random_header_field", "it_will_be_dropped"));
+        informationalHeader.push_back(HeaderField("another_random_header_field", "drop_this_too"));
+
+        HPack::BitOStream ostream(writer.outboundFrame().buffer);
+        const bool result = encoder.encodeResponse(ostream, informationalHeader);
+        Q_ASSERT(result);
+
+        writer.writeHEADERS(*socket, maxFrameSize);
+    }
+
     writer.start(FrameType::HEADERS, FrameFlag::END_HEADERS, streamID);
     if (emptyBody)
         writer.addFlag(FrameFlag::END_STREAM);
@@ -873,7 +920,10 @@ void Http2Server::sendResponse(quint32 streamID, bool emptyBody)
     } else if (!authenticationHeader.isEmpty() && !hasAuth) {
         header.push_back({ ":status", "401" });
         header.push_back(HPack::HeaderField("www-authenticate", authenticationHeader));
-        authenticationHeader.clear();
+    } else if (authenticationRequired) {
+        header.push_back({ ":status", "401" });
+    } else if (sendEarlyError) {
+        header.push_back({ ":status", "403" });
     } else {
         header.push_back({":status", "200"});
     }
