@@ -1,32 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QTest>
 #include <QtWidgets/qgraphicsanchorlayout.h>
 #include <private/qgraphicsanchorlayout_p.h>
 #include <QtWidgets/qgraphicswidget.h>
@@ -85,13 +60,13 @@ private slots:
 class RectWidget : public QGraphicsWidget
 {
 public:
-    RectWidget(QGraphicsItem *parent = 0) : QGraphicsWidget(parent){}
+    RectWidget(QGraphicsItem *parent = nullptr) : QGraphicsWidget(parent){}
 
-    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) override
     {
         Q_UNUSED(option);
         Q_UNUSED(widget);
-        painter->drawRoundRect(rect());
+        painter->drawRoundedRect(rect(), 25, 25, Qt::RelativeSize);
         painter->drawLine(rect().topLeft(), rect().bottomRight());
         painter->drawLine(rect().bottomLeft(), rect().topRight());
     }
@@ -159,11 +134,7 @@ static bool layoutHasConflict(QGraphicsAnchorLayout *l)
 
 static bool usedSimplex(QGraphicsAnchorLayout *l, Qt::Orientation o)
 {
-    QGraphicsAnchorLayoutPrivate::Orientation oo = (o == Qt::Horizontal) ?
-        QGraphicsAnchorLayoutPrivate::Horizontal :
-        QGraphicsAnchorLayoutPrivate::Vertical;
-
-    return QGraphicsAnchorLayoutPrivate::get(l)->lastCalculationUsedSimplex[oo];
+    return QGraphicsAnchorLayoutPrivate::get(l)->lastCalculationUsedSimplex[o];
 }
 
 void tst_QGraphicsAnchorLayout::simple()
@@ -1116,8 +1087,8 @@ public:
         vspacing = 10;
     }
 
-    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = 0,
-                            const QWidget * widget = 0 ) const;
+    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = nullptr,
+                            const QWidget * widget = nullptr) const override;
 
     int hspacing;
     int vspacing;
@@ -1125,8 +1096,8 @@ public:
     int layoutSpacing(QSizePolicy::ControlType control1,
                       QSizePolicy::ControlType control2,
                       Qt::Orientation orientation,
-                      const QStyleOption *option = 0,
-                      const QWidget *widget = 0) const;
+                      const QStyleOption *option = nullptr,
+                      const QWidget *widget = nullptr) const override;
 
 };
 
@@ -1136,8 +1107,8 @@ public:
 int CustomLayoutStyle::layoutSpacing(QSizePolicy::ControlType control1,
                                 QSizePolicy::ControlType control2,
                                 Qt::Orientation orientation,
-                                const QStyleOption * /*option = 0*/,
-                                const QWidget * /*widget = 0*/) const
+                                const QStyleOption * /*option = nullptr*/,
+                                const QWidget * /*widget = nullptr*/) const
 {
     if (orientation == Qt::Horizontal) {
         switch (CT2(control1, control2)) {
@@ -1157,8 +1128,8 @@ int CustomLayoutStyle::layoutSpacing(QSizePolicy::ControlType control1,
     }
 }
 
-int CustomLayoutStyle::pixelMetric(PixelMetric metric, const QStyleOption * option /*= 0*/,
-                                   const QWidget * widget /*= 0*/ ) const
+int CustomLayoutStyle::pixelMetric(PixelMetric metric, const QStyleOption * option /*= nullptr*/,
+                                   const QWidget * widget /*= nullptr*/ ) const
 {
     switch (metric) {
         case PM_LayoutLeftMargin:
@@ -1362,7 +1333,7 @@ static inline QByteArray msgStability(const QRectF &actual, const QRectF &expect
 
 void tst_QGraphicsAnchorLayout::stability()
 {
-    QVector<QRectF> geometries;
+    QList<QRectF> geometries;
     geometries.resize(7);
     QGraphicsWidget p(0, Qt::Window);
     // it usually fails after 3-4 iterations
